(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 12.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     36418,       1102]
NotebookOptionsPosition[     33220,       1042]
NotebookOutlinePosition[     33557,       1057]
CellTagsIndexPosition[     33514,       1054]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Trying", " ", "to", " ", "compute", " ", "differentials", " ", "of", " ",
      "r", " ", "needed", " ", "in", " ", "Drift"}], " ", "+", " ", 
    RowBox[{"Kepler", " ", 
     RowBox[{"problem", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Note", ":", "  ", 
     RowBox[{"below", "  ", "d"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"h", "+", 
         RowBox[{"eta", "*", "g2"}], "+", 
         RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}], 
      "\[IndentingNewLine]", "r"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
       RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
       RowBox[{"k", " ", "g2", "\[IndentingNewLine]", "h"}]}], " ", "=", " ", 
      
      RowBox[{
       RowBox[{
        RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
        RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
        RowBox[{"k", " ", "g3", "\[IndentingNewLine]", "g0"}]}], " ", "=", 
       " ", 
       RowBox[{
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2", " ", "\[IndentingNewLine]", "g3"}]}], " ",
         "=", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"s", "-", "g1"}], ")"}], "/", "beta"}], " ", 
           "\[IndentingNewLine]", "g0", " ", "g2"}], " ", "-", " ", 
          RowBox[{"g1", "^", "2"}]}], " ", "=", " ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"-", "g2"}], "\[IndentingNewLine]", 
            RowBox[{"g0", "^", "2"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"g1", "^", "2"}], " ", "beta"}]}], " ", "=", " ", 
          RowBox[{
           RowBox[{"1", "\[IndentingNewLine]", "zeta"}], " ", "=", " ", 
           RowBox[{
            RowBox[{"k", "-", 
             RowBox[{"beta", " ", "r0", "\[IndentingNewLine]", "h1"}]}], " ", 
            "=", " ", 
            RowBox[{
             RowBox[{"g2", "^", "2"}], " ", "-", " ", 
             RowBox[{"g1", " ", "g3"}]}]}]}]}]}]}]}]}]}]}], 
   "\[IndentingNewLine]", " ", "*)"}]}]], "Input",
 CellChangeTimes->{{3.776198140701968*^9, 3.776198320251252*^9}, {
  3.776202145270097*^9, 3.776202149610155*^9}, {3.776215247114793*^9, 
  3.776215248410046*^9}, {3.7764526048178797`*^9, 3.776452608298151*^9}, {
  3.7766111002427273`*^9, 3.776611150430565*^9}, {3.776615220729578*^9, 
  3.776615226632469*^9}},
 CellLabel->"In[1]:=",ExpressionUUID->"53329b6c-a80c-45ac-ae62-f6fb243392bd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"dgamma", " ", "=", " ", 
   RowBox[{
    FractionBox["sqb", "r"], " ", 
    RowBox[{"(", 
     RowBox[{"dh", " ", "+", "  ", 
      RowBox[{
       FractionBox["dk", "r0"], 
       RowBox[{"(", 
        RowBox[{"d", "-", " ", 
         RowBox[{"r0", " ", "g3"}]}], ")"}]}], "-", 
      RowBox[{"dx0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", "d"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], 
          FractionBox["x0", 
           SuperscriptBox["r0", "3"]]}], "+", 
         RowBox[{"g2", " ", "v0"}]}], ")"}]}], "-", 
      RowBox[{"dv0", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", "  ", "v0"}], " ", "+", " ", 
         RowBox[{"g2", " ", "x0"}]}], ")"}]}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.776198140701968*^9, 3.776198320251252*^9}, {
   3.776202145270097*^9, 3.776202149610155*^9}, {3.776215247114793*^9, 
   3.776215248410046*^9}, {3.7764526048178797`*^9, 3.776452608298151*^9}, {
   3.7766111002427273`*^9, 3.776611150430565*^9}, 3.7769476091218033`*^9, 
   3.7769479095508204`*^9},
 CellLabel->"In[2]:=",ExpressionUUID->"e5f914b2-2bf0-439b-99e7-b7d2a93687e8"],

Cell[BoxData[
 FractionBox[
  RowBox[{"sqb", " ", 
   RowBox[{"(", 
    RowBox[{"dh", "+", 
     FractionBox[
      RowBox[{"dk", " ", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}]}], "r0"], "-", 
     RowBox[{"dv0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"d", " ", "v0"}], "+", 
        RowBox[{"g2", " ", "x0"}]}], ")"}]}], "-", 
     RowBox[{"dx0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"g2", " ", "v0"}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}], "r"]], "Output",
 CellChangeTimes->{3.776202160614881*^9, 3.776611155025895*^9, 
  3.7769482494020042`*^9},
 CellLabel->"Out[2]=",ExpressionUUID->"a3fd5c9f-5c77-4b6e-b845-5e4890bc5da9"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Now", ",", " ", 
   RowBox[{
    RowBox[{"substitute", " ", "dx0"}], " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"dx0", " ", "-", " ", 
      RowBox[{"h", " ", "dv0"}], " ", "-", " ", 
      RowBox[{"dh", " ", "v0", " ", "and", " ", "x0"}]}], " ", "\[Rule]", " ", 
     RowBox[{"x0", " ", "-", " ", 
      RowBox[{"h", " ", 
       RowBox[{"v0", ":"}]}]}]}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.776969729006197*^9, 3.7769697446460447`*^9}, {
  3.776983356475065*^9, 
  3.776983363406624*^9}},ExpressionUUID->"aac02e55-c01b-476a-a15b-\
1b374277b81a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dgamma", " ", "=", " ", 
  FractionBox[
   RowBox[{"sqb", " ", 
    RowBox[{"(", 
     RowBox[{"dh", "+", 
      FractionBox[
       RowBox[{"dk", " ", 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"g3", " ", "r0"}]}], ")"}]}], "r0"], "-", 
      RowBox[{"dv0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "v0"}], "+", 
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{"x0", "-", 
            RowBox[{"h", " ", "v0"}]}], ")"}]}]}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"dx0", "-", 
         RowBox[{"h", " ", "dv0"}], " ", "-", " ", 
         RowBox[{"dh", " ", "v0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", "v0"}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{"x0", "-", 
             RowBox[{"h", " ", "v0"}]}], ")"}]}], 
          SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}], "r"]}]], "Input",
 CellChangeTimes->{{3.7769697502307053`*^9, 3.776969765045656*^9}, {
  3.776983383000235*^9, 3.776983392315576*^9}},
 CellLabel->"In[21]:=",ExpressionUUID->"85d0731c-0df0-4f79-932a-e71c71c9eac6"],

Cell[BoxData[
 FractionBox[
  RowBox[{"sqb", " ", 
   RowBox[{"(", 
    RowBox[{"dh", "+", 
     FractionBox[
      RowBox[{"dk", " ", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}]}], "r0"], "-", 
     RowBox[{"dv0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"d", " ", "v0"}], "+", 
        RowBox[{"g2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}]}], ")"}]}], 
     "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"dx0", "-", 
        RowBox[{"dv0", " ", "h"}], "-", 
        RowBox[{"dh", " ", "v0"}]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"g2", " ", "v0"}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}], "r"]], "Output",
 CellChangeTimes->{3.776969765766659*^9, 3.77698340428231*^9},
 CellLabel->"Out[21]=",ExpressionUUID->"7cbdee5b-091a-4a5a-90b2-84a5706022e7"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"dh", ",", "dk", ",", "dv0", ",", "dx0"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.776969783953364*^9, 3.776969819309062*^9}, 
   3.776983422298176*^9},
 CellLabel->"In[22]:=",ExpressionUUID->"7b731464-2480-44bb-bf7b-778117c540ea"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"dk", " ", 
    RowBox[{"(", 
     RowBox[{"d", "-", 
      RowBox[{"g3", " ", "r0"}]}], ")"}], " ", "sqb"}], 
   RowBox[{"r", " ", "r0"}]], "+", 
  FractionBox[
   RowBox[{"dx0", " ", "sqb", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "g2"}], " ", "v0"}], "-", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"d", " ", "k"}], "+", 
          RowBox[{"g1", " ", 
           SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
       SuperscriptBox["r0", "3"]]}], ")"}]}], "r"], "+", 
  FractionBox[
   RowBox[{"dv0", " ", "sqb", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "d"}], " ", "v0"}], "-", 
      RowBox[{"g2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], "+", 
      RowBox[{"h", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", "v0"}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
          SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}], "r"], "+", 
  FractionBox[
   RowBox[{"dh", " ", "sqb", " ", 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      RowBox[{"v0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", "v0"}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
          SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}], "r"]}]], "Output",\

 CellChangeTimes->{3.776969819610735*^9, 3.776983422970579*^9},
 CellLabel->"Out[22]=",ExpressionUUID->"facf74a3-7fad-48bb-a5ab-eab39a08bcc2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dgamma", " ", "=", 
  RowBox[{
   FractionBox["sqb", "r"], 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"dk", " ", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}]}], 
      RowBox[{" ", "r0"}]], "+", 
     RowBox[{"dx0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "g2"}], " ", "v0"}], "-", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
     RowBox[{"dv0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "d"}], " ", "v0"}], "-", 
        RowBox[{"g2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], "+", 
        RowBox[{"h", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"g2", " ", "v0"}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}], "+", 
     RowBox[{"dh", " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"g2", " ", 
           RowBox[{"(", 
            RowBox[{
             FractionBox[
              RowBox[{"2", "k"}], "r0"], "-", "beta"}], ")"}]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"d", " ", "k"}], "+", 
              RowBox[{"g1", " ", 
               SuperscriptBox["r0", "2"]}]}], ")"}], "eta"}], 
           SuperscriptBox["r0", "3"]]}], ")"}]}], ")"}]}]}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.776969826109988*^9, 3.7769698325349894`*^9}, 
   3.7769834449871807`*^9, {3.776990394511505*^9, 3.776990498616626*^9}},
 CellLabel->"In[42]:=",ExpressionUUID->"0f287db2-df2e-454d-88ff-eac4c6e10361"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "r"], 
  RowBox[{"sqb", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"dk", " ", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}]}], "r0"], "+", 
     RowBox[{"dh", " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"g2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "beta"}], "+", 
           FractionBox[
            RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
        FractionBox[
         RowBox[{"eta", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}]}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
     RowBox[{"dx0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "g2"}], " ", "v0"}], "-", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
     RowBox[{"dv0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "d"}], " ", "v0"}], "-", 
        RowBox[{"g2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], "+", 
        RowBox[{"h", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"g2", " ", "v0"}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.776983445897149*^9, 3.776990500802559*^9},
 CellLabel->"Out[42]=",ExpressionUUID->"04aa5a63-0599-4ec3-9ffd-852f4d544489"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"Collect", " ", "x0"}], " ", "&"}], " ", "v0", " ", "in", " ", 
   "dv0", " ", 
   RowBox[{"terms", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.7769698440423517`*^9, 3.776969852698691*^9}},
 CellLabel->"In[23]:=",ExpressionUUID->"2e21dc65-bf5a-4acd-a746-c45b7b0134e0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "d"}], " ", "v0"}], "-", 
     RowBox[{"g2", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], "+", 
     RowBox[{"h", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"g2", " ", "v0"}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}], ",", 
   RowBox[{"{", 
    RowBox[{"x0", ",", "v0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.77696985905929*^9, 3.77696986409094*^9}, 
   3.776983459115656*^9, 3.776990525027944*^9},
 CellLabel->"In[43]:=",ExpressionUUID->"43b1f5ac-cb5a-4005-a545-c92b39201790"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "d"}], "+", 
     RowBox[{"2", " ", "g2", " ", "h"}], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["h", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "k"}], "+", 
         RowBox[{"g1", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}]}], 
      SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "g2"}], "+", 
     FractionBox[
      RowBox[{"h", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "k"}], "+", 
         RowBox[{"g1", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}]}], 
      SuperscriptBox["r0", "3"]]}], ")"}], " ", "x0"}]}]], "Output",
 CellChangeTimes->{3.776969864362262*^9, 3.776983460414027*^9, 
  3.776990525738183*^9},
 CellLabel->"Out[43]=",ExpressionUUID->"5da69f1a-20d7-4140-a23d-c9ae0dd8f33f"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"Collect", " ", "x0"}], " ", "&"}], " ", "v0", " ", "in", " ", 
   "dx0", " ", 
   RowBox[{"terms", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.776969866953989*^9, 3.776969880855567*^9}},
 CellLabel->"In[25]:=",ExpressionUUID->"f299d9d0-5fdf-4f55-83f6-55fbbb7f2e89"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "g2"}], " ", "v0"}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "k"}], "+", 
         RowBox[{"g1", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}]}], 
      SuperscriptBox["r0", "3"]]}], ")"}], ",", 
   RowBox[{"{", 
    RowBox[{"x0", ",", "v0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.776969886319037*^9, 3.776969890224201*^9}, 
   3.776983472938987*^9, 3.7769905338512774`*^9},
 CellLabel->"In[44]:=",ExpressionUUID->"cf6eef9a-f3a6-43ea-bd71-5fd711312095"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", "g2"}], "+", 
     FractionBox[
      RowBox[{"h", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "k"}], "+", 
         RowBox[{"g1", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}]}], 
      SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "-", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"d", " ", "k"}], "+", 
      RowBox[{"g1", " ", 
       SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
   SuperscriptBox["r0", "3"]]}]], "Output",
 CellChangeTimes->{3.77696989052455*^9, 3.776983473603587*^9, 
  3.7769905345151987`*^9},
 CellLabel->"Out[44]=",ExpressionUUID->"31dce48a-b621-4862-9118-ebf5957bb409"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Complete", " ", 
   RowBox[{"expression", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.776969984763114*^9, 3.7769699914181623`*^9}},
 CellLabel->"In[28]:=",ExpressionUUID->"520163fc-ba76-4a12-89c4-0207a50c874c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dgamma", " ", "=", 
  RowBox[{
   FractionBox["sqb", "r"], 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"dk", " ", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}]}], 
      RowBox[{" ", "r0"}]], "+", 
     RowBox[{"dx0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "g2"}], "+", 
           FractionBox[
            RowBox[{"h", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "-", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
     RowBox[{"dv0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "d"}], "+", 
           RowBox[{"2", " ", "g2", " ", "h"}], "-", 
           FractionBox[
            RowBox[{
             SuperscriptBox["h", "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "g2"}], "+", 
           FractionBox[
            RowBox[{"h", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "x0"}]}], ")"}]}], "+", 
     
     RowBox[{"dh", " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"g2", " ", 
           RowBox[{"(", 
            RowBox[{
             FractionBox[
              RowBox[{"2", "k"}], "r0"], "-", "beta"}], ")"}]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"d", " ", "k"}], "+", 
              RowBox[{"g1", " ", 
               SuperscriptBox["r0", "2"]}]}], ")"}], "eta"}], 
           SuperscriptBox["r0", "3"]]}], ")"}]}], ")"}]}]}], 
    ")"}]}]}]], "Input",
 CellChangeTimes->{{3.776970010976091*^9, 3.776970087827468*^9}, {
   3.77698351238348*^9, 3.776983551751061*^9}, 3.776983585244289*^9, {
   3.776990550084008*^9, 3.776990574041844*^9}},
 CellLabel->"In[45]:=",ExpressionUUID->"43177c2f-0fef-4fdf-9f55-28fa68b8f18e"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "r"], 
  RowBox[{"sqb", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"dk", " ", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}]}], "r0"], "+", 
     RowBox[{"dh", " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"g2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "beta"}], "+", 
           FractionBox[
            RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
        FractionBox[
         RowBox[{"eta", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}]}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
     RowBox[{"dx0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "g2"}], "+", 
           FractionBox[
            RowBox[{"h", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "-", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
     RowBox[{"dv0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "d"}], "+", 
           RowBox[{"2", " ", "g2", " ", "h"}], "-", 
           FractionBox[
            RowBox[{
             SuperscriptBox["h", "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "g2"}], "+", 
           FractionBox[
            RowBox[{"h", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "x0"}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.776970091175267*^9, 3.776983586855551*^9, 
  3.7769905766996603`*^9},
 CellLabel->"Out[45]=",ExpressionUUID->"17eff0c2-03fc-4e13-bedb-5258aa03d9b5"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Simplifying", ":"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.776970101239655*^9, 3.7769701065997753`*^9}},
 CellLabel->"In[30]:=",ExpressionUUID->"c80ebed6-f7f8-4040-b266-ee325b9c4eb5"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{"c1", "=", 
  RowBox[{"d", "-", 
   RowBox[{"r0", " ", "g3"}]}]}], "\n", 
 RowBox[{"c2", "=", 
  RowBox[{
   RowBox[{"eta", " ", "g0"}], "+", 
   RowBox[{"g1", " ", "zeta"}]}]}], "\n", 
 RowBox[{"c3", "=", 
  RowBox[{
   RowBox[{"d", " ", "k"}], "+", 
   RowBox[{"g1", " ", 
    SuperscriptBox["r0", "2"]}]}]}], "\n"}], "Input",
 CellChangeTimes->{{3.7769836842451563`*^9, 3.7769837028935127`*^9}},
 CellLabel->"In[28]:=",ExpressionUUID->"1bec9e0a-6dbb-4843-975c-80ae19b69541"],

Cell[BoxData[
 RowBox[{"d", "-", 
  RowBox[{"g3", " ", "r0"}]}]], "Output",
 CellChangeTimes->{3.7769837045046387`*^9},
 CellLabel->"Out[28]=",ExpressionUUID->"f15af577-7e4c-4768-a27a-b1c734a8413d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"eta", " ", "g0"}], "+", 
  RowBox[{"g1", " ", "zeta"}]}]], "Output",
 CellChangeTimes->{3.776983704507234*^9},
 CellLabel->"Out[29]=",ExpressionUUID->"5ff50756-6c0e-4ae0-a5f9-52d54c41470c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", " ", "k"}], "+", 
  RowBox[{"g1", " ", 
   SuperscriptBox["r0", "2"]}]}]], "Output",
 CellChangeTimes->{3.776983704508918*^9},
 CellLabel->"Out[30]=",ExpressionUUID->"41b0a1ff-66eb-4649-92f3-5fdfb9358cb5"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"dgamma2", " ", "=", " ", 
   RowBox[{"dgamma", " ", "=", 
    RowBox[{
     FractionBox["sqb", "r"], 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"dk", " ", "c1"}], 
        RowBox[{" ", "r0"}]], "+", 
       RowBox[{"dx0", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "g2"}], "+", 
             FractionBox[
              RowBox[{"h", " ", "c3"}], 
              SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "-", 
          FractionBox[
           RowBox[{"c3", " ", "x0"}], 
           SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
       RowBox[{"dv0", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "d"}], "+", 
             RowBox[{"2", " ", "g2", " ", "h"}], "-", 
             FractionBox[
              RowBox[{
               SuperscriptBox["h", "2"], " ", "c3"}], 
              SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "g2"}], "+", 
             FractionBox[
              RowBox[{"h", " ", "c3"}], 
              SuperscriptBox["r0", "3"]]}], ")"}], " ", "x0"}]}], ")"}]}], 
       "+", 
       RowBox[{"dh", " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"g2", " ", 
           RowBox[{"(", 
            RowBox[{
             FractionBox[
              RowBox[{"2", "k"}], "r0"], "-", "beta"}], ")"}]}], "+", 
          FractionBox[
           RowBox[{"c3", " ", "eta"}], 
           SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.7769905875083427`*^9, 3.776990647148533*^9}, {
  3.776990765415567*^9, 
  3.776990767316766*^9}},ExpressionUUID->"703cb57c-9a58-47a0-ba89-\
c1e727c02cb3"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "r"], 
  RowBox[{"sqb", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"dk", " ", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}]}], "r0"], "+", 
     RowBox[{"dh", " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"g2", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "beta"}], "+", 
           FractionBox[
            RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
        FractionBox[
         RowBox[{"eta", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}]}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
     RowBox[{"dx0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "g2"}], "+", 
           FractionBox[
            RowBox[{"h", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "-", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}], "+", 
     RowBox[{"dv0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "d"}], "+", 
           RowBox[{"2", " ", "g2", " ", "h"}], "-", 
           FractionBox[
            RowBox[{
             SuperscriptBox["h", "2"], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "v0"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "g2"}], "+", 
           FractionBox[
            RowBox[{"h", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}]}], 
            SuperscriptBox["r0", "3"]]}], ")"}], " ", "x0"}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.7769906577966223`*^9},
 CellLabel->"Out[46]=",ExpressionUUID->"2fc553c5-68d0-43e1-bf2f-e9865f1c12dd"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dgamma", " ", "-", " ", "dgamma2"}]], "Input",
 CellChangeTimes->{{3.7769906604846563`*^9, 3.776990663531514*^9}},
 CellLabel->"In[47]:=",ExpressionUUID->"713086b8-b05b-4cb4-afa6-4e8063c6d482"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.776990664175296*^9},
 CellLabel->"Out[47]=",ExpressionUUID->"dbfe41df-0cae-41a6-b97c-5cf9b330d569"]
}, Open  ]]
},
WindowSize->{1160, 507},
WindowMargins->{{19, Automatic}, {Automatic, 3}},
FrontEndVersion->"12.0 for Mac OS X x86 (64-bit) (April 8, 2019)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 2647, 67, 241, "Input",ExpressionUUID->"53329b6c-a80c-45ac-ae62-f6fb243392bd"],
Cell[CellGroupData[{
Cell[3230, 91, 1300, 34, 72, "Input",ExpressionUUID->"e5f914b2-2bf0-439b-99e7-b7d2a93687e8"],
Cell[4533, 127, 968, 29, 68, "Output",ExpressionUUID->"a3fd5c9f-5c77-4b6e-b845-5e4890bc5da9"]
}, Open  ]],
Cell[5516, 159, 612, 15, 30, "Input",ExpressionUUID->"aac02e55-c01b-476a-a15b-1b374277b81a"],
Cell[CellGroupData[{
Cell[6153, 178, 1395, 40, 70, "Input",ExpressionUUID->"85d0731c-0df0-4f79-932a-e71c71c9eac6"],
Cell[7551, 220, 1322, 41, 68, "Output",ExpressionUUID->"7cbdee5b-091a-4a5a-90b2-84a5706022e7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8910, 266, 332, 8, 30, "Input",ExpressionUUID->"7b731464-2480-44bb-bf7b-778117c540ea"],
Cell[9245, 276, 2342, 75, 130, "Output",ExpressionUUID->"facf74a3-7fad-48bb-a5ab-eab39a08bcc2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11624, 356, 2496, 76, 130, "Input",ExpressionUUID->"0f287db2-df2e-454d-88ff-eac4c6e10361"],
Cell[14123, 434, 2348, 73, 110, "Output",ExpressionUUID->"04aa5a63-0599-4ec3-9ffd-852f4d544489"]
}, Open  ]],
Cell[16486, 510, 348, 8, 30, "Input",ExpressionUUID->"2e21dc65-bf5a-4acd-a746-c45b7b0134e0"],
Cell[CellGroupData[{
Cell[16859, 522, 1065, 32, 56, "Input",ExpressionUUID->"43b1f5ac-cb5a-4005-a545-c92b39201790"],
Cell[17927, 556, 945, 30, 60, "Output",ExpressionUUID->"5da69f1a-20d7-4140-a23d-c9ae0dd8f33f"]
}, Open  ]],
Cell[18887, 589, 346, 8, 30, "Input",ExpressionUUID->"f299d9d0-5fdf-4f55-83f6-55fbbb7f2e89"],
Cell[CellGroupData[{
Cell[19258, 601, 774, 23, 56, "Input",ExpressionUUID->"cf6eef9a-f3a6-43ea-bd71-5fd711312095"],
Cell[20035, 626, 753, 24, 60, "Output",ExpressionUUID->"31dce48a-b621-4862-9118-ebf5957bb409"]
}, Open  ]],
Cell[20803, 653, 266, 5, 30, "Input",ExpressionUUID->"520163fc-ba76-4a12-89c4-0207a50c874c"],
Cell[CellGroupData[{
Cell[21094, 662, 2886, 87, 130, "Input",ExpressionUUID->"43177c2f-0fef-4fdf-9f55-28fa68b8f18e"],
Cell[23983, 751, 2714, 83, 110, "Output",ExpressionUUID->"17eff0c2-03fc-4e13-bedb-5258aa03d9b5"]
}, Open  ]],
Cell[26712, 837, 236, 4, 30, "Input",ExpressionUUID->"c80ebed6-f7f8-4040-b266-ee325b9c4eb5"],
Cell[CellGroupData[{
Cell[26973, 845, 501, 14, 94, "Input",ExpressionUUID->"1bec9e0a-6dbb-4843-975c-80ae19b69541"],
Cell[27477, 861, 198, 4, 34, "Output",ExpressionUUID->"f15af577-7e4c-4768-a27a-b1c734a8413d"],
Cell[27678, 867, 224, 5, 34, "Output",ExpressionUUID->"5ff50756-6c0e-4ae0-a5f9-52d54c41470c"],
Cell[27905, 874, 244, 6, 34, "Output",ExpressionUUID->"41b0a1ff-66eb-4649-92f3-5fdfb9358cb5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[28186, 885, 1942, 58, 76, "Input",ExpressionUUID->"703cb57c-9a58-47a0-ba89-c1e727c02cb3"],
Cell[30131, 945, 2667, 82, 110, "Output",ExpressionUUID->"2fc553c5-68d0-43e1-bf2f-e9865f1c12dd"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32835, 1032, 217, 3, 30, "Input",ExpressionUUID->"713086b8-b05b-4cb4-afa6-4e8063c6d482"],
Cell[33055, 1037, 149, 2, 34, "Output",ExpressionUUID->"dbfe41df-0cae-41a6-b97c-5cf9b330d569"]
}, Open  ]]
}
]
*)

