(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 12.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    257184,       7405]
NotebookOptionsPosition[    241662,       7151]
NotebookOutlinePosition[    241999,       7166]
CellTagsIndexPosition[    241956,       7163]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Trying", " ", "to", " ", "compute", " ", "differentials", " ", "of", " ",
      "r", " ", "needed", " ", "in", " ", "Drift"}], " ", "+", " ", 
    RowBox[{"Kepler", " ", 
     RowBox[{"problem", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"Note", ":", "  ", 
     RowBox[{"below", "  ", "d"}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"h", "+", 
         RowBox[{"eta", "*", "g2"}], "+", 
         RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}], 
      "\[IndentingNewLine]", "r"}], " ", "=", " ", 
     RowBox[{
      RowBox[{
       RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
       RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
       RowBox[{"k", " ", "g2", "\[IndentingNewLine]", "h"}]}], " ", "=", " ", 
      
      RowBox[{
       RowBox[{
        RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
        RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
        RowBox[{"k", " ", "g3", "\[IndentingNewLine]", "g0"}]}], " ", "=", 
       " ", 
       RowBox[{
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2", " ", "\[IndentingNewLine]", "g3"}]}], " ",
         "=", " ", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{"s", "-", "g1"}], ")"}], "/", "beta"}], " ", 
           "\[IndentingNewLine]", "g0", " ", "g2"}], " ", "-", " ", 
          RowBox[{"g1", "^", "2"}]}], " ", "=", " ", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"-", "g2"}], "\[IndentingNewLine]", 
            RowBox[{"g0", "^", "2"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"g1", "^", "2"}], " ", "beta"}]}], " ", "=", " ", 
          RowBox[{
           RowBox[{"1", "\[IndentingNewLine]", "zeta"}], " ", "=", " ", 
           RowBox[{
            RowBox[{"k", "-", 
             RowBox[{"beta", " ", "r0", "\[IndentingNewLine]", "h1"}]}], " ", 
            "=", " ", 
            RowBox[{
             RowBox[{"g2", "^", "2"}], " ", "-", " ", 
             RowBox[{"g1", " ", "g3"}]}]}]}]}]}]}]}]}]}]}], 
   "\[IndentingNewLine]", " ", "*)"}]}]], "Input",
 CellChangeTimes->{{3.776198140701968*^9, 3.776198320251252*^9}, {
  3.776202145270097*^9, 3.776202149610155*^9}, {3.776215247114793*^9, 
  3.776215248410046*^9}, {3.7764526048178797`*^9, 3.776452608298151*^9}, {
  3.7766111002427273`*^9, 3.776611150430565*^9}, {3.776615220729578*^9, 
  3.776615226632469*^9}},
 CellLabel->"In[1]:=",ExpressionUUID->"53329b6c-a80c-45ac-ae62-f6fb243392bd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"dgamma", " ", "=", " ", 
   RowBox[{
    FractionBox["sqb", "r"], " ", 
    RowBox[{"(", 
     RowBox[{"dh", " ", "+", "  ", 
      RowBox[{
       FractionBox["dk", "r0"], 
       RowBox[{"(", 
        RowBox[{"d", "-", " ", 
         RowBox[{"r0", " ", "g3"}]}], ")"}]}], "-", 
      RowBox[{"dx0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", "d"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], 
          FractionBox["x0", 
           SuperscriptBox["r0", "3"]]}], "+", 
         RowBox[{"g2", " ", "v0"}]}], ")"}]}], "-", 
      RowBox[{"dv0", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", "  ", "v0"}], " ", "+", " ", 
         RowBox[{"g2", " ", "x0"}]}], ")"}]}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.776198140701968*^9, 3.776198320251252*^9}, {
   3.776202145270097*^9, 3.776202149610155*^9}, {3.776215247114793*^9, 
   3.776215248410046*^9}, {3.7764526048178797`*^9, 3.776452608298151*^9}, {
   3.7766111002427273`*^9, 3.776611150430565*^9}, 3.7769476091218033`*^9, 
   3.7769479095508204`*^9},
 CellLabel->"In[1]:=",ExpressionUUID->"e5f914b2-2bf0-439b-99e7-b7d2a93687e8"],

Cell[BoxData[
 FractionBox[
  RowBox[{"sqb", " ", 
   RowBox[{"(", 
    RowBox[{"dh", "+", 
     FractionBox[
      RowBox[{"dk", " ", 
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}]}], "r0"], "-", 
     RowBox[{"dv0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"d", " ", "v0"}], "+", 
        RowBox[{"g2", " ", "x0"}]}], ")"}]}], "-", 
     RowBox[{"dx0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"g2", " ", "v0"}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
         SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}]}], "r"]], "Output",
 CellChangeTimes->{3.776202160614881*^9, 3.776611155025895*^9, 
  3.7769482494020042`*^9, 3.776984129467828*^9, 3.776990883110292*^9, 
  3.777692950386231*^9},
 CellLabel->"Out[1]=",ExpressionUUID->"4955c78b-2203-47ea-9a21-650a64b5616b"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Corrected", " ", "expression", " ", "for", " ", 
   RowBox[{"dr", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.77698401640345*^9, 3.776984022690288*^9}},
 CellLabel->"In[31]:=",ExpressionUUID->"3d3905a2-a8e5-4aff-90a6-57805de85496"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dr", " ", "=", " ", 
  RowBox[{
   RowBox[{
    FractionBox["dgamma", "sqb"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"zeta", " ", "g1"}], "+", 
      RowBox[{"eta", " ", "g0"}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["dk", 
     RowBox[{"beta", " ", "r0", " "}]], 
    RowBox[{"(", 
     RowBox[{"r0", "-", "r", "-", 
      RowBox[{"k", " ", "g2"}]}], ")"}]}], "+", 
   RowBox[{"dx0", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["1", 
        RowBox[{"beta", " ", 
         SuperscriptBox["r0", "3"]}]], 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", 
          RowBox[{"(", 
           RowBox[{"r", "+", 
            RowBox[{"k", " ", "g2"}]}], ")"}]}], "-", 
         RowBox[{"zeta", " ", "r0", " ", "g0"}]}], ")"}], "x0"}], " ", "+", 
      " ", 
      RowBox[{"g1", "  ", "v0"}]}], ")"}]}], " ", "+", " ", 
   RowBox[{"dv0", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g1"}], "+", 
         RowBox[{"2", " ", "k", " ", "g2"}]}], ")"}], " ", 
       FractionBox["v0", "beta"]}], " ", "+", " ", 
      RowBox[{"g1", " ", "x0"}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.776198428651246*^9, 3.7761985237438927`*^9}, {
   3.776610948519197*^9, 3.7766109842635393`*^9}, {3.776948058948588*^9, 
   3.776948132858786*^9}, 3.7769840143511972`*^9, {3.776989740194901*^9, 
   3.776989740296968*^9}},
 CellLabel->"In[2]:=",ExpressionUUID->"ec64950c-8ed3-4bc5-92a5-293069313a0b"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"dk", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], ")"}]}], 
   RowBox[{"beta", " ", "r0"}]], "+", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g1"}], "+", 
         RowBox[{"2", " ", "g2", " ", "k"}]}], ")"}], " ", "v0"}], "beta"], 
     "+", 
     RowBox[{"g1", " ", "x0"}]}], ")"}]}], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"dh", "+", 
      FractionBox[
       RowBox[{"dk", " ", 
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"g3", " ", "r0"}]}], ")"}]}], "r0"], "-", 
      RowBox[{"dv0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "v0"}], "+", 
         RowBox[{"g2", " ", "x0"}]}], ")"}]}], "-", 
      RowBox[{"dx0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", "v0"}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
          SuperscriptBox["r0", "3"]]}], ")"}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g0"}], "+", 
      RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
  RowBox[{"dx0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g1", " ", "v0"}], "+", 
     FractionBox[
      RowBox[{"x0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]]}], ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.776202167353901*^9, 3.776611158988332*^9, {3.7769481350080137`*^9, 
   3.776948156282922*^9}, 3.776948253329575*^9, 3.776984025948564*^9, 
   3.776984135030624*^9, 3.776990887170259*^9, 3.777692950536005*^9},
 CellLabel->"Out[2]=",ExpressionUUID->"fc05a247-3d57-4595-b5df-ae9e6d744f8c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"dx0", ",", "dv0", ",", "dk", ",", "dh"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.776611166372304*^9, 3.776611177466592*^9}, {
  3.77661154142327*^9, 3.776611543580521*^9}, {3.776948219270524*^9, 
  3.776948256683364*^9}},
 CellLabel->"In[3]:=",ExpressionUUID->"c4b0219d-ef0f-4a85-945c-de2fde90772c"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"dh", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g0"}], "+", 
      RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
  RowBox[{"dk", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
      RowBox[{"beta", " ", "r0"}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g1"}], "+", 
         RowBox[{"2", " ", "g2", " ", "k"}]}], ")"}], " ", "v0"}], "beta"], 
     "+", 
     RowBox[{"g1", " ", "x0"}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "v0"}], "+", 
         RowBox[{"g2", " ", "x0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
  RowBox[{"dx0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g1", " ", "v0"}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", "v0"}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
          SuperscriptBox["r0", "3"]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
     FractionBox[
      RowBox[{"x0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.776611177789357*^9, 3.776611544116699*^9, 
  3.7769481586091948`*^9, 3.776948220081491*^9, 3.776948258422385*^9, 
  3.776984032326845*^9, 3.776984138361259*^9, 3.776990892504528*^9},
 CellLabel->"Out[3]=",ExpressionUUID->"fc76868f-18c3-4155-85d7-ea4774492c50"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Collect", " ", "dx0", " ", 
   RowBox[{"term", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.77769298796614*^9, 3.777693006251122*^9}},
 CellLabel->"In[5]:=",ExpressionUUID->"47b9fc33-5d07-4434-a043-7f9a73672425"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g1", " ", "v0"}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", "v0"}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
          SuperscriptBox["r0", "3"]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
     FractionBox[
      RowBox[{"x0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]]}], ")"}], ",", 
   RowBox[{"{", 
    RowBox[{"x0", ",", "v0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.77769300811644*^9, 3.777693018267169*^9}},
 CellLabel->"In[6]:=",ExpressionUUID->"2c452f06-9851-4112-9a70-63932153ef0e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"v0", " ", 
   RowBox[{"(", 
    RowBox[{"g1", "-", 
     FractionBox[
      RowBox[{"g2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
  RowBox[{"x0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"d", " ", "k"}], "+", 
          RowBox[{"g1", " ", 
           SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"eta", " ", "g0"}], "+", 
          RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
       RowBox[{"r", " ", 
        SuperscriptBox["r0", "3"]}]]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"k", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
       RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.77769301865767*^9},
 CellLabel->"Out[6]=",ExpressionUUID->"4bb8a2af-287c-48fb-bb5b-881170e64fdc"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Collect", " ", "dv0", " ", 
   RowBox[{"term", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.777693020076057*^9, 3.777693027154389*^9}},
 CellLabel->"In[7]:=",ExpressionUUID->"c9e66de5-4f55-4c4c-9961-82f75df0e5e8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g1"}], "+", 
         RowBox[{"2", " ", "g2", " ", "k"}]}], ")"}], " ", "v0"}], "beta"], 
     "+", 
     RowBox[{"g1", " ", "x0"}], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "v0"}], "+", 
         RowBox[{"g2", " ", "x0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}], ",", 
   RowBox[{"{", 
    RowBox[{"x0", ",", "v0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.777693028307803*^9, 3.777693039034711*^9}},
 CellLabel->"In[8]:=",ExpressionUUID->"68f55207-a636-4c7c-a40a-237f2da6c8d5"],

Cell[BoxData[
 RowBox[{
  RowBox[{"v0", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g1"}], "+", 
       RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
     FractionBox[
      RowBox[{"d", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
  RowBox[{"x0", " ", 
   RowBox[{"(", 
    RowBox[{"g1", "-", 
     FractionBox[
      RowBox[{"g2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}]}]], "Output",\

 CellChangeTimes->{3.77769303934191*^9},
 CellLabel->"Out[8]=",ExpressionUUID->"b0ac71be-dfd7-4696-8b66-e735fc2bbc66"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"Substitute", " ", "dx0"}], " ", "\[Rule]", " ", 
     RowBox[{
      RowBox[{"dx0", " ", "-", " ", 
       RowBox[{"h", "*", "dv0"}], " ", "-", " ", 
       RowBox[{"dh", " ", "*", " ", "v0", " ", "and", " ", "x0"}]}], " ", 
      "\[Rule]", " ", 
      RowBox[{"x0", "-", 
       RowBox[{"h", "*", "v0"}]}]}]}], ",", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"and", " ", 
         RowBox[{"v0", "^", "2"}]}], " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"2", 
          RowBox[{"k", "/", "r0"}]}], "-", "beta"}]}], " ", "&"}], " ", 
      RowBox[{"x0", ".", "v0"}]}], " ", "\[Rule]", " ", 
     RowBox[{"eta", ":"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"dr", " ", "=", 
   RowBox[{
    FractionBox[
     RowBox[{"dh", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"eta", " ", "g0"}], "+", 
        RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
    RowBox[{"dk", " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{
          RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
        RowBox[{"beta", " ", "r0"}]], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", "-", 
           RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"eta", " ", "g0"}], "+", 
           RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
        RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
    RowBox[{"dv0", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"v0", " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{"eta", " ", "g1"}], "+", 
            RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
          FractionBox[
           RowBox[{"d", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x0", "-", 
          RowBox[{"h", " ", "v0"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g1", "-", 
          FractionBox[
           RowBox[{"g2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}]}], 
      ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"dx0", " ", "-", " ", 
       RowBox[{"h", " ", "dv0"}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"v0", " ", 
        RowBox[{"(", 
         RowBox[{"g1", "-", 
          FractionBox[
           RowBox[{"g2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x0", "-", 
          RowBox[{"h", " ", "v0"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"eta", " ", "g0"}], "+", 
               RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
            RowBox[{"r", " ", 
             SuperscriptBox["r0", "3"]}]]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
           RowBox[{"beta", " ", 
            SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "-", 
    RowBox[{"dh", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"2", "k"}], "r0"], "-", "beta"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g1", "-", 
          FractionBox[
           RowBox[{"g2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
       RowBox[{"eta", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"eta", " ", "g0"}], "+", 
               RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
            RowBox[{"r", " ", 
             SuperscriptBox["r0", "3"]}]]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
           RowBox[{"beta", " ", 
            SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}]}]}]], "Input",\

 CellChangeTimes->{{3.776611196479528*^9, 3.7766112141683817`*^9}, {
   3.776897754400957*^9, 3.776897789240497*^9}, {3.77694816533681*^9, 
   3.776948181950849*^9}, {3.77694826808764*^9, 3.776948280595683*^9}, {
   3.776984037402067*^9, 3.776984081135673*^9}, {3.776984146477818*^9, 
   3.776984234315776*^9}, 3.7769882316151867`*^9, {3.7769882903098593`*^9, 
   3.776988458896253*^9}, {3.776991199938158*^9, 3.776991203155594*^9}, {
   3.776995825316324*^9, 3.776995827807672*^9}, 3.776995902604393*^9, 
   3.777692967874095*^9, {3.777693049885871*^9, 3.7776930968738623`*^9}},
 CellLabel->"In[9]:=",ExpressionUUID->"04b71007-a2aa-46f4-a938-b8fcee4ceb50"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"dh", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g0"}], "+", 
      RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
  RowBox[{"dk", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
      RowBox[{"beta", " ", "r0"}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"eta", " ", "g1"}], "+", 
          RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
        FractionBox[
         RowBox[{"d", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}]}], 
    ")"}]}], "-", 
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
     RowBox[{"eta", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"dx0", "-", 
     RowBox[{"dv0", " ", "h"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.776897789756968*^9, 3.7769481825231028`*^9, 3.776948281276082*^9, {
   3.776984218719161*^9, 3.776984235195571*^9}, 3.776988479669457*^9, 
   3.7769909068601522`*^9, 3.776991206585088*^9, 3.7769958428140907`*^9, 
   3.7769959038374157`*^9, 3.777693097667494*^9},
 CellLabel->"Out[9]=",ExpressionUUID->"6f0e28ed-9bc4-4425-bc09-07b4d1c655e1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"dx0", ",", "dv0", ",", "dk", ",", "dh"}], "}"}]}], 
  "]"}]], "Input",
 CellChangeTimes->{{3.7768977906562967`*^9, 3.776897798473324*^9}},
 CellLabel->"In[10]:=",ExpressionUUID->"fec8e82c-efd6-4867-a823-49d5929c062a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dk", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
      RowBox[{"beta", " ", "r0"}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], "r"], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "-", 
     RowBox[{"eta", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"dx0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"eta", " ", "g1"}], "+", 
          RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
        FractionBox[
         RowBox[{"d", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "-", 
     RowBox[{"h", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"v0", " ", 
         RowBox[{"(", 
          RowBox[{"g1", "-", 
           FractionBox[
            RowBox[{"g2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"eta", " ", "g0"}], "+", 
               RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"d", " ", "k"}], "+", 
                RowBox[{"g1", " ", 
                 SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"eta", " ", "g0"}], "+", 
                RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
             RowBox[{"r", " ", 
              SuperscriptBox["r0", "3"]}]]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
            RowBox[{"beta", " ", 
             SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{
  3.776897799033997*^9, 3.776948184810842*^9, 3.7769482848895073`*^9, {
   3.776984221695921*^9, 3.776984237614769*^9}, 3.776988483198073*^9, 
   3.776990909459796*^9, 3.776991212243175*^9, 3.776995848842078*^9, 
   3.7769959063099337`*^9, 3.777693100431069*^9},
 CellLabel->"Out[10]=",ExpressionUUID->"f214e0f6-85fc-456d-bba1-9cdbe3c090bf"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Collect", " ", 
   RowBox[{"{", 
    RowBox[{"x0", ",", "v0"}], "}"}], " ", "in", " ", "dv0", " ", 
   RowBox[{"term", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.776897955915698*^9, 
  3.7768979646187153`*^9}},ExpressionUUID->"9fa6d673-9623-427b-9407-\
78624850703f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"eta", " ", "g1"}], "+", 
          RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
        FractionBox[
         RowBox[{"d", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "-", 
     RowBox[{"h", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"v0", " ", 
         RowBox[{"(", 
          RowBox[{"g1", "-", 
           FractionBox[
            RowBox[{"g2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"eta", " ", "g0"}], "+", 
               RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"d", " ", "k"}], "+", 
                RowBox[{"g1", " ", 
                 SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"eta", " ", "g0"}], "+", 
                RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
             RowBox[{"r", " ", 
              SuperscriptBox["r0", "3"]}]]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
            RowBox[{"beta", " ", 
             SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], ")"}], ",", 
   RowBox[{"{", 
    RowBox[{"x0", ",", "v0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.776897939748164*^9, 3.77689794718049*^9}, 
   3.776948344418603*^9, 3.776984246400463*^9, 3.776988493675294*^9, 
   3.776991222113275*^9, 3.776995861833914*^9, 3.7769959143430767`*^9, 
   3.7776931137722187`*^9},
 CellLabel->"In[11]:=",ExpressionUUID->"c9d2383d-a036-4f78-b220-89a9454bf72a"],

Cell[BoxData[
 RowBox[{
  RowBox[{"x0", " ", 
   RowBox[{"(", 
    RowBox[{"g1", "-", 
     FractionBox[
      RowBox[{"g2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
     RowBox[{"h", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"v0", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g1"}], "+", 
       RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
     FractionBox[
      RowBox[{"d", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
     RowBox[{"2", " ", "h", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["h", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.776897947503841*^9, 3.776948346168757*^9, 
  3.7769842472973337`*^9, 3.776988494367036*^9, 3.776990913913703*^9, 
  3.776991223329343*^9, 3.77699586269093*^9, 3.776995914996966*^9, 
  3.777693114496805*^9},
 CellLabel->"Out[11]=",ExpressionUUID->"ff30ac28-e574-47ce-8999-03449efe1c2d"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Collect", " ", 
   RowBox[{"{", 
    RowBox[{"x0", ",", "v0"}], "}"}], " ", "in", " ", "dx0", " ", 
   RowBox[{"term", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.776897972785655*^9, 
  3.776897972857336*^9}},ExpressionUUID->"23904c48-5c7f-4f38-a10d-\
1dffce1fbd7f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "h"}], " ", "v0"}], "+", "x0"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}], ",", 
   RowBox[{"{", 
    RowBox[{"x0", ",", "v0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.776897980345562*^9, 3.776897984539198*^9}, 
   3.7769483585213737`*^9, 3.776984263423216*^9, 3.7769885033212013`*^9, 
   3.776991237639798*^9, 3.777693125171224*^9},
 CellLabel->"In[12]:=",ExpressionUUID->"66f595b3-9a00-4612-95df-a4919c2ed717"],

Cell[BoxData[
 RowBox[{
  RowBox[{"x0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"d", " ", "k"}], "+", 
          RowBox[{"g1", " ", 
           SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"eta", " ", "g0"}], "+", 
          RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
       RowBox[{"r", " ", 
        SuperscriptBox["r0", "3"]}]]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"k", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
       RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
  RowBox[{"v0", " ", 
   RowBox[{"(", 
    RowBox[{"g1", "-", 
     FractionBox[
      RowBox[{"g2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
     RowBox[{"h", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.7768979848680677`*^9, 3.7769483594231367`*^9, 
  3.7769842641536818`*^9, 3.776988503975362*^9, 3.776990916533379*^9, 
  3.7769912382990313`*^9, 3.77769312587649*^9},
 CellLabel->"Out[12]=",ExpressionUUID->"e1020760-5a71-4965-84e5-11e7c2c10310"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Semi", "-", 
   RowBox[{"final", " ", 
    RowBox[{"version", ":"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.776898155795938*^9, 3.776898159167433*^9}, {
  3.776949105875556*^9, 
  3.776949107029307*^9}},ExpressionUUID->"f7d26cbe-f478-4850-8b72-\
772aeff8061f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"dr3", " ", "=", " ", 
   RowBox[{
    RowBox[{"dk", " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{
          RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
        RowBox[{"beta", " ", "r0"}]], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{"d", "-", 
           RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"eta", " ", "g0"}], "+", 
           RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
        RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
    RowBox[{"dh", " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], "r"], "-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "beta"}], "+", 
          FractionBox[
           RowBox[{"2", " ", "k"}], "r0"]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{"g1", "-", 
          FractionBox[
           RowBox[{"g2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "-", 
       RowBox[{"eta", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"eta", " ", "g0"}], "+", 
               RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
            RowBox[{"r", " ", 
             SuperscriptBox["r0", "3"]}]]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
           RowBox[{"beta", " ", 
            SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
    RowBox[{"dx0", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"x0", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox[
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"d", " ", "k"}], "+", 
               RowBox[{"g1", " ", 
                SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"eta", " ", "g0"}], "+", 
               RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
            RowBox[{"r", " ", 
             SuperscriptBox["r0", "3"]}]]}], "+", 
          FractionBox[
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
           RowBox[{"beta", " ", 
            SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
       RowBox[{"v0", " ", 
        RowBox[{"(", 
         RowBox[{"g1", "-", 
          FractionBox[
           RowBox[{"g2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
          RowBox[{"h", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              FractionBox[
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"d", " ", "k"}], "+", 
                  RowBox[{"g1", " ", 
                   SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"eta", " ", "g0"}], "+", 
                  RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
               RowBox[{"r", " ", 
                SuperscriptBox["r0", "3"]}]]}], "+", 
             FractionBox[
              RowBox[{
               RowBox[{"k", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
               RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
              RowBox[{"beta", " ", 
               SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], ")"}]}], 
    "+", 
    RowBox[{"dv0", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"x0", " ", 
        RowBox[{"(", 
         RowBox[{"g1", "-", 
          FractionBox[
           RowBox[{"g2", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
          RowBox[{"h", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              FractionBox[
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"d", " ", "k"}], "+", 
                  RowBox[{"g1", " ", 
                   SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"eta", " ", "g0"}], "+", 
                  RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
               RowBox[{"r", " ", 
                SuperscriptBox["r0", "3"]}]]}], "+", 
             FractionBox[
              RowBox[{
               RowBox[{"k", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
               RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
              RowBox[{"beta", " ", 
               SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
       RowBox[{"v0", " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{
            RowBox[{"eta", " ", "g1"}], "+", 
            RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
          FractionBox[
           RowBox[{"d", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
          RowBox[{"2", " ", "h", " ", 
           RowBox[{"(", 
            RowBox[{"g1", "-", 
             FractionBox[
              RowBox[{"g2", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"eta", " ", "g0"}], "+", 
                 RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
          RowBox[{
           SuperscriptBox["h", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              FractionBox[
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"d", " ", "k"}], "+", 
                  RowBox[{"g1", " ", 
                   SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"eta", " ", "g0"}], "+", 
                  RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
               RowBox[{"r", " ", 
                SuperscriptBox["r0", "3"]}]]}], "+", 
             FractionBox[
              RowBox[{
               RowBox[{"k", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
               RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
              RowBox[{"beta", " ", 
               SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], 
      ")"}]}]}]}], "\[IndentingNewLine]", "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.777693137254795*^9, 3.777693166154434*^9}},
 CellLabel->"In[13]:=",ExpressionUUID->"f882e2b1-47a9-4619-b178-8b87b1331dda"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dk", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
      RowBox[{"beta", " ", "r0"}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], "r"], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "-", 
     RowBox[{"eta", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"dx0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        RowBox[{"h", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"d", " ", "k"}], "+", 
                RowBox[{"g1", " ", 
                 SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"eta", " ", "g0"}], "+", 
                RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
             RowBox[{"r", " ", 
              SuperscriptBox["r0", "3"]}]]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
            RowBox[{"beta", " ", 
             SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], ")"}]}], 
  "+", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        RowBox[{"h", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"d", " ", "k"}], "+", 
                RowBox[{"g1", " ", 
                 SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"eta", " ", "g0"}], "+", 
                RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
             RowBox[{"r", " ", 
              SuperscriptBox["r0", "3"]}]]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
            RowBox[{"beta", " ", 
             SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"eta", " ", "g1"}], "+", 
          RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
        FractionBox[
         RowBox[{"d", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        RowBox[{"2", " ", "h", " ", 
         RowBox[{"(", 
          RowBox[{"g1", "-", 
           FractionBox[
            RowBox[{"g2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"eta", " ", "g0"}], "+", 
               RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["h", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"d", " ", "k"}], "+", 
                RowBox[{"g1", " ", 
                 SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"eta", " ", "g0"}], "+", 
                RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
             RowBox[{"r", " ", 
              SuperscriptBox["r0", "3"]}]]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
            RowBox[{"beta", " ", 
             SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.777693167700736*^9},
 CellLabel->"Out[13]=",ExpressionUUID->"bd780f74-c409-447b-b977-8bccc8641338"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dr", " ", "=", 
  RowBox[{
   RowBox[{"dk", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
       RowBox[{"beta", " ", "r0"}]], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"d", "-", 
          RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"eta", " ", "g0"}], "+", 
          RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
       RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
   RowBox[{"dh", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "g1"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "beta"}], "+", 
         FractionBox[
          RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"eta", " ", "g0"}], "+", 
        RowBox[{"g1", " ", "zeta"}]}], "r"], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"g2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "beta"}], "+", 
             FractionBox[
              RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
          FractionBox[
           RowBox[{"eta", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"d", " ", "k"}], "+", 
              RowBox[{"g1", " ", 
               SuperscriptBox["r0", "2"]}]}], ")"}]}], 
           SuperscriptBox["r0", "3"]]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"eta", " ", "g0"}], "+", 
          RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
      FractionBox[
       RowBox[{"eta", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
       RowBox[{"beta", " ", 
        SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
   RowBox[{"dx0", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"x0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"d", " ", "k"}], "+", 
              RowBox[{"g1", " ", 
               SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", "g0"}], "+", 
              RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
           RowBox[{"r", " ", 
            SuperscriptBox["r0", "3"]}]]}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"k", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
           RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
      RowBox[{"v0", " ", 
       RowBox[{"(", 
        RowBox[{"g1", "-", 
         FractionBox[
          RowBox[{"g2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
         FractionBox[
          RowBox[{"h", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]], "-", 
         FractionBox[
          RowBox[{"h", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
   RowBox[{"dv0", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"x0", " ", 
       RowBox[{"(", 
        RowBox[{"g1", "-", 
         FractionBox[
          RowBox[{"g2", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
         FractionBox[
          RowBox[{"h", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]], "-", 
         FractionBox[
          RowBox[{"h", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
      RowBox[{"v0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
         FractionBox[
          RowBox[{
           RowBox[{"eta", " ", "g1"}], "+", 
           RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
         FractionBox[
          RowBox[{"d", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
         FractionBox[
          RowBox[{"2", " ", "g2", " ", "h", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
         FractionBox[
          RowBox[{
           SuperscriptBox["h", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]], "+", 
         FractionBox[
          RowBox[{
           SuperscriptBox["h", "2"], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{
  3.776898171587162*^9, {3.7768984068779793`*^9, 3.776898495489285*^9}, {
   3.776898546757969*^9, 3.776898637553863*^9}, {3.776898669462574*^9, 
   3.7768987003438*^9}, {3.776898749473977*^9, 3.7768987917260027`*^9}, {
   3.776898839054337*^9, 3.776898845809492*^9}, {3.776902046924141*^9, 
   3.776902055037846*^9}, {3.7769088369741488`*^9, 3.7769088374352407`*^9}, 
   3.776909052861559*^9, 3.7769485158060303`*^9, {3.776948563087536*^9, 
   3.776948609534032*^9}, 3.776948863384659*^9, {3.7769489473693657`*^9, 
   3.776949020684278*^9}, {3.776949051056493*^9, 3.776949060848167*^9}, {
   3.7769846464450417`*^9, 3.776984729716003*^9}, {3.776984764683214*^9, 
   3.776984794907154*^9}, {3.776984851312603*^9, 3.776984882084465*^9}, {
   3.776984915396862*^9, 3.776984981984962*^9}, {3.7769850256207237`*^9, 
   3.7769850291814203`*^9}, {3.776985092607129*^9, 3.7769851179747467`*^9}, {
   3.7769851482642097`*^9, 3.776985149416173*^9}, {3.776985182808675*^9, 
   3.776985187775631*^9}, {3.776988515660851*^9, 3.7769885537849007`*^9}, {
   3.776991256532342*^9, 3.7769912866641407`*^9}, {3.7769914656601877`*^9, 
   3.776991466689455*^9}, 3.776995875728744*^9, 3.7769959250249147`*^9},
 CellLabel->"In[14]:=",ExpressionUUID->"5ddd5a2f-d46b-484e-813e-96d850cb906d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dk", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
      RowBox[{"beta", " ", "r0"}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "g1"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], "r"], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "beta"}], "+", 
            FractionBox[
             RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
         FractionBox[
          RowBox[{"eta", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}]}], 
          SuperscriptBox["r0", "3"]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
     FractionBox[
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
  RowBox[{"dx0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "-", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "-", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"eta", " ", "g1"}], "+", 
          RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
        FractionBox[
         RowBox[{"d", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"2", " ", "g2", " ", "h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        FractionBox[
         RowBox[{
          SuperscriptBox["h", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "+", 
        FractionBox[
         RowBox[{
          SuperscriptBox["h", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.776991287297529*^9, 3.776991467645105*^9, 
  3.776995877324524*^9, 3.7769959258599167`*^9, 3.7776931700916357`*^9},
 CellLabel->"Out[14]=",ExpressionUUID->"b933522a-6e79-447f-b846-a9ae7d2c4941"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dr", " ", "-", " ", "dr3"}]], "Input",
 CellChangeTimes->{{3.777693172433578*^9, 3.777693173710487*^9}},
 CellLabel->"In[15]:=",ExpressionUUID->"a7ffe149-f88e-46ce-9bb0-27b379ce4166"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "g1"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], "r"], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "beta"}], "+", 
            FractionBox[
             RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
         FractionBox[
          RowBox[{"eta", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}]}], 
          SuperscriptBox["r0", "3"]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
     FractionBox[
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]]}], ")"}]}], "-", 
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], "r"], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "-", 
     RowBox[{"eta", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"dx0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "-", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "-", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"eta", " ", "g1"}], "+", 
          RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
        FractionBox[
         RowBox[{"d", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"2", " ", "g2", " ", "h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        FractionBox[
         RowBox[{
          SuperscriptBox["h", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "+", 
        FractionBox[
         RowBox[{
          SuperscriptBox["h", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "-", 
  RowBox[{"dx0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        RowBox[{"h", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"d", " ", "k"}], "+", 
                RowBox[{"g1", " ", 
                 SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"eta", " ", "g0"}], "+", 
                RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
             RowBox[{"r", " ", 
              SuperscriptBox["r0", "3"]}]]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
            RowBox[{"beta", " ", 
             SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], ")"}]}], 
  "-", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        RowBox[{"h", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"d", " ", "k"}], "+", 
                RowBox[{"g1", " ", 
                 SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"eta", " ", "g0"}], "+", 
                RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
             RowBox[{"r", " ", 
              SuperscriptBox["r0", "3"]}]]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
            RowBox[{"beta", " ", 
             SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"eta", " ", "g1"}], "+", 
          RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
        FractionBox[
         RowBox[{"d", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        RowBox[{"2", " ", "h", " ", 
         RowBox[{"(", 
          RowBox[{"g1", "-", 
           FractionBox[
            RowBox[{"g2", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"eta", " ", "g0"}], "+", 
               RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}], "+", 
        RowBox[{
         SuperscriptBox["h", "2"], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            FractionBox[
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"d", " ", "k"}], "+", 
                RowBox[{"g1", " ", 
                 SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"eta", " ", "g0"}], "+", 
                RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
             RowBox[{"r", " ", 
              SuperscriptBox["r0", "3"]}]]}], "+", 
           FractionBox[
            RowBox[{
             RowBox[{"k", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
             RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
            RowBox[{"beta", " ", 
             SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.777693174105406*^9},
 CellLabel->"Out[15]=",ExpressionUUID->"d9d42105-a8fa-4468-849d-15fd8a5cf370"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.777693175847413*^9, 3.7776931769422894`*^9}},
 CellLabel->"In[16]:=",ExpressionUUID->"b266fc52-da22-4737-a7d7-167082344612"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.77769317724254*^9},
 CellLabel->"Out[16]=",ExpressionUUID->"4feff2ca-31fe-43e4-b153-429a09fb5e65"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"c1", "=", 
    RowBox[{"d", "-", 
     RowBox[{"r0", " ", "g3"}]}]}], "\n", 
   RowBox[{"c2", "=", 
    RowBox[{
     RowBox[{"eta", " ", "g0"}], "+", 
     RowBox[{"g1", " ", "zeta"}]}]}], "\n", 
   RowBox[{"c3", "=", 
    RowBox[{
     RowBox[{"d", " ", "k"}], "+", 
     RowBox[{"g1", " ", 
      SuperscriptBox["r0", "2"]}]}]}], "\n", 
   RowBox[{"c4", "=", 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], "+", 
     RowBox[{"2", "g0", " ", "r0"}]}]}], "\[IndentingNewLine]", 
   RowBox[{"c18", " ", "=", " ", 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
     RowBox[{"2", " ", "g2", " ", "k"}]}]}], "\n", 
   RowBox[{"c20", " ", "=", " ", 
    RowBox[{
     RowBox[{"k", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
     RowBox[{"g0", " ", "r0", " ", "zeta"}]}]}], 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.7769491328764477`*^9, 3.7769491334115953`*^9}, {
  3.776984663789102*^9, 3.776984679699718*^9}, {3.776985084623288*^9, 
  3.7769850901306334`*^9}, {3.776990952509952*^9, 3.776990960236972*^9}},
 CellLabel->"In[15]:=",ExpressionUUID->"7b09522d-a9ad-429b-8e6a-62696f3cc0de"],

Cell[BoxData[
 RowBox[{"d", "-", 
  RowBox[{"g3", " ", "r0"}]}]], "Output",
 CellChangeTimes->{3.7769491355248203`*^9, 3.776990926016759*^9, 
  3.77699096091949*^9, 3.7769910597414722`*^9, 3.777687334883114*^9, 
  3.777689444872463*^9, 3.777693208586227*^9, 3.778009887697353*^9},
 CellLabel->"Out[15]=",ExpressionUUID->"f1b9f8da-fce5-496f-9171-45625c4e27fa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"eta", " ", "g0"}], "+", 
  RowBox[{"g1", " ", "zeta"}]}]], "Output",
 CellChangeTimes->{3.7769491355248203`*^9, 3.776990926016759*^9, 
  3.77699096091949*^9, 3.7769910597414722`*^9, 3.777687334883114*^9, 
  3.777689444872463*^9, 3.777693208586227*^9, 3.778009887700149*^9},
 CellLabel->"Out[16]=",ExpressionUUID->"a8308bde-92c6-48d1-ad6f-06ad58a58d3f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"d", " ", "k"}], "+", 
  RowBox[{"g1", " ", 
   SuperscriptBox["r0", "2"]}]}]], "Output",
 CellChangeTimes->{3.7769491355248203`*^9, 3.776990926016759*^9, 
  3.77699096091949*^9, 3.7769910597414722`*^9, 3.777687334883114*^9, 
  3.777689444872463*^9, 3.777693208586227*^9, 3.77800988770156*^9},
 CellLabel->"Out[17]=",ExpressionUUID->"9b3f0f51-4daf-415b-9a48-da8cc060944f"],

Cell[BoxData[
 RowBox[{
  RowBox[{"eta", " ", "g1"}], "+", 
  RowBox[{"2", " ", "g0", " ", "r0"}]}]], "Output",
 CellChangeTimes->{3.7769491355248203`*^9, 3.776990926016759*^9, 
  3.77699096091949*^9, 3.7769910597414722`*^9, 3.777687334883114*^9, 
  3.777689444872463*^9, 3.777693208586227*^9, 3.778009887702984*^9},
 CellLabel->"Out[18]=",ExpressionUUID->"262cbdb9-d94f-4fef-bc3f-e98a39df3a3c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"eta", " ", "g1"}], "+", 
  RowBox[{"2", " ", "g2", " ", "k"}]}]], "Output",
 CellChangeTimes->{3.7769491355248203`*^9, 3.776990926016759*^9, 
  3.77699096091949*^9, 3.7769910597414722`*^9, 3.777687334883114*^9, 
  3.777689444872463*^9, 3.777693208586227*^9, 3.778009887704488*^9},
 CellLabel->"Out[19]=",ExpressionUUID->"e604a93d-e38e-4d40-95a7-1e76eead12ad"],

Cell[BoxData[
 RowBox[{
  RowBox[{"k", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
  RowBox[{"g0", " ", "r0", " ", "zeta"}]}]], "Output",
 CellChangeTimes->{3.7769491355248203`*^9, 3.776990926016759*^9, 
  3.77699096091949*^9, 3.7769910597414722`*^9, 3.777687334883114*^9, 
  3.777689444872463*^9, 3.777693208586227*^9, 3.778009887705912*^9},
 CellLabel->"Out[20]=",ExpressionUUID->"6b52690d-0193-4ecf-a077-f0651ee862ec"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Simplify", " ", 
   RowBox[{"further", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.7769487106857443`*^9, 3.776948765996578*^9}, {
  3.776948824159123*^9, 3.776948855877203*^9}, {3.776949114845767*^9, 
  3.77694911541495*^9}, {3.776949145477469*^9, 3.776949190870964*^9}, {
  3.7769705374135103`*^9, 
  3.7769705417568827`*^9}},ExpressionUUID->"5ca034f5-8343-48cc-8f0a-\
07db0c3ba317"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dr2", " ", "=", 
  RowBox[{
   RowBox[{"dk", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
       RowBox[{"beta", " ", "r0"}]], "+", 
      FractionBox[
       RowBox[{"c1", " ", "c2"}], 
       RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
   RowBox[{"dh", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"g2", " ", "c2", " "}], "r"], "-", "g1"}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "beta"}], "+", 
         FractionBox[
          RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
      FractionBox["c2", "r"], "+", 
      FractionBox[
       RowBox[{"eta", " ", "c3", " ", "c2"}], 
       RowBox[{"r", " ", 
        SuperscriptBox["r0", "3"]}]], "-", 
      FractionBox[
       RowBox[{"eta", " ", "c20"}], 
       RowBox[{"beta", " ", 
        SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
   RowBox[{"dx0", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"x0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"c2", " ", "c3"}], 
           RowBox[{"r", " ", 
            SuperscriptBox["r0", "3"]}]]}], "+", 
         FractionBox["c20", 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
      RowBox[{"v0", " ", 
       RowBox[{"(", 
        RowBox[{"g1", "-", 
         FractionBox[
          RowBox[{"g2", " ", "c2"}], "r"], "+", 
         FractionBox[
          RowBox[{"h", " ", "c2", " ", "c3"}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]], "-", 
         FractionBox[
          RowBox[{"h", " ", "c20"}], 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
   RowBox[{"dv0", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"x0", " ", 
       RowBox[{"(", 
        RowBox[{"g1", "-", 
         FractionBox[
          RowBox[{"g2", " ", "c2"}], "r"], "+", 
         FractionBox[
          RowBox[{"h", " ", "c3", " ", "c2"}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]], "-", 
         FractionBox[
          RowBox[{"h", " ", "c20"}], 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
      RowBox[{"v0", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
         FractionBox["c18", "beta"], "-", 
         FractionBox[
          RowBox[{"d", " ", "c2"}], "r"], "+", 
         FractionBox[
          RowBox[{"2", " ", "g2", " ", "h", " ", "c2"}], "r"], "-", 
         FractionBox[
          RowBox[{
           SuperscriptBox["h", "2"], " ", "c3", " ", "c2"}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]], "+", 
         FractionBox[
          RowBox[{
           SuperscriptBox["h", "2"], " ", "c20"}], 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}]}]], "Input",
 CellChangeTimes->{{3.7769491551169243`*^9, 3.776949256253709*^9}, {
   3.776949348166195*^9, 3.7769493795136633`*^9}, {3.776949549196755*^9, 
   3.7769495587779713`*^9}, {3.776970615059394*^9, 3.776970641733863*^9}, {
   3.776970683927231*^9, 3.776970684256755*^9}, 3.7769848384040422`*^9, {
   3.776988566834506*^9, 3.776988765806467*^9}, {3.7769888745269413`*^9, 
   3.7769888827419367`*^9}, {3.776989040655405*^9, 3.776989041472027*^9}, 
   3.776990932679525*^9, {3.77699132430626*^9, 3.776991331287022*^9}, {
   3.7769959502470827`*^9, 3.776995986879425*^9}, {3.7780310408881607`*^9, 
   3.77803106766177*^9}},
 CellLabel->"In[23]:=",ExpressionUUID->"749d2d9f-b759-4bbd-8637-75a3f6857347"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dk", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "g2"}], " ", "k"}], "-", "r", "+", "r0"}], 
      RowBox[{"beta", " ", "r0"}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"d", "-", 
         RowBox[{"g3", " ", "r0"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"r", " ", "r0"}]]}], ")"}]}], "+", 
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], "r"], "+", 
     FractionBox[
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "k"}], "+", 
         RowBox[{"g1", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"r", " ", 
       SuperscriptBox["r0", "3"]}]], "-", 
     FractionBox[
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "g1"}], "+", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}]}], 
    ")"}]}], "+", 
  RowBox[{"dx0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g0"}], "+", 
             RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]]}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"k", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
          RowBox[{"g0", " ", "r0", " ", "zeta"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "-", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"dv0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"x0", " ", 
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "-", 
        FractionBox[
         RowBox[{"h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
     RowBox[{"v0", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
        FractionBox[
         RowBox[{
          RowBox[{"eta", " ", "g1"}], "+", 
          RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
        FractionBox[
         RowBox[{"d", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "+", 
        FractionBox[
         RowBox[{"2", " ", "g2", " ", "h", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
        FractionBox[
         RowBox[{
          SuperscriptBox["h", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"d", " ", "k"}], "+", 
            RowBox[{"g1", " ", 
             SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"r", " ", 
          SuperscriptBox["r0", "3"]}]], "+", 
        FractionBox[
         RowBox[{
          SuperscriptBox["h", "2"], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"k", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
            RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.776990929460455*^9, 3.776991062228381*^9, 
  3.77699133367022*^9, 3.776991474528963*^9, 3.776995990744899*^9, 
  3.777693210639011*^9},
 CellLabel->"Out[23]=",ExpressionUUID->"4e20d382-8e52-4b4d-8ebe-14a672d67e1e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dr", " ", "-", " ", "dr2"}]], "Input",
 CellChangeTimes->{{3.7769909345243597`*^9, 3.776990937709453*^9}, {
  3.776991356302053*^9, 3.776991357780257*^9}, {3.7769914811448*^9, 
  3.776991484049222*^9}},
 CellLabel->"In[24]:=",ExpressionUUID->"85ecd29e-7dc6-4a46-bb6b-50623f9d5fc4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "g1"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], "r"], "+", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "beta"}], "+", 
            FractionBox[
             RowBox[{"2", " ", "k"}], "r0"]}], ")"}]}], "+", 
         FractionBox[
          RowBox[{"eta", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"d", " ", "k"}], "+", 
             RowBox[{"g1", " ", 
              SuperscriptBox["r0", "2"]}]}], ")"}]}], 
          SuperscriptBox["r0", "3"]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"], "-", 
     FractionBox[
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]]}], ")"}]}], "-", 
  RowBox[{"dh", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], "r"], "+", 
     FractionBox[
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"d", " ", "k"}], "+", 
         RowBox[{"g1", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g0"}], "+", 
         RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"r", " ", 
       SuperscriptBox["r0", "3"]}]], "-", 
     FractionBox[
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g2", " ", "k"}], "+", "r"}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}]], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "beta"}], "+", 
        FractionBox[
         RowBox[{"2", " ", "k"}], "r0"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "g1"}], "+", 
        FractionBox[
         RowBox[{"g2", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"eta", " ", "g0"}], "+", 
            RowBox[{"g1", " ", "zeta"}]}], ")"}]}], "r"]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.777693212929142*^9},
 CellLabel->"Out[24]=",ExpressionUUID->"af1653e1-804c-496b-ba53-04fe5b8489c0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.7769959953195963`*^9, 3.776995996598722*^9}},
 CellLabel->"In[25]:=",ExpressionUUID->"da464f5d-15d8-455a-967b-5f814bfda060"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.776995996854189*^9, 3.77769321456304*^9},
 CellLabel->"Out[25]=",ExpressionUUID->"7ad9fa14-b279-4a14-8f34-51f79d68ba82"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"See", " ", "if", " ", "I", " ", "can", " ", "simplify", " ", 
   RowBox[{"dv0", ".", "v0"}], " ", 
   RowBox[{"term", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.777687305737104*^9, 3.7776873170784903`*^9}, {
  3.777689423424028*^9, 
  3.777689423452648*^9}},ExpressionUUID->"cb695c37-a5f2-4ae9-9f5d-\
c447b915243f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
      FractionBox["c18", "beta"], "-", 
      FractionBox[
       RowBox[{"d", " ", "c2"}], "r"], "+", 
      FractionBox[
       RowBox[{"2", " ", "g2", " ", "h", " ", "c2"}], "r"], "-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["h", "2"], " ", "c3", " ", "c2"}], 
       RowBox[{"r", " ", 
        SuperscriptBox["r0", "3"]}]], "+", 
      FractionBox[
       RowBox[{
        SuperscriptBox["h", "2"], " ", "c20"}], 
       RowBox[{"beta", " ", 
        SuperscriptBox["r0", "3"]}]]}], ")"}], " ", "/.", " ", 
    RowBox[{"d", " ", "\[Rule]", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"h", "+", 
        RowBox[{"eta", "*", "g2"}], "+", 
        RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}], " ", "/.",
    " ", 
   RowBox[{"h", " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
     RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
     RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
  RowBox[{"r", " ", "\[Rule]", " ", 
   RowBox[{
    RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
    RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
    RowBox[{"k", " ", "g2"}]}]}]}]], "Input",
 CellChangeTimes->{{3.777687318751251*^9, 3.777687320246567*^9}, {
  3.777689439967181*^9, 3.777689478781611*^9}, {3.777689589735141*^9, 
  3.777689613269949*^9}},
 CellLabel->
  "In[182]:=",ExpressionUUID->"7b93e751-e210-4f4f-82cf-2aee0376560b"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"eta", " ", "g1"}], "+", 
    RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
  RowBox[{"2", " ", "g1", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g2"}], "+", 
     RowBox[{"g3", " ", "k"}], "+", 
     RowBox[{"g1", " ", "r0"}]}], ")"}]}], "+", 
  FractionBox[
   RowBox[{"2", " ", "g2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g2"}], "+", 
      RowBox[{"g3", " ", "k"}], "+", 
      RowBox[{"g1", " ", "r0"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g0"}], "+", 
      RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
   RowBox[{
    RowBox[{"eta", " ", "g1"}], "+", 
    RowBox[{"g2", " ", "k"}], "+", 
    RowBox[{"g0", " ", "r0"}]}]], "-", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "eta", " ", "g2"}], "+", 
      RowBox[{"3", " ", "g3", " ", "k"}], "+", 
      RowBox[{"g1", " ", "r0"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g0"}], "+", 
      RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
   RowBox[{"beta", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g1"}], "+", 
      RowBox[{"g2", " ", "k"}], "+", 
      RowBox[{"g0", " ", "r0"}]}], ")"}]}]], "-", 
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g2"}], "+", 
       RowBox[{"g3", " ", "k"}], "+", 
       RowBox[{"g1", " ", "r0"}]}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"g1", " ", 
       SuperscriptBox["r0", "2"]}], "+", 
      FractionBox[
       RowBox[{"k", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", "eta", " ", "g2"}], "+", 
          RowBox[{"3", " ", "g3", " ", "k"}], "+", 
          RowBox[{"g1", " ", "r0"}]}], ")"}]}], "beta"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g0"}], "+", 
      RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
   RowBox[{
    SuperscriptBox["r0", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g1"}], "+", 
      RowBox[{"g2", " ", "k"}], "+", 
      RowBox[{"g0", " ", "r0"}]}], ")"}]}]], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g2"}], "+", 
       RowBox[{"g3", " ", "k"}], "+", 
       RowBox[{"g1", " ", "r0"}]}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"k", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g1"}], "+", 
         RowBox[{"2", " ", "g2", " ", "k"}], "+", 
         RowBox[{"g0", " ", "r0"}]}], ")"}]}], "-", 
      RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], 
   RowBox[{"beta", " ", 
    SuperscriptBox["r0", "3"]}]]}]], "Output",
 CellChangeTimes->{3.777689479275063*^9, 3.7776896137849483`*^9},
 CellLabel->
  "Out[182]=",ExpressionUUID->"cc39e4d9-bdc2-415c-bbc2-0921a86b7f30"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"eta", " ", "g1"}], "+", 
    RowBox[{"2", " ", "g2", " ", "k"}], "-", 
    RowBox[{"2", " ", "g1", " ", "beta", " ", "h"}]}], "beta"], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"g1", " ", "r0"}], "-", 
      RowBox[{"g3", " ", "k"}], "-", 
      RowBox[{"2", " ", "g0", " ", "h"}]}], ")"}], " ", "c2"}], 
   RowBox[{"beta", " ", "r"}]], "+", 
  RowBox[{
   SuperscriptBox["h", "2"], 
   FractionBox[
    RowBox[{" ", 
     RowBox[{
      RowBox[{"r", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{"r", "+", " ", 
            RowBox[{"g2", " ", "k"}]}], ")"}]}], "-", 
         RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], "-", 
      RowBox[{"c2", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"g1", " ", "beta", " ", 
          SuperscriptBox["r0", "2"]}], "+", 
         RowBox[{"k", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "eta", " ", "g2"}], "+", 
            RowBox[{"3", " ", "g3", " ", "k"}], "+", 
            RowBox[{"g1", " ", "r0"}]}], ")"}]}]}], ")"}], " "}]}]}], 
    RowBox[{"beta", " ", "r", " ", 
     SuperscriptBox["r0", "3"]}]]}]}]], "Input",
 CellChangeTimes->{{3.777689647865703*^9, 3.7776896552244577`*^9}, {
  3.777689686754079*^9, 3.777689705776935*^9}, {3.777689752067355*^9, 
  3.777690061490732*^9}, {3.777690093406459*^9, 3.777690156162654*^9}, {
  3.777690202829974*^9, 
  3.777690383542495*^9}},ExpressionUUID->"67d76b09-86e0-4efb-a78b-\
a12cdacdc517"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"r", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"k", " ", 
        RowBox[{"(", 
         RowBox[{"r", "+", " ", 
          RowBox[{"g2", " ", "k"}]}], ")"}]}], "-", 
       RowBox[{"g0", " ", "r0", " ", "zeta"}]}], ")"}]}], "-", 
    RowBox[{"c2", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"g1", " ", "beta", " ", 
        SuperscriptBox["r0", "2"]}], "+", 
       RowBox[{"k", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", "eta", " ", "g2"}], "+", 
          RowBox[{"3", " ", "g3", " ", "k"}], "+", 
          RowBox[{"g1", " ", "r0"}]}], ")"}]}]}], ")"}]}]}], " ", "/.", " ", 
   RowBox[{"r", " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
     RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
     RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
  RowBox[{"zeta", " ", "\[Rule]", " ", 
   RowBox[{"k", " ", "-", " ", 
    RowBox[{"beta", " ", "r0", " "}]}]}]}]], "Input",
 CellChangeTimes->{{3.777690396213887*^9, 3.777690415629406*^9}},
 CellLabel->
  "In[183]:=",ExpressionUUID->"0ea04d4e-d261-404e-b23c-65483ecdafc0"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], "+", 
     RowBox[{"g2", " ", "k"}], "+", 
     RowBox[{"g0", " ", "r0"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "g0"}], " ", "r0", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}], "+", 
     RowBox[{"k", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"eta", " ", "g1"}], "+", 
        RowBox[{"2", " ", "g2", " ", "k"}], "+", 
        RowBox[{"g0", " ", "r0"}]}], ")"}]}]}], ")"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g0"}], "+", 
     RowBox[{"g1", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"beta", " ", "g1", " ", 
      SuperscriptBox["r0", "2"]}], "+", 
     RowBox[{"k", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "eta", " ", "g2"}], "+", 
        RowBox[{"3", " ", "g3", " ", "k"}], "+", 
        RowBox[{"g1", " ", "r0"}]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.777690416627817*^9},
 CellLabel->
  "Out[183]=",ExpressionUUID->"19c3bdb2-1c2c-4c4f-8ad1-44660fd17e7b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Expand", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.777690424310767*^9, 3.7776904307178698`*^9}},
 CellLabel->
  "In[184]:=",ExpressionUUID->"80a0bb6f-4739-40d4-8cec-775969014b4a"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["eta", "2"], " ", 
   SuperscriptBox["g1", "2"], " ", "k"}], "-", 
  RowBox[{"2", " ", 
   SuperscriptBox["eta", "2"], " ", "g0", " ", "g2", " ", "k"}], "+", 
  RowBox[{"eta", " ", "g1", " ", "g2", " ", 
   SuperscriptBox["k", "2"]}], "-", 
  RowBox[{"3", " ", "eta", " ", "g0", " ", "g3", " ", 
   SuperscriptBox["k", "2"]}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox["g2", "2"], " ", 
   SuperscriptBox["k", "3"]}], "-", 
  RowBox[{"3", " ", "g1", " ", "g3", " ", 
   SuperscriptBox["k", "3"]}], "+", 
  RowBox[{
  "2", " ", "beta", " ", "eta", " ", "g1", " ", "g2", " ", "k", " ", "r0"}], 
  "-", 
  RowBox[{
   SuperscriptBox["g1", "2"], " ", 
   SuperscriptBox["k", "2"], " ", "r0"}], "+", 
  RowBox[{"2", " ", "g0", " ", "g2", " ", 
   SuperscriptBox["k", "2"], " ", "r0"}], "+", 
  RowBox[{"3", " ", "beta", " ", "g1", " ", "g3", " ", 
   SuperscriptBox["k", "2"], " ", "r0"}], "+", 
  RowBox[{"beta", " ", "g0", " ", "g2", " ", "k", " ", 
   SuperscriptBox["r0", "2"]}], "+", 
  RowBox[{"beta", " ", 
   SuperscriptBox["g0", "2"], " ", 
   SuperscriptBox["r0", "3"]}], "+", 
  RowBox[{
   SuperscriptBox["beta", "2"], " ", 
   SuperscriptBox["g1", "2"], " ", 
   SuperscriptBox["r0", "3"]}]}]], "Output",
 CellChangeTimes->{3.777690431039816*^9},
 CellLabel->
  "Out[184]=",ExpressionUUID->"fa82927d-9c72-48af-9f2d-0c39810cfd90"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "eta", ",", "r0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.77769043350841*^9, 3.777690442069701*^9}},
 CellLabel->
  "In[185]:=",ExpressionUUID->"d190f851-72af-4d84-9145-dcfafee72abd"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["g2", "2"]}], "-", 
     RowBox[{"3", " ", "g1", " ", "g3"}]}], ")"}], " ", 
   SuperscriptBox["k", "3"]}], "+", 
  RowBox[{"beta", " ", 
   SuperscriptBox["r0", "3"]}], "+", 
  RowBox[{
   SuperscriptBox["k", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"g1", " ", "g2"}], "-", 
        RowBox[{"3", " ", "g0", " ", "g3"}]}], ")"}]}], "+", 
     RowBox[{"beta", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"3", " ", "g1", " ", "g3"}], "-", " ", 
        SuperscriptBox["g2", "2"]}], ")"}], " ", "r0"}]}], ")"}]}], "+", 
  RowBox[{"k", " ", "beta", " ", "g2", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["eta", "2"], " ", "g2"}], "+", 
     RowBox[{"2", "  ", "eta", " ", "g1", "  ", "r0"}], "+", " ", 
     RowBox[{"g0", " ", 
      SuperscriptBox["r0", "2"]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.777690451167603*^9, 
  3.777690625829619*^9}},ExpressionUUID->"f885f30a-c63a-4759-8152-\
17ba648fe1c9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"eta", " ", "g1"}], "+", 
    RowBox[{"2", " ", "g2", " ", "k"}], "-", 
    RowBox[{"2", " ", "g1", " ", "beta", " ", "h"}]}], "beta"], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"g1", " ", "r0"}], "-", 
      RowBox[{"g3", " ", "k"}], "-", 
      RowBox[{"2", " ", "g0", " ", "h"}]}], ")"}], " ", "c2"}], 
   RowBox[{"beta", " ", "r"}]], "+", 
  RowBox[{
   SuperscriptBox["h", "2"], 
   FractionBox[
    RowBox[{" ", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox["g2", "2"]}], "-", 
         RowBox[{"3", " ", "g1", " ", "g3"}]}], ")"}], " ", 
       SuperscriptBox["k", "3"]}], "+", 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}], "+", 
      RowBox[{
       SuperscriptBox["k", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g1", " ", "g2"}], "-", 
            RowBox[{"3", " ", "g0", " ", "g3"}]}], ")"}]}], "+", 
         RowBox[{"beta", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"3", " ", "g1", " ", "g3"}], "-", " ", 
            SuperscriptBox["g2", "2"]}], ")"}], " ", "r0"}]}], ")"}]}], "+", 
      RowBox[{"k", " ", "beta", " ", "g2", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["eta", "2"], " ", "g2"}], "+", 
         RowBox[{"2", "  ", "eta", " ", "g1", "  ", "r0"}], "+", " ", 
         RowBox[{"g0", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}], " "}]}]}], 
    RowBox[{"beta", " ", "r", " ", 
     SuperscriptBox["r0", "3"]}]]}]}]], "Input",
 CellChangeTimes->{3.777690682391182*^9},
 CellLabel->
  "In[186]:=",ExpressionUUID->"4c41ccac-c0ea-4308-ac55-1dec6c9c77a1"],

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{
    RowBox[{"eta", " ", "g1"}], "-", 
    RowBox[{"2", " ", "beta", " ", "g1", " ", "h"}], "+", 
    RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox["h", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox["g2", "2"]}], "-", 
         RowBox[{"3", " ", "g1", " ", "g3"}]}], ")"}], " ", 
       SuperscriptBox["k", "3"]}], "+", 
      RowBox[{"beta", " ", 
       SuperscriptBox["r0", "3"]}], "+", 
      RowBox[{
       SuperscriptBox["k", "2"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"g1", " ", "g2"}], "-", 
            RowBox[{"3", " ", "g0", " ", "g3"}]}], ")"}]}], "+", 
         RowBox[{"beta", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox["g2", "2"]}], "+", 
            RowBox[{"3", " ", "g1", " ", "g3"}]}], ")"}], " ", "r0"}]}], 
        ")"}]}], "+", 
      RowBox[{"beta", " ", "g2", " ", "k", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["eta", "2"], " ", "g2"}], "+", 
         RowBox[{"2", " ", "eta", " ", "g1", " ", "r0"}], "+", 
         RowBox[{"g0", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}]}]}], ")"}]}], 
   RowBox[{"beta", " ", "r", " ", 
    SuperscriptBox["r0", "3"]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "2"}], " ", "g0", " ", "h"}], "-", 
      RowBox[{"g3", " ", "k"}], "+", 
      RowBox[{"g1", " ", "r0"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g0"}], "+", 
      RowBox[{"g1", " ", "zeta"}]}], ")"}]}], 
   RowBox[{"beta", " ", "r"}]]}]], "Output",
 CellChangeTimes->{3.777690687248961*^9},
 CellLabel->
  "Out[186]=",ExpressionUUID->"79ed7c08-1c23-482f-9b6c-2bb474ef1762"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       FractionBox[
        RowBox[{
         RowBox[{"eta", " ", "g1"}], "+", 
         RowBox[{"2", " ", "g2", " ", "k"}]}], "beta"], "-", 
       RowBox[{"2", " ", "g1", " ", "h"}], "+", 
       FractionBox[
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"g1", " ", "r0"}], "-", 
           RowBox[{"g3", " ", "k"}], "-", 
           RowBox[{"2", " ", "g0", " ", "h"}]}], ")"}], " ", "c2"}], 
        RowBox[{"beta", " ", "r"}]], "+", 
       RowBox[{
        SuperscriptBox["h", "2"], 
        FractionBox[
         RowBox[{" ", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"2", " ", 
               SuperscriptBox["g2", "2"]}], "-", 
              RowBox[{"3", " ", "g1", " ", "g3"}]}], ")"}], " ", 
            SuperscriptBox["k", "3"]}], "+", 
           RowBox[{"beta", " ", 
            SuperscriptBox["r0", "3"]}], "+", 
           RowBox[{
            SuperscriptBox["k", "2"], " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"g1", " ", "g2"}], "-", 
                 RowBox[{"3", " ", "g0", " ", "g3"}]}], ")"}]}], "+", 
              RowBox[{"beta", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"3", " ", "g1", " ", "g3"}], "-", " ", 
                 SuperscriptBox["g2", "2"]}], ")"}], " ", "r0"}]}], ")"}]}], 
           "+", 
           RowBox[{"k", " ", "beta", " ", "g2", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"eta", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"eta", " ", "g2"}], "+", " ", 
                 RowBox[{"g1", " ", "r0"}]}], ")"}]}], " ", "+", " ", 
              RowBox[{"r0", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"eta", " ", "g1"}], "+", " ", 
                 RowBox[{"g0", " ", "r0"}]}], ")"}]}]}], ")"}], " "}]}]}], 
         RowBox[{"beta", " ", "r", " ", 
          SuperscriptBox["r0", "3"]}]]}], "-", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
         FractionBox["c18", "beta"], "-", 
         FractionBox[
          RowBox[{"d", " ", "c2"}], "r"], "+", 
         FractionBox[
          RowBox[{"2", " ", "g2", " ", "h", " ", "c2"}], "r"], "-", 
         FractionBox[
          RowBox[{
           SuperscriptBox["h", "2"], " ", "c3", " ", "c2"}], 
          RowBox[{"r", " ", 
           SuperscriptBox["r0", "3"]}]], "+", 
         FractionBox[
          RowBox[{
           SuperscriptBox["h", "2"], " ", "c20"}], 
          RowBox[{"beta", " ", 
           SuperscriptBox["r0", "3"]}]]}], ")"}]}], " ", "/.", " ", 
      RowBox[{"zeta", " ", "\[Rule]", " ", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
     RowBox[{"d", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"h", "+", 
         RowBox[{"eta", "*", "g2"}], "+", 
         RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}], " ", 
    "/.", " ", 
    RowBox[{"h", " ", "\[Rule]", " ", 
     RowBox[{
      RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
      RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
      RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
   RowBox[{"r", " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
     RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
     RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
  RowBox[{"g0", " ", "\[Rule]", " ", 
   RowBox[{"1", "-", 
    RowBox[{"beta", " ", "g2"}]}]}]}]], "Input",
 CellChangeTimes->{{3.777690761565156*^9, 3.777690764885811*^9}, {
  3.777690805325873*^9, 3.777690898250841*^9}, {3.777691237653791*^9, 
  3.777691271094998*^9}, {3.7776916394262114`*^9, 3.777691643720356*^9}},
 CellLabel->
  "In[192]:=",ExpressionUUID->"76d5ec85-e7d3-4444-86cf-b3b5c22db434"],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", "g2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g2"}], "+", 
       RowBox[{"g3", " ", "k"}], "+", 
       RowBox[{"g1", " ", "r0"}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", 
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"beta", " ", "g2"}]}], ")"}]}], "+", 
       RowBox[{"g1", " ", 
        RowBox[{"(", 
         RowBox[{"k", "-", 
          RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], "+", 
     RowBox[{"g2", " ", "k"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", 
        RowBox[{"beta", " ", "g2"}]}], ")"}], " ", "r0"}]}]]}], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "eta", " ", "g2"}], "+", 
      RowBox[{"3", " ", "g3", " ", "k"}], "+", 
      RowBox[{"g1", " ", "r0"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2"}]}], ")"}]}], "+", 
      RowBox[{"g1", " ", 
       RowBox[{"(", 
        RowBox[{"k", "-", 
         RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}]}], 
   RowBox[{"beta", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g1"}], "+", 
      RowBox[{"g2", " ", "k"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2"}]}], ")"}], " ", "r0"}]}], ")"}]}]], "+", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2"}]}], ")"}]}], "+", 
      RowBox[{"g1", " ", 
       RowBox[{"(", 
        RowBox[{"k", "-", 
         RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "g3"}], " ", "k"}], "+", 
      RowBox[{"g1", " ", "r0"}], "-", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2"}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g2"}], "+", 
         RowBox[{"g3", " ", "k"}], "+", 
         RowBox[{"g1", " ", "r0"}]}], ")"}]}]}], ")"}]}], 
   RowBox[{"beta", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g1"}], "+", 
      RowBox[{"g2", " ", "k"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2"}]}], ")"}], " ", "r0"}]}], ")"}]}]], "+", 
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g2"}], "+", 
       RowBox[{"g3", " ", "k"}], "+", 
       RowBox[{"g1", " ", "r0"}]}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2"}]}], ")"}]}], "+", 
      RowBox[{"g1", " ", 
       RowBox[{"(", 
        RowBox[{"k", "-", 
         RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"g1", " ", 
       SuperscriptBox["r0", "2"]}], "+", 
      FractionBox[
       RowBox[{"k", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", "eta", " ", "g2"}], "+", 
          RowBox[{"3", " ", "g3", " ", "k"}], "+", 
          RowBox[{"g1", " ", "r0"}]}], ")"}]}], "beta"]}], ")"}]}], 
   RowBox[{
    SuperscriptBox["r0", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g1"}], "+", 
      RowBox[{"g2", " ", "k"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", 
         RowBox[{"beta", " ", "g2"}]}], ")"}], " ", "r0"}]}], ")"}]}]], "-", 
  FractionBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g2"}], "+", 
       RowBox[{"g3", " ", "k"}], "+", 
       RowBox[{"g1", " ", "r0"}]}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"beta", " ", "g2"}]}], ")"}]}], " ", "r0", " ", 
       RowBox[{"(", 
        RowBox[{"k", "-", 
         RowBox[{"beta", " ", "r0"}]}], ")"}]}], "+", 
      RowBox[{"k", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g1"}], "+", 
         RowBox[{"2", " ", "g2", " ", "k"}], "+", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"1", "-", 
            RowBox[{"beta", " ", "g2"}]}], ")"}], " ", "r0"}]}], ")"}]}]}], 
     ")"}]}], 
   RowBox[{"beta", " ", 
    SuperscriptBox["r0", "3"]}]], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"eta", " ", "g2"}], "+", 
        RowBox[{"g3", " ", "k"}], "+", 
        RowBox[{"g1", " ", "r0"}]}], ")"}], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox["g2", "2"]}], "-", 
          RowBox[{"3", " ", "g1", " ", "g3"}]}], ")"}], " ", 
        SuperscriptBox["k", "3"]}], "+", 
       RowBox[{"beta", " ", 
        SuperscriptBox["r0", "3"]}], "+", 
       RowBox[{
        SuperscriptBox["k", "2"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"eta", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"g1", " ", "g2"}], "-", 
             RowBox[{"3", " ", 
              RowBox[{"(", 
               RowBox[{"1", "-", 
                RowBox[{"beta", " ", "g2"}]}], ")"}], " ", "g3"}]}], ")"}]}], 
          "+", 
          RowBox[{"beta", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", 
              SuperscriptBox["g2", "2"]}], "+", 
             RowBox[{"3", " ", "g1", " ", "g3"}]}], ")"}], " ", "r0"}]}], 
         ")"}]}], "+", 
       RowBox[{"beta", " ", "g2", " ", "k", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"eta", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g2"}], "+", 
             RowBox[{"g1", " ", "r0"}]}], ")"}]}], "+", 
          RowBox[{"r0", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"eta", " ", "g1"}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"1", "-", 
                RowBox[{"beta", " ", "g2"}]}], ")"}], " ", "r0"}]}], 
            ")"}]}]}], ")"}]}]}], ")"}]}], ")"}], "/", 
   RowBox[{"(", 
    RowBox[{"beta", " ", 
     SuperscriptBox["r0", "3"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g1"}], "+", 
       RowBox[{"g2", " ", "k"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"beta", " ", "g2"}]}], ")"}], " ", "r0"}]}], ")"}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{{3.77769124885513*^9, 3.777691272658802*^9}, 
   3.777691644083699*^9},
 CellLabel->
  "Out[192]=",ExpressionUUID->"1786ef8d-ebd6-4c93-9e24-fc9852f9ac2e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.777691250806057*^9, 3.77769125284543*^9}},
 CellLabel->
  "In[193]:=",ExpressionUUID->"c58a2338-2944-49e5-9a0d-fe159321e125"],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["g1", "2"], "+", 
      RowBox[{"g2", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "2"}], "+", 
         RowBox[{"beta", " ", "g2"}]}], ")"}]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g2"}], "+", 
       RowBox[{"g3", " ", "k"}], "+", 
       RowBox[{"g1", " ", "r0"}]}], ")"}], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["eta", "2"], " ", "k"}], "-", 
      RowBox[{
       SuperscriptBox["k", "2"], " ", "r0"}], "+", 
      RowBox[{
       SuperscriptBox["beta", "2"], " ", 
       SuperscriptBox["r0", "3"]}]}], ")"}]}], 
   RowBox[{"beta", " ", 
    SuperscriptBox["r0", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g1"}], "+", "r0", "+", 
      RowBox[{"g2", " ", 
       RowBox[{"(", 
        RowBox[{"k", "-", 
         RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}]}]]}]], "Output",
 CellChangeTimes->{{3.777691253274219*^9, 3.777691275281115*^9}, 
   3.777691646018815*^9},
 CellLabel->
  "Out[193]=",ExpressionUUID->"1d7ab61c-c3d4-4663-89e0-05d19e0aaddc"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"g2", "*", 
  RowBox[{"(", 
   RowBox[{"g0", "  ", "-", " ", "1", " ", "+", "  ", 
    RowBox[{"beta", " ", "g2"}]}], ")"}]}]], "Input",
 CellChangeTimes->{{3.777691667701941*^9, 3.777691703611165*^9}, {
  3.7776917430358067`*^9, 
  3.777691772643145*^9}},ExpressionUUID->"346d9a50-3203-4a55-bec9-\
c10ff1e4e411"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Okay", ",", " ", 
   RowBox[{
   "now", " ", "see", " ", "if", " ", "I", " ", "can", " ", "simplify", " ", 
    "the", " ", 
    RowBox[{"dv0", " ", ".", " ", "x0"}], " ", 
    RowBox[{"term", ":"}]}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.777691815506995*^9, 
  3.777691826409186*^9}},ExpressionUUID->"9bea5a63-e6fd-4f17-8cb6-\
b09087afe8f5"],

Cell[BoxData[
 RowBox[{"(", 
  RowBox[{"g1", "-", 
   FractionBox[
    RowBox[{"g2", " ", "c2"}], "r"], "+", 
   RowBox[{"h", 
    RowBox[{"(", 
     FractionBox[
      RowBox[{" ", 
       RowBox[{
        RowBox[{"beta", " ", "c3", " ", "c2"}], "-", 
        RowBox[{"r", " ", "c20"}]}]}], 
      RowBox[{"beta", " ", "r", " ", 
       SuperscriptBox["r0", "3"]}]], ")"}]}]}], ")"}]], "Input",
 CellChangeTimes->{{3.7776918499602203`*^9, 
  3.7776918712471933`*^9}},ExpressionUUID->"2786cd89-4914-46b4-baae-\
d874c1c8d48f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"beta", " ", "c3", " ", "c2"}], "-", 
      RowBox[{"r", " ", "c20"}]}], " ", "/.", " ", 
     RowBox[{"zeta", " ", "\[Rule]", " ", 
      RowBox[{"k", "-", 
       RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
    RowBox[{"d", " ", "\[Rule]", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"h", "+", 
        RowBox[{"eta", "*", "g2"}], "+", 
        RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}], " ", "/.",
    " ", 
   RowBox[{"h", " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
     RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
     RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
  RowBox[{"r", " ", "\[Rule]", " ", 
   RowBox[{
    RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
    RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
    RowBox[{"k", " ", "g2", " "}]}]}]}]], "Input",
 CellChangeTimes->{{3.777691876046978*^9, 3.777691896066188*^9}},
 CellLabel->
  "In[195]:=",ExpressionUUID->"0aa8508b-6614-4f8c-bd79-45bb90af864c"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g1"}], "+", 
      RowBox[{"g2", " ", "k"}], "+", 
      RowBox[{"g0", " ", "r0"}]}], ")"}]}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "g0"}], " ", "r0", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}], "+", 
     RowBox[{"k", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"eta", " ", "g1"}], "+", 
        RowBox[{"2", " ", "g2", " ", "k"}], "+", 
        RowBox[{"g0", " ", "r0"}]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{"beta", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g0"}], "+", 
     RowBox[{"g1", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g1", " ", 
      SuperscriptBox["r0", "2"]}], "+", 
     FractionBox[
      RowBox[{"k", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "eta", " ", "g2"}], "+", 
         RowBox[{"3", " ", "g3", " ", "k"}], "+", 
         RowBox[{"g1", " ", "r0"}]}], ")"}]}], "beta"]}], ")"}]}]}]], "Output",\

 CellChangeTimes->{{3.777691882239085*^9, 3.777691896929818*^9}},
 CellLabel->
  "Out[195]=",ExpressionUUID->"f17344e3-2c78-4e1c-a694-995f7d92755e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g1"}], "+", 
      RowBox[{"g2", " ", "k"}], "+", 
      RowBox[{"g0", " ", "r0"}]}], ")"}]}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "g0"}], " ", "r0", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}], "+", 
     RowBox[{"k", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"eta", " ", "g1"}], "+", 
        RowBox[{"2", " ", "g2", " ", "k"}], "+", 
        RowBox[{"g0", " ", "r0"}]}], ")"}]}]}], ")"}]}], "+", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g0"}], "+", 
     RowBox[{"g1", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g1", " ", "beta", " ", 
      SuperscriptBox["r0", "2"]}], "+", 
     RowBox[{"2", " ", "k", " ", "eta", " ", "g2"}], "+", 
     RowBox[{"3", 
      SuperscriptBox["k", "2"], " ", "g3"}], "+", 
     RowBox[{"g1", " ", "k", " ", "r0"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.777691910121223*^9, 3.7776919297589617`*^9}},
 CellLabel->
  "In[196]:=",ExpressionUUID->"3681943e-f99b-4f72-855c-3abb28b3a865"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "eta", " ", "g2", " ", "k"}], "+", 
     RowBox[{"3", " ", "g3", " ", 
      SuperscriptBox["k", "2"]}], "+", 
     RowBox[{"g1", " ", "k", " ", "r0"}], "+", 
     RowBox[{"beta", " ", "g1", " ", 
      SuperscriptBox["r0", "2"]}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g0"}], "+", 
     RowBox[{"g1", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "eta"}], " ", "g1"}], "-", 
     RowBox[{"g2", " ", "k"}], "-", 
     RowBox[{"g0", " ", "r0"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "g0"}], " ", "r0", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}], "+", 
     RowBox[{"k", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"eta", " ", "g1"}], "+", 
        RowBox[{"2", " ", "g2", " ", "k"}], "+", 
        RowBox[{"g0", " ", "r0"}]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.777691938136962*^9},
 CellLabel->
  "Out[196]=",ExpressionUUID->"d0941fe1-b499-432e-8bcb-1abf0179423f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Expand", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.777691945629479*^9, 3.7776919473334627`*^9}},
 CellLabel->
  "In[197]:=",ExpressionUUID->"fc972c3a-4923-43dd-9cba-62e600961b00"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["eta", "2"]}], " ", 
   SuperscriptBox["g1", "2"], " ", "k"}], "+", 
  RowBox[{"2", " ", 
   SuperscriptBox["eta", "2"], " ", "g0", " ", "g2", " ", "k"}], "-", 
  RowBox[{"eta", " ", "g1", " ", "g2", " ", 
   SuperscriptBox["k", "2"]}], "+", 
  RowBox[{"3", " ", "eta", " ", "g0", " ", "g3", " ", 
   SuperscriptBox["k", "2"]}], "-", 
  RowBox[{"2", " ", 
   SuperscriptBox["g2", "2"], " ", 
   SuperscriptBox["k", "3"]}], "+", 
  RowBox[{"3", " ", "g1", " ", "g3", " ", 
   SuperscriptBox["k", "3"]}], "-", 
  RowBox[{
  "2", " ", "beta", " ", "eta", " ", "g1", " ", "g2", " ", "k", " ", "r0"}], 
  "+", 
  RowBox[{
   SuperscriptBox["g1", "2"], " ", 
   SuperscriptBox["k", "2"], " ", "r0"}], "-", 
  RowBox[{"2", " ", "g0", " ", "g2", " ", 
   SuperscriptBox["k", "2"], " ", "r0"}], "-", 
  RowBox[{"3", " ", "beta", " ", "g1", " ", "g3", " ", 
   SuperscriptBox["k", "2"], " ", "r0"}], "-", 
  RowBox[{"beta", " ", "g0", " ", "g2", " ", "k", " ", 
   SuperscriptBox["r0", "2"]}], "-", 
  RowBox[{"beta", " ", 
   SuperscriptBox["g0", "2"], " ", 
   SuperscriptBox["r0", "3"]}], "-", 
  RowBox[{
   SuperscriptBox["beta", "2"], " ", 
   SuperscriptBox["g1", "2"], " ", 
   SuperscriptBox["r0", "3"]}]}]], "Output",
 CellChangeTimes->{3.777691947616782*^9},
 CellLabel->
  "Out[197]=",ExpressionUUID->"665635f9-57a1-4bbe-9e66-f44dd8a6b0e7"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.7776919494293013`*^9, 3.777691951549428*^9}},
 CellLabel->
  "In[198]:=",ExpressionUUID->"e17b271c-d958-4592-b36a-31565452ebd2"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["eta", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["g1", "2"], "-", 
     RowBox[{"2", " ", "g0", " ", "g2"}]}], ")"}], " ", "k"}], "-", 
  RowBox[{"2", " ", 
   SuperscriptBox["g2", "2"], " ", 
   SuperscriptBox["k", "3"]}], "+", 
  RowBox[{"3", " ", "g1", " ", "g3", " ", 
   SuperscriptBox["k", "3"]}], "+", 
  RowBox[{
   SuperscriptBox["g1", "2"], " ", 
   SuperscriptBox["k", "2"], " ", "r0"}], "-", 
  RowBox[{"3", " ", "beta", " ", "g1", " ", "g3", " ", 
   SuperscriptBox["k", "2"], " ", "r0"}], "-", 
  RowBox[{"beta", " ", 
   SuperscriptBox["g0", "2"], " ", 
   SuperscriptBox["r0", "3"]}], "-", 
  RowBox[{
   SuperscriptBox["beta", "2"], " ", 
   SuperscriptBox["g1", "2"], " ", 
   SuperscriptBox["r0", "3"]}], "-", 
  RowBox[{"g0", " ", "g2", " ", "k", " ", "r0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "k"}], "+", 
     RowBox[{"beta", " ", "r0"}]}], ")"}]}], "-", 
  RowBox[{"eta", " ", "k", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "3"}], " ", "g0", " ", "g3", " ", "k"}], "+", 
     RowBox[{"g1", " ", "g2", " ", 
      RowBox[{"(", 
       RowBox[{"k", "+", 
        RowBox[{"2", " ", "beta", " ", "r0"}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.7776919519066973`*^9},
 CellLabel->
  "Out[198]=",ExpressionUUID->"615dbd83-84ce-4244-bb96-782cfadc0d23"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["eta", "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["g1", "2"], "-", 
       RowBox[{"2", " ", "g0", " ", "g2"}]}], ")"}], " ", "k"}], "-", 
    RowBox[{"2", " ", 
     SuperscriptBox["g2", "2"], " ", 
     SuperscriptBox["k", "3"]}], "+", 
    RowBox[{"3", " ", "g1", " ", "g3", " ", 
     SuperscriptBox["k", "3"]}], "+", 
    RowBox[{
     SuperscriptBox["g1", "2"], " ", 
     SuperscriptBox["k", "2"], " ", "r0"}], "-", 
    RowBox[{"3", " ", "beta", " ", "g1", " ", "g3", " ", 
     SuperscriptBox["k", "2"], " ", "r0"}], "-", 
    RowBox[{"beta", " ", 
     SuperscriptBox["g0", "2"], " ", 
     SuperscriptBox["r0", "3"]}], "-", 
    RowBox[{
     SuperscriptBox["beta", "2"], " ", 
     SuperscriptBox["g1", "2"], " ", 
     SuperscriptBox["r0", "3"]}], "-", 
    RowBox[{"g0", " ", "g2", " ", "k", " ", "r0", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", " ", "k"}], "+", 
       RowBox[{"beta", " ", "r0"}]}], ")"}]}], "-", 
    RowBox[{"eta", " ", "k", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "3"}], " ", "g0", " ", "g3", " ", "k"}], "+", 
       RowBox[{"g1", " ", "g2", " ", 
        RowBox[{"(", 
         RowBox[{"k", "+", 
          RowBox[{"2", " ", "beta", " ", "r0"}]}], ")"}]}]}], ")"}]}]}], ",", 
   
   RowBox[{"{", 
    RowBox[{"k", ",", "eta", ",", "r0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.777691978692726*^9, 3.777691984901225*^9}},
 CellLabel->
  "In[199]:=",ExpressionUUID->"ce792024-a8f3-4b39-bfc2-c2622c5dc987"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "2"}], " ", 
      SuperscriptBox["g2", "2"]}], "+", 
     RowBox[{"3", " ", "g1", " ", "g3"}]}], ")"}], " ", 
   SuperscriptBox["k", "3"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "beta"}], " ", 
      SuperscriptBox["g0", "2"]}], "-", 
     RowBox[{
      SuperscriptBox["beta", "2"], " ", 
      SuperscriptBox["g1", "2"]}]}], ")"}], " ", 
   SuperscriptBox["r0", "3"]}], "+", 
  RowBox[{
   SuperscriptBox["k", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "g1"}], " ", "g2"}], "+", 
        RowBox[{"3", " ", "g0", " ", "g3"}]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g1", "2"], "-", 
        RowBox[{"2", " ", "g0", " ", "g2"}], "-", 
        RowBox[{"3", " ", "beta", " ", "g1", " ", "g3"}]}], ")"}], " ", 
      "r0"}]}], ")"}]}], "+", 
  RowBox[{"k", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["eta", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["g1", "2"]}], "+", 
        RowBox[{"2", " ", "g0", " ", "g2"}]}], ")"}]}], "-", 
     RowBox[{"2", " ", "beta", " ", "eta", " ", "g1", " ", "g2", " ", "r0"}], 
     "-", 
     RowBox[{"beta", " ", "g0", " ", "g2", " ", 
      SuperscriptBox["r0", "2"]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.777691985237521*^9},
 CellLabel->
  "Out[199]=",ExpressionUUID->"ed4cd0bc-3a06-4e07-a0fb-eaca39e0af4a"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "g1", " ", "g3"}], "-", 
     RowBox[{"2", " ", 
      SuperscriptBox["g2", "2"]}]}], ")"}], " ", 
   SuperscriptBox["k", "3"]}], "+", " ", 
  RowBox[{
   SuperscriptBox["k", "2"], "eta", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "g0", " ", "g3"}], "-", 
     RowBox[{"g1", " ", "g2"}]}], ")"}]}], "+", 
  RowBox[{"beta", " ", 
   SuperscriptBox["k", "2"], 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["g2", "2"], "-", 
     RowBox[{"3", "  ", "g1", " ", "g3"}]}], ")"}], "r0"}], "-", 
  RowBox[{"beta", " ", 
   SuperscriptBox["r0", "3"]}], "-", 
  RowBox[{"k", " ", "g2", " ", "beta", 
   RowBox[{"(", " ", 
    RowBox[{
     RowBox[{
      SuperscriptBox["eta", "2"], " ", "g2"}], "+", 
     RowBox[{"2", "  ", "eta", " ", "g1", " ", "r0"}], "+", " ", 
     RowBox[{"g0", " ", 
      SuperscriptBox["r0", "2"]}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.777692054458645*^9, 3.777692213543558*^9}, {
  3.7776922519299583`*^9, 
  3.777692332330082*^9}},ExpressionUUID->"211f4346-5eec-4e40-85a0-\
57a5260796e1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"g1", "*", "r"}], "-", 
    RowBox[{"g2", "*", "c2"}]}], " ", "/.", " ", 
   RowBox[{"r", " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
     RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
     RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
  RowBox[{"zeta", " ", "\[Rule]", " ", 
   RowBox[{"k", " ", "-", " ", 
    RowBox[{"beta", " ", "r0"}]}]}]}]], "Input",
 CellChangeTimes->{{3.777692682762106*^9, 3.777692714511804*^9}, {
  3.7776927932208977`*^9, 3.77769279327353*^9}},
 CellLabel->"In[26]:=",ExpressionUUID->"d8a22895-ddf5-442c-b4c4-275c57c4ab99"],

Cell[BoxData[
 RowBox[{
  RowBox[{"g1", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], "+", 
     RowBox[{"g2", " ", "k"}], "+", 
     RowBox[{"g0", " ", "r0"}]}], ")"}]}], "-", 
  RowBox[{"g2", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g0"}], "+", 
     RowBox[{"g1", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{{3.777692696540688*^9, 3.777692714848205*^9}, 
   3.777692794413496*^9, 3.777693341127325*^9},
 CellLabel->"Out[26]=",ExpressionUUID->"175381db-2169-4bd1-ba37-100030d6f3b3"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"eta", ",", "k", ",", "r0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.777692719990939*^9, 3.777692725462515*^9}},
 CellLabel->"In[27]:=",ExpressionUUID->"a498c6a4-63d8-43f1-aca6-58b39dcce549"],

Cell[BoxData[
 RowBox[{
  RowBox[{"eta", " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["g1", "2"], "-", 
     RowBox[{"g0", " ", "g2"}]}], ")"}]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g0", " ", "g1"}], "+", 
     RowBox[{"beta", " ", "g1", " ", "g2"}]}], ")"}], " ", "r0"}]}]], "Output",\

 CellChangeTimes->{3.777693342491021*^9},
 CellLabel->"Out[27]=",ExpressionUUID->"104f5b5a-05a9-48f6-88e4-8a2c5aeac77f"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"eta", " ", "g2"}], "+", 
  RowBox[{"g1", " ", "r0"}]}]], "Input",
 CellChangeTimes->{{3.777692802298293*^9, 
  3.7776928283932457`*^9}},ExpressionUUID->"8f680817-27a8-4159-97f3-\
a4b6451a334c"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.777692747305538*^9, 3.777692754249116*^9}, 
   3.777692807544915*^9},ExpressionUUID->"feb7a859-b25a-42d9-8b72-\
7533fad5596e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"g0", "*", "r"}], "-", 
    RowBox[{"g1", "*", "c2"}]}], " ", "/.", " ", 
   RowBox[{"r", " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
     RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
     RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
  RowBox[{"zeta", " ", "\[Rule]", " ", 
   RowBox[{"k", " ", "-", " ", 
    RowBox[{"beta", " ", "r0"}]}]}]}]], "Input",
 CellChangeTimes->{{3.777693512901681*^9, 3.7776935148205767`*^9}},
 CellLabel->"In[28]:=",ExpressionUUID->"4acf9634-22ef-48d4-ac60-9d9d952b54cd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"g0", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], "+", 
     RowBox[{"g2", " ", "k"}], "+", 
     RowBox[{"g0", " ", "r0"}]}], ")"}]}], "-", 
  RowBox[{"g1", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g0"}], "+", 
     RowBox[{"g1", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.777693520126795*^9},
 CellLabel->"Out[28]=",ExpressionUUID->"8c68fcb8-c605-44b8-b206-8a0039276741"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"eta", ",", "k", ",", "r0"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.777693522982201*^9, 3.777693527421489*^9}},
 CellLabel->"In[29]:=",ExpressionUUID->"c5c8c83a-5af2-4cd9-8b52-76cdadcb59fc"],

Cell[BoxData[
 RowBox[{"r0", "-", 
  RowBox[{"g2", " ", "k"}]}]], "Input",
 CellChangeTimes->{{3.777693541045107*^9, 
  3.777693556339449*^9}},ExpressionUUID->"cef57c66-7f51-4aff-a8b7-\
46248b4923d4"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Check", " ", "the", " ", "order", " ", "of", " ", "the", " ", "dv0", " ", 
   RowBox[{"terms", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.7780099232965727`*^9, 3.7780099302943497`*^9}},
 CellLabel->"In[21]:=",ExpressionUUID->"9b8320e8-f2c4-446c-b832-4d14c4974f2c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"x0", " ", 
              RowBox[{"(", 
               RowBox[{"g1", "-", 
                FractionBox[
                 RowBox[{"h", " ", "c20"}], 
                 RowBox[{"beta", " ", 
                  SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
             RowBox[{"v0", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
                FractionBox["c18", "beta"], "-", 
                FractionBox[
                 RowBox[{"d", " ", "c2"}], "r"], "+", 
                FractionBox[
                 RowBox[{"2", " ", "g2", " ", "h", " ", "c2"}], "r"], "-", 
                FractionBox[
                 RowBox[{
                  SuperscriptBox["h", "2"], " ", "c3", " ", "c2"}], 
                 RowBox[{"r", " ", 
                  SuperscriptBox["r0", "3"]}]], "+", 
                FractionBox[
                 RowBox[{
                  SuperscriptBox["h", "2"], " ", "c20"}], 
                 RowBox[{"beta", " ", 
                  SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], "/.", " ", 
            RowBox[{"zeta", " ", "\[Rule]", " ", 
             RowBox[{"k", "-", 
              RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
           RowBox[{"d", " ", "\[Rule]", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"h", "+", 
               RowBox[{"eta", "*", "g2"}], "+", 
               RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}],
           " ", "/.", " ", 
          RowBox[{"h", " ", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
            RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
            RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
         RowBox[{"r", " ", "\[Rule]", " ", 
          RowBox[{
           RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
           RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
           RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "6"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.778009931719599*^9, 3.7780099789103603`*^9}, 
   3.778010037257018*^9, 3.778010687494666*^9},
 CellLabel->"In[47]:=",ExpressionUUID->"86ce98b7-7406-4b04-b880-bed88514cd67"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", "eta", " ", "k"}], "+", 
          RowBox[{"beta", " ", "eta", " ", "r0"}]}], ")"}], " ", "x0"}], 
       ")"}], " ", 
      SuperscriptBox["gamma", "2"]}], 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["beta", "2"], " ", 
        SuperscriptBox["r0", "2"]}], ")"}]}]]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "3"}], " ", 
        SuperscriptBox["eta", "2"], " ", "k"}], "-", 
       RowBox[{"6", " ", 
        SuperscriptBox["k", "2"], " ", "r0"}], "-", 
       RowBox[{"beta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}], "+", 
       RowBox[{"3", " ", 
        SuperscriptBox["beta", "2"], " ", 
        SuperscriptBox["r0", "3"]}]}], ")"}], " ", "x0", " ", 
     SuperscriptBox["gamma", "3"]}], 
    RowBox[{"6", " ", 
     SuperscriptBox["beta", 
      RowBox[{"5", "/", "2"}]], " ", 
     SuperscriptBox["r0", "3"]}]], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"3", " ", "k", " ", "v0"}], 
       RowBox[{"4", " ", 
        SuperscriptBox["beta", "2"]}]], "+", 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "16"}], " ", "eta", " ", 
           SuperscriptBox["k", "2"]}], "+", 
          RowBox[{"8", " ", "beta", " ", "eta", " ", "k", " ", "r0"}], "+", 
          RowBox[{"7", " ", 
           SuperscriptBox["beta", "2"], " ", "eta", " ", 
           SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0"}], 
       RowBox[{"24", " ", 
        SuperscriptBox["beta", "3"], " ", 
        SuperscriptBox["r0", "3"]}]]}], ")"}], " ", 
    SuperscriptBox["gamma", "4"]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"108", " ", "beta", " ", "eta", " ", "k", " ", 
        SuperscriptBox["r0", "2"], " ", "v0"}], "+", 
       RowBox[{"15", " ", "beta", " ", 
        SuperscriptBox["eta", "2"], " ", "k", " ", "x0"}], "-", 
       RowBox[{"20", " ", 
        SuperscriptBox["k", "3"], " ", "x0"}], "+", 
       RowBox[{"30", " ", "beta", " ", 
        SuperscriptBox["k", "2"], " ", "r0", " ", "x0"}], "+", 
       RowBox[{"11", " ", 
        SuperscriptBox["beta", "2"], " ", "k", " ", 
        SuperscriptBox["r0", "2"], " ", "x0"}], "-", 
       RowBox[{"15", " ", 
        SuperscriptBox["beta", "3"], " ", 
        SuperscriptBox["r0", "3"], " ", "x0"}]}], ")"}], " ", 
     SuperscriptBox["gamma", "5"]}], 
    RowBox[{"120", " ", 
     SuperscriptBox["beta", 
      RowBox[{"7", "/", "2"}]], " ", 
     SuperscriptBox["r0", "3"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"342", " ", 
        SuperscriptBox["eta", "2"], " ", "k", " ", "r0", " ", "v0"}], "+", 
       RowBox[{"68", " ", 
        SuperscriptBox["k", "2"], " ", 
        SuperscriptBox["r0", "2"], " ", "v0"}], "-", 
       RowBox[{"108", " ", "beta", " ", "k", " ", 
        SuperscriptBox["r0", "3"], " ", "v0"}], "+", 
       RowBox[{"86", " ", "eta", " ", 
        SuperscriptBox["k", "2"], " ", "x0"}], "-", 
       RowBox[{
       "32", " ", "beta", " ", "eta", " ", "k", " ", "r0", " ", "x0"}], "-", 
       RowBox[{"31", " ", 
        SuperscriptBox["beta", "2"], " ", "eta", " ", 
        SuperscriptBox["r0", "2"], " ", "x0"}]}], ")"}], " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"720", " ", 
     SuperscriptBox["beta", "3"], " ", 
     SuperscriptBox["r0", "3"]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "7"],
    SeriesData[$CellContext`gamma, 0, {}, 2, 7, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[-1, 2] $CellContext`beta^(-2) $CellContext`r0^(-2) (
     2 $CellContext`eta $CellContext`k + $CellContext`beta $CellContext`eta \
$CellContext`r0) $CellContext`x0, 
    Rational[1, 6] $CellContext`beta^
     Rational[-5, 
       2] $CellContext`r0^(-3) ((-3) $CellContext`eta^2 $CellContext`k - 
     6 $CellContext`k^2 $CellContext`r0 - $CellContext`beta $CellContext`k \
$CellContext`r0^2 + 3 $CellContext`beta^2 $CellContext`r0^3) $CellContext`x0, 
    Rational[3, 4] $CellContext`beta^(-2) $CellContext`k $CellContext`v0 + 
    Rational[1, 
       24] $CellContext`beta^(-3) $CellContext`r0^(-3) ((-16) \
$CellContext`eta $CellContext`k^2 + 
      8 $CellContext`beta $CellContext`eta $CellContext`k $CellContext`r0 + 
      7 $CellContext`beta^2 $CellContext`eta $CellContext`r0^2) \
$CellContext`x0, 
    Rational[1, 120] $CellContext`beta^Rational[-7, 2] $CellContext`r0^(-3) (
     108 $CellContext`beta $CellContext`eta $CellContext`k $CellContext`r0^2 \
$CellContext`v0 + 
     15 $CellContext`beta $CellContext`eta^2 $CellContext`k $CellContext`x0 - 
     20 $CellContext`k^3 $CellContext`x0 + 
     30 $CellContext`beta $CellContext`k^2 $CellContext`r0 $CellContext`x0 + 
     11 $CellContext`beta^2 $CellContext`k $CellContext`r0^2 $CellContext`x0 - 
     15 $CellContext`beta^3 $CellContext`r0^3 $CellContext`x0), 
    Rational[1, 720] $CellContext`beta^(-3) $CellContext`r0^(-3) (
     342 $CellContext`eta^2 $CellContext`k $CellContext`r0 $CellContext`v0 + 
     68 $CellContext`k^2 $CellContext`r0^2 $CellContext`v0 - 
     108 $CellContext`beta $CellContext`k $CellContext`r0^3 $CellContext`v0 + 
     86 $CellContext`eta $CellContext`k^2 $CellContext`x0 - 
     32 $CellContext`beta $CellContext`eta $CellContext`k $CellContext`r0 \
$CellContext`x0 - 
     31 $CellContext`beta^2 $CellContext`eta $CellContext`r0^2 \
$CellContext`x0)}, 2, 7, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.778009979345044*^9, 3.778010689021652*^9},
 CellLabel->"Out[47]=",ExpressionUUID->"7cf803f0-1bd6-464d-a759-dc2fa9b60088"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Now", " ", "focus", " ", "on", " ", "individual", " ", 
    RowBox[{"terms", ":"}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"dv0", ".", "x0"}], " ", 
    RowBox[{"term", ":"}]}], " ", "*)"}]}]], "Input",
 CellChangeTimes->{{3.778009995378441*^9, 3.7780100030222692`*^9}, {
  3.7780275130434504`*^9, 
  3.778027520559948*^9}},ExpressionUUID->"38e88ef0-d371-4d60-9f1a-\
89558da3eaa5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"x0", " ", 
             RowBox[{"(", 
              RowBox[{"g1", "-", 
               FractionBox[
                RowBox[{"h", " ", "c20"}], 
                RowBox[{"beta", " ", 
                 SuperscriptBox["r0", "3"]}]], "-", 
               FractionBox[
                RowBox[{"g2", " ", "c2"}], "r"], "+", 
               FractionBox[
                RowBox[{"h", " ", "c3", " ", "c2"}], 
                RowBox[{"r", " ", 
                 SuperscriptBox["r0", "3"]}]]}], ")"}]}], "/.", " ", 
            RowBox[{"zeta", " ", "\[Rule]", " ", 
             RowBox[{"k", "-", 
              RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
           RowBox[{"d", " ", "\[Rule]", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"h", "+", 
               RowBox[{"eta", "*", "g2"}], "+", 
               RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}],
           " ", "/.", " ", 
          RowBox[{"h", " ", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
            RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
            RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
         RowBox[{"r", " ", "\[Rule]", " ", 
          RowBox[{
           RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
           RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
           RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "6"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.778010011992378*^9, 3.778010128125351*^9}, {
  3.7780113313103943`*^9, 3.7780113323415937`*^9}},
 CellLabel->"In[56]:=",ExpressionUUID->"71147eb9-1811-43ab-9c97-da5dc2252d01"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{"k", " ", "x0"}], ")"}], " ", 
      SuperscriptBox["gamma", "3"]}], 
     RowBox[{"3", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["beta", 
         RowBox[{"3", "/", "2"}]], " ", "r0"}], ")"}]}]]}], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     RowBox[{"(", 
      RowBox[{"eta", " ", "k", " ", "x0"}], ")"}], " ", 
     SuperscriptBox["gamma", "4"]}], 
    RowBox[{"12", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["beta", "2"], " ", 
       SuperscriptBox["r0", "2"]}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "10"}], " ", 
        SuperscriptBox["eta", "2"], " ", "k"}], "+", 
       RowBox[{"3", " ", "beta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0", " ", 
     SuperscriptBox["gamma", "5"]}], 
    RowBox[{"60", " ", 
     SuperscriptBox["beta", 
      RowBox[{"5", "/", "2"}]], " ", 
     SuperscriptBox["r0", "3"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"15", " ", 
        SuperscriptBox["eta", "3"], " ", "k"}], "-", 
       RowBox[{"54", " ", "eta", " ", 
        SuperscriptBox["k", "2"], " ", "r0"}], "+", 
       RowBox[{"47", " ", "beta", " ", "eta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0", " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"360", " ", 
     SuperscriptBox["beta", "3"], " ", 
     SuperscriptBox["r0", "4"]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "7"],
    SeriesData[$CellContext`gamma, 0, {}, 3, 7, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[-2, 3] $CellContext`beta^
     Rational[-3, 2] $CellContext`k $CellContext`r0^(-1) $CellContext`x0, 
    Rational[-7, 
      12] $CellContext`beta^(-2) $CellContext`eta $CellContext`k \
$CellContext`r0^(-2) $CellContext`x0, 
    Rational[1, 60] $CellContext`beta^
     Rational[-5, 
       2] $CellContext`r0^(-3) ((-10) $CellContext`eta^2 $CellContext`k + 
     3 $CellContext`beta $CellContext`k $CellContext`r0^2) $CellContext`x0, 
    Rational[1, 360] $CellContext`beta^(-3) $CellContext`r0^(-4) (
     15 $CellContext`eta^3 $CellContext`k - 
     54 $CellContext`eta $CellContext`k^2 $CellContext`r0 + 
     47 $CellContext`beta $CellContext`eta $CellContext`k $CellContext`r0^2) \
$CellContext`x0}, 3, 7, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.778010015283999*^9, 3.778010128662158*^9}, 
   3.778011343353126*^9},
 CellLabel->"Out[56]=",ExpressionUUID->"07e48ec0-c825-4123-8d3f-ddf11ce3435a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"First", ",", " ", 
    RowBox[{
     RowBox[{"combine", " ", "the", " ", "two", " ", "first"}], "-", 
     RowBox[{"order", " ", "in", " ", "gamma", " ", 
      RowBox[{"terms", ":"}]}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"g1", "-", 
        FractionBox[
         RowBox[{"h", " ", "c20"}], 
         RowBox[{"beta", " ", 
          SuperscriptBox["r0", "3"]}]]}], ")"}], "/.", " ", 
      RowBox[{"zeta", " ", "\[Rule]", " ", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
     RowBox[{"d", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"h", "+", 
         RowBox[{"eta", "*", "g2"}], "+", 
         RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}], " ", 
    "/.", " ", 
    RowBox[{"h", " ", "\[Rule]", " ", 
     RowBox[{
      RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
      RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
      RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
   RowBox[{"r", " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
     RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
     RowBox[{"k", " ", "g2", " "}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.778010703667967*^9, 3.778010733253149*^9}},
 CellLabel->"In[49]:=",ExpressionUUID->"4110c55b-bdae-4210-8ad1-f6e1d5d95dff"],

Cell[BoxData[
 RowBox[{"g1", "-", 
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"eta", " ", "g2"}], "+", 
      RowBox[{"g3", " ", "k"}], "+", 
      RowBox[{"g1", " ", "r0"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "g0"}], " ", "r0", " ", 
       RowBox[{"(", 
        RowBox[{"k", "-", 
         RowBox[{"beta", " ", "r0"}]}], ")"}]}], "+", 
      RowBox[{"k", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"eta", " ", "g1"}], "+", 
         RowBox[{"2", " ", "g2", " ", "k"}], "+", 
         RowBox[{"g0", " ", "r0"}]}], ")"}]}]}], ")"}]}], 
   RowBox[{"beta", " ", 
    SuperscriptBox["r0", "3"]}]]}]], "Output",
 CellChangeTimes->{{3.77801071952759*^9, 3.778010733691368*^9}},
 CellLabel->"Out[49]=",ExpressionUUID->"808646b7-8e16-4ea6-aa1e-9f152e7a4e05"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Together", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.778010735562787*^9, 3.7780107376337147`*^9}},
 CellLabel->"In[50]:=",ExpressionUUID->"2d608bd3-89ba-4f2d-89ef-df899f9a030d"],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox["eta", "2"]}], " ", "g1", " ", "g2", " ", "k"}], "-", 
   RowBox[{"2", " ", "eta", " ", 
    SuperscriptBox["g2", "2"], " ", 
    SuperscriptBox["k", "2"]}], "-", 
   RowBox[{"eta", " ", "g1", " ", "g3", " ", 
    SuperscriptBox["k", "2"]}], "-", 
   RowBox[{"2", " ", "g2", " ", "g3", " ", 
    SuperscriptBox["k", "3"]}], "-", 
   RowBox[{"eta", " ", 
    SuperscriptBox["g1", "2"], " ", "k", " ", "r0"}], "-", 
   RowBox[{"2", " ", "g1", " ", "g2", " ", 
    SuperscriptBox["k", "2"], " ", "r0"}], "-", 
   RowBox[{"beta", " ", "eta", " ", "g0", " ", "g2", " ", 
    SuperscriptBox["r0", "2"]}], "-", 
   RowBox[{"beta", " ", "g0", " ", "g3", " ", "k", " ", 
    SuperscriptBox["r0", "2"]}], "+", 
   RowBox[{"beta", " ", "g1", " ", 
    SuperscriptBox["r0", "3"]}], "-", 
   RowBox[{"beta", " ", "g0", " ", "g1", " ", 
    SuperscriptBox["r0", "3"]}]}], 
  RowBox[{"beta", " ", 
   SuperscriptBox["r0", "3"]}]]], "Output",
 CellChangeTimes->{3.778010738160512*^9},
 CellLabel->"Out[50]=",ExpressionUUID->"e6186320-58cb-442f-891b-c65ccf64b3cc"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["eta", "2"]}], " ", "g1", " ", "g2", " ", "k"}], "-", 
    RowBox[{"2", " ", "eta", " ", 
     SuperscriptBox["g2", "2"], " ", 
     SuperscriptBox["k", "2"]}], "-", 
    RowBox[{"eta", " ", "g1", " ", "g3", " ", 
     SuperscriptBox["k", "2"]}], "-", 
    RowBox[{"2", " ", "g2", " ", "g3", " ", 
     SuperscriptBox["k", "3"]}], "-", 
    RowBox[{"eta", " ", 
     SuperscriptBox["g1", "2"], " ", "k", " ", "r0"}], "-", 
    RowBox[{"2", " ", "g1", " ", "g2", " ", 
     SuperscriptBox["k", "2"], " ", "r0"}], "-", 
    RowBox[{"beta", " ", "eta", " ", "g0", " ", "g2", " ", 
     SuperscriptBox["r0", "2"]}], "-", 
    RowBox[{"beta", " ", "g0", " ", "g3", " ", "k", " ", 
     SuperscriptBox["r0", "2"]}], "+", 
    RowBox[{"beta", " ", "g1", " ", 
     SuperscriptBox["r0", "3"]}], "-", 
    RowBox[{"beta", " ", "g0", " ", "g1", " ", 
     SuperscriptBox["r0", "3"]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"r0", ",", "eta", ",", "k"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7780107445456676`*^9, 3.7780107516810627`*^9}},
 CellLabel->"In[51]:=",ExpressionUUID->"a83389ef-650e-4248-b84d-87ed17b8d491"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["eta", "2"]}], " ", "g1", " ", "g2", " ", "k"}], "-", 
  RowBox[{"eta", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["g2", "2"]}], "+", 
     RowBox[{"g1", " ", "g3"}]}], ")"}], " ", 
   SuperscriptBox["k", "2"]}], "-", 
  RowBox[{"2", " ", "g2", " ", "g3", " ", 
   SuperscriptBox["k", "3"]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", 
      SuperscriptBox["g1", "2"]}], "+", 
     RowBox[{"2", " ", "g1", " ", "g2", " ", "k"}]}], ")"}], "k", " ", "r0"}],
   "-", 
  RowBox[{"beta", 
   RowBox[{"(", " ", 
    RowBox[{
     RowBox[{"eta", " ", "g0", " ", "g2"}], "+", " ", 
     RowBox[{"g0", " ", "g3", " ", "k"}]}], ")"}], " ", 
   SuperscriptBox["r0", "2"]}], "+", 
  RowBox[{
   SuperscriptBox["beta", "2"], " ", "g1", " ", "g2", " ", 
   SuperscriptBox["r0", "3"]}]}]], "Input",
 CellChangeTimes->{{3.7780108960988283`*^9, 
  3.778010955447941*^9}},ExpressionUUID->"8225ac50-168c-4cab-b13f-\
0ce22d867e34"],

Cell[BoxData[
 RowBox[{
  RowBox[{"beta", " ", "g1", " ", "g2"}], "-", 
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"h", "-", 
       RowBox[{"r0", " ", "g1"}]}], ")"}], "g0", " ", 
     SuperscriptBox["r0", "2"], " ", "beta"}], " ", "+", 
    RowBox[{"k", " ", "h", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", "  ", "g1"}], "+", 
       RowBox[{"2", " ", "g2", " ", "k"}]}], ")"}]}]}], 
   RowBox[{"beta", " ", 
    SuperscriptBox["r0", "3"]}]]}]], "Input",
 CellChangeTimes->{{3.7780110608116283`*^9, 
  3.778011312725452*^9}},ExpressionUUID->"38ebeebc-98f9-4b22-aa59-\
3a8206aba9b3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"x0", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"beta", " ", "g1", " ", "g2"}], "-", 
               FractionBox[
                RowBox[{
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{"h", "-", 
                    RowBox[{"r0", " ", "g1"}]}], ")"}], "g0", " ", 
                  SuperscriptBox["r0", "2"], " ", "beta"}], " ", "+", 
                 RowBox[{"k", " ", "h", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"eta", "  ", "g1"}], "+", 
                    RowBox[{"2", " ", "g2", " ", "k"}]}], ")"}]}]}], 
                RowBox[{"beta", " ", 
                 SuperscriptBox["r0", "3"]}]], "-", 
               FractionBox[
                RowBox[{"g2", " ", "c2"}], "r"], "+", 
               FractionBox[
                RowBox[{"h", " ", "c3", " ", "c2"}], 
                RowBox[{"r", " ", 
                 SuperscriptBox["r0", "3"]}]]}], ")"}]}], "/.", " ", 
            RowBox[{"zeta", " ", "\[Rule]", " ", 
             RowBox[{"k", "-", 
              RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
           RowBox[{"d", " ", "\[Rule]", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"h", "+", 
               RowBox[{"eta", "*", "g2"}], "+", 
               RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}],
           " ", "/.", " ", 
          RowBox[{"h", " ", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
            RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
            RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
         RowBox[{"r", " ", "\[Rule]", " ", 
          RowBox[{
           RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
           RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
           RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "6"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.778011353132597*^9, 3.778011379787016*^9}},
 CellLabel->"In[73]:=",ExpressionUUID->"19429d45-8e46-42c8-9bd7-ddab5ab6732b"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{"k", " ", "x0"}], ")"}], " ", 
      SuperscriptBox["gamma", "3"]}], 
     RowBox[{"3", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["beta", 
         RowBox[{"3", "/", "2"}]], " ", "r0"}], ")"}]}]]}], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     RowBox[{"(", 
      RowBox[{"eta", " ", "k", " ", "x0"}], ")"}], " ", 
     SuperscriptBox["gamma", "4"]}], 
    RowBox[{"12", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["beta", "2"], " ", 
       SuperscriptBox["r0", "2"]}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "10"}], " ", 
        SuperscriptBox["eta", "2"], " ", "k"}], "+", 
       RowBox[{"3", " ", "beta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0", " ", 
     SuperscriptBox["gamma", "5"]}], 
    RowBox[{"60", " ", 
     SuperscriptBox["beta", 
      RowBox[{"5", "/", "2"}]], " ", 
     SuperscriptBox["r0", "3"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"15", " ", 
        SuperscriptBox["eta", "3"], " ", "k"}], "-", 
       RowBox[{"54", " ", "eta", " ", 
        SuperscriptBox["k", "2"], " ", "r0"}], "+", 
       RowBox[{"47", " ", "beta", " ", "eta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0", " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"360", " ", 
     SuperscriptBox["beta", "3"], " ", 
     SuperscriptBox["r0", "4"]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "7"],
    SeriesData[$CellContext`gamma, 0, {}, 3, 7, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[-2, 3] $CellContext`beta^
     Rational[-3, 2] $CellContext`k $CellContext`r0^(-1) $CellContext`x0, 
    Rational[-7, 
      12] $CellContext`beta^(-2) $CellContext`eta $CellContext`k \
$CellContext`r0^(-2) $CellContext`x0, 
    Rational[1, 60] $CellContext`beta^
     Rational[-5, 
       2] $CellContext`r0^(-3) ((-10) $CellContext`eta^2 $CellContext`k + 
     3 $CellContext`beta $CellContext`k $CellContext`r0^2) $CellContext`x0, 
    Rational[1, 360] $CellContext`beta^(-3) $CellContext`r0^(-4) (
     15 $CellContext`eta^3 $CellContext`k - 
     54 $CellContext`eta $CellContext`k^2 $CellContext`r0 + 
     47 $CellContext`beta $CellContext`eta $CellContext`k $CellContext`r0^2) \
$CellContext`x0}, 3, 7, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.778011364305004*^9, 3.778012139912447*^9},
 CellLabel->"Out[73]=",ExpressionUUID->"c002ef47-5c2d-4cc5-8939-daff0aa64451"]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.778012235668507*^9, 3.778012347523027*^9}, {
   3.778012400699394*^9, 3.778012401367897*^9}, {3.778012495196313*^9, 
   3.7780124972910137`*^9}, 
   3.778029093600687*^9},ExpressionUUID->"5ef14266-0107-46d9-8ef5-\
871eba81c558"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.778012548298581*^9, 3.778012669796537*^9}, {
   3.778012709306596*^9, 3.7780127107400427`*^9}, {3.7780127411570883`*^9, 
   3.77801294752605*^9}, {3.7780129792431507`*^9, 3.778013067904934*^9}, {
   3.7780131441226873`*^9, 3.778013212168106*^9}, {3.77802763110108*^9, 
   3.778027834854019*^9}, {3.7780278673250713`*^9, 3.778027923010427*^9}, 
   3.778029097671492*^9},ExpressionUUID->"cc1b2afa-2caf-43db-bbd4-\
8dab27b3d74d"],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Try", " ", 
   RowBox[{"again", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.778028053412081*^9, 3.7780280562026577`*^9}},
 CellLabel->"In[83]:=",ExpressionUUID->"554b7b6a-85ab-4746-acc6-5ff1d4f2e304"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"x0", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"beta", " ", "g1", " ", "g2"}], "+", 
               FractionBox[
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
                   RowBox[{"k", " ", "g3"}]}], ")"}], " ", "eta", " ", "g0", 
                 " ", "c3", " "}], 
                RowBox[{"r", " ", 
                 SuperscriptBox["r0", "3"]}]], "+", 
               FractionBox[
                RowBox[{" ", 
                 RowBox[{"g1", " ", "g0", " ", 
                  RowBox[{"(", " ", 
                   RowBox[{
                    RowBox[{"2", "k", "  ", 
                    SuperscriptBox["eta", "2"], " ", "g2"}], " ", "+", " ", 
                    RowBox[{"3", " ", "eta", " ", 
                    SuperscriptBox["k", "2"], " ", "g3"}]}], ")"}], " "}]}], 
                RowBox[{"beta", " ", "r", " ", 
                 SuperscriptBox["r0", "2"]}]], "-", 
               RowBox[{
                FractionBox["k", 
                 RowBox[{"beta", " ", 
                  SuperscriptBox["r0", "3"]}]], 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"eta", "  ", "g1", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
                    RowBox[{"k", " ", "g3"}]}], ")"}]}], " ", "+", " ", 
                  RowBox[{"g3", " ", "g0", " ", 
                   SuperscriptBox["r0", "2"], " ", "beta"}], " ", "+", 
                  RowBox[{"2", " ", "h", " ", "g2", " ", "k"}]}], ")"}]}], 
               "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"g1", " ", "zeta"}], ")"}], 
                RowBox[{"(", 
                 RowBox[{
                  FractionBox[
                   RowBox[{"h", " ", "c3", " "}], 
                   RowBox[{"r", " ", 
                    SuperscriptBox["r0", "3"]}]], "-", 
                  FractionBox[
                   RowBox[{"g2", " "}], "r"]}], ")"}]}], "-", 
               FractionBox[
                RowBox[{"eta", " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"beta", " ", "g2", " ", "g0", " ", "r0"}], " ", 
                   "+", 
                   RowBox[{"k", " ", 
                    SuperscriptBox["g1", "2"]}]}], ")"}], 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
                   RowBox[{"k", " ", "g2"}]}], " ", ")"}]}], 
                RowBox[{"beta", " ", "r", " ", 
                 SuperscriptBox["r0", "2"]}]]}], " ", ")"}]}], "/.", " ", 
            RowBox[{"zeta", " ", "\[Rule]", " ", 
             RowBox[{"k", "-", 
              RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
           RowBox[{"d", " ", "\[Rule]", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"h", "+", 
               RowBox[{"eta", "*", "g2"}], "+", 
               RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}],
           " ", "/.", " ", 
          RowBox[{"h", " ", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
            RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
            RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
         RowBox[{"r", " ", "\[Rule]", " ", 
          RowBox[{
           RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
           RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
           RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "6"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.7780280932838287`*^9, 3.7780282052638683`*^9}, {
  3.7780282538074713`*^9, 3.778028903707653*^9}, {3.7780289473642263`*^9, 
  3.778028993844963*^9}, {3.778029079891124*^9, 3.778029328109026*^9}, {
  3.778029365132901*^9, 3.778029372634542*^9}, {3.778029430810363*^9, 
  3.778029653294387*^9}, {3.778029741519649*^9, 3.778030141362014*^9}, {
  3.778030172337906*^9, 3.778030254793277*^9}, {3.778030334259881*^9, 
  3.778030398829008*^9}, {3.7780304932020187`*^9, 3.778030494128862*^9}},
 CellLabel->
  "In[143]:=",ExpressionUUID->"ce5dfe0a-4173-4e35-a882-4c0137d14c26"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{"k", " ", "x0"}], ")"}], " ", 
      SuperscriptBox["gamma", "3"]}], 
     RowBox[{"3", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["beta", 
         RowBox[{"3", "/", "2"}]], " ", "r0"}], ")"}]}]]}], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     RowBox[{"(", 
      RowBox[{"eta", " ", "k", " ", "x0"}], ")"}], " ", 
     SuperscriptBox["gamma", "4"]}], 
    RowBox[{"12", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["beta", "2"], " ", 
       SuperscriptBox["r0", "2"]}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "10"}], " ", 
        SuperscriptBox["eta", "2"], " ", "k"}], "+", 
       RowBox[{"3", " ", "beta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0", " ", 
     SuperscriptBox["gamma", "5"]}], 
    RowBox[{"60", " ", 
     SuperscriptBox["beta", 
      RowBox[{"5", "/", "2"}]], " ", 
     SuperscriptBox["r0", "3"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"15", " ", 
        SuperscriptBox["eta", "3"], " ", "k"}], "-", 
       RowBox[{"54", " ", "eta", " ", 
        SuperscriptBox["k", "2"], " ", "r0"}], "+", 
       RowBox[{"47", " ", "beta", " ", "eta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "x0", " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"360", " ", 
     SuperscriptBox["beta", "3"], " ", 
     SuperscriptBox["r0", "4"]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "7"],
    SeriesData[$CellContext`gamma, 0, {}, 3, 7, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[-2, 3] $CellContext`beta^
     Rational[-3, 2] $CellContext`k $CellContext`r0^(-1) $CellContext`x0, 
    Rational[-7, 
      12] $CellContext`beta^(-2) $CellContext`eta $CellContext`k \
$CellContext`r0^(-2) $CellContext`x0, 
    Rational[1, 60] $CellContext`beta^
     Rational[-5, 
       2] $CellContext`r0^(-3) ((-10) $CellContext`eta^2 $CellContext`k + 
     3 $CellContext`beta $CellContext`k $CellContext`r0^2) $CellContext`x0, 
    Rational[1, 360] $CellContext`beta^(-3) $CellContext`r0^(-4) (
     15 $CellContext`eta^3 $CellContext`k - 
     54 $CellContext`eta $CellContext`k^2 $CellContext`r0 + 
     47 $CellContext`beta $CellContext`eta $CellContext`k $CellContext`r0^2) \
$CellContext`x0}, 3, 7, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.778029081242894*^9, 3.778029200376259*^9}, {
   3.778029248115961*^9, 3.778029258460003*^9}, {3.778029291395472*^9, 
   3.77802932914227*^9}, 3.778029373679356*^9, {3.778029433673102*^9, 
   3.778029442410746*^9}, 3.7780294740147448`*^9, 3.778029538523397*^9, 
   3.778029583188656*^9, 3.7780297206002817`*^9, {3.778029761102746*^9, 
   3.778029802029951*^9}, {3.7780298338728037`*^9, 3.778029873299719*^9}, {
   3.778029905169917*^9, 3.778029918119506*^9}, {3.778029969768033*^9, 
   3.778030062966955*^9}, {3.7780301012236156`*^9, 3.778030142297687*^9}, {
   3.7780301808650427`*^9, 3.778030209812645*^9}, {3.778030242705825*^9, 
   3.778030255241786*^9}, {3.778030339068736*^9, 3.778030415377246*^9}, 
   3.778030494595193*^9, 3.7780310920146227`*^9},
 CellLabel->
  "Out[143]=",ExpressionUUID->"d92883a1-814f-41a1-bbc1-99b0b7d461e0"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Now", " ", "expand", " ", 
   RowBox[{"dv0", ".", "v0"}], " ", 
   RowBox[{"term", ":"}]}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.7780311028035803`*^9, 3.7780311088414927`*^9}},
 CellLabel->
  "In[144]:=",ExpressionUUID->"09f4ea65-efdd-43db-8197-fc1a3f2aa7ff"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"v0", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
               FractionBox["c18", "beta"], "-", 
               FractionBox[
                RowBox[{"d", " ", "c2"}], "r"], "+", 
               FractionBox[
                RowBox[{"2", " ", "g2", " ", "h", " ", "c2"}], "r"], "-", 
               FractionBox[
                RowBox[{
                 SuperscriptBox["h", "2"], " ", "c3", " ", "c2"}], 
                RowBox[{"r", " ", 
                 SuperscriptBox["r0", "3"]}]], "+", 
               FractionBox[
                RowBox[{
                 SuperscriptBox["h", "2"], " ", "c20"}], 
                RowBox[{"beta", " ", 
                 SuperscriptBox["r0", "3"]}]]}], ")"}]}], "/.", " ", 
            RowBox[{"zeta", " ", "\[Rule]", " ", 
             RowBox[{"k", "-", 
              RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
           RowBox[{"d", " ", "\[Rule]", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"h", "+", 
               RowBox[{"eta", "*", "g2"}], "+", 
               RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}],
           " ", "/.", " ", 
          RowBox[{"h", " ", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
            RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
            RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
         RowBox[{"r", " ", "\[Rule]", " ", 
          RowBox[{
           RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
           RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
           RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "7"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.778031113594091*^9, 3.778031137426024*^9}},
 CellLabel->
  "In[147]:=",ExpressionUUID->"af329d1b-7fe8-45ec-9e77-0081c2432267"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox[
    RowBox[{"3", " ", "k", " ", "v0", " ", 
     SuperscriptBox["gamma", "4"]}], 
    RowBox[{"4", " ", 
     SuperscriptBox["beta", "2"]}]], "+", 
   FractionBox[
    RowBox[{"9", " ", "eta", " ", "k", " ", "v0", " ", 
     SuperscriptBox["gamma", "5"]}], 
    RowBox[{"10", " ", 
     SuperscriptBox["beta", 
      RowBox[{"5", "/", "2"}]], " ", "r0"}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"171", " ", 
        SuperscriptBox["eta", "2"], " ", "k"}], "+", 
       RowBox[{"34", " ", 
        SuperscriptBox["k", "2"], " ", "r0"}], "-", 
       RowBox[{"54", " ", "beta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "v0", " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"360", " ", 
     SuperscriptBox["beta", "3"], " ", 
     SuperscriptBox["r0", "2"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"63", " ", 
        SuperscriptBox["eta", "3"], " ", "k"}], "+", 
       RowBox[{"686", " ", "eta", " ", 
        SuperscriptBox["k", "2"], " ", "r0"}], "-", 
       RowBox[{"747", " ", "beta", " ", "eta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "v0", " ", 
     SuperscriptBox["gamma", "7"]}], 
    RowBox[{"2520", " ", 
     SuperscriptBox["beta", 
      RowBox[{"7", "/", "2"}]], " ", 
     SuperscriptBox["r0", "3"]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "8"],
    SeriesData[$CellContext`gamma, 0, {}, 4, 8, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[3, 4] $CellContext`beta^(-2) $CellContext`k $CellContext`v0, 
    Rational[9, 10] $CellContext`beta^
     Rational[-5, 
       2] $CellContext`eta $CellContext`k $CellContext`r0^(-1) \
$CellContext`v0, 
    Rational[1, 360] $CellContext`beta^(-3) $CellContext`r0^(-2) (
     171 $CellContext`eta^2 $CellContext`k + 
     34 $CellContext`k^2 $CellContext`r0 - 
     54 $CellContext`beta $CellContext`k $CellContext`r0^2) $CellContext`v0, 
    Rational[1, 2520] $CellContext`beta^Rational[-7, 2] $CellContext`r0^(-3) (
     63 $CellContext`eta^3 $CellContext`k + 
     686 $CellContext`eta $CellContext`k^2 $CellContext`r0 - 
     747 $CellContext`beta $CellContext`eta $CellContext`k $CellContext`r0^2) \
$CellContext`v0}, 4, 8, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.778031126348537*^9, 3.7780311378744917`*^9}},
 CellLabel->
  "Out[147]=",ExpressionUUID->"dc766367-d0b9-4d04-9d9d-4b309ba73938"]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.778175396930953*^9, 
  3.778175405608468*^9}},ExpressionUUID->"137ae34a-5865-42be-addb-\
dc0a7bc2dbb3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"-", "v0"}], 
              RowBox[{"(", 
               RowBox[{
                RowBox[{
                 FractionBox["k", 
                  RowBox[{"beta", " ", "r"}]], 
                 RowBox[{"(", " ", 
                  RowBox[{
                   RowBox[{"eta", " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"2", "g0", " ", "g3"}], "-", 
                    RowBox[{"g1", " ", "g2"}], "+", "g3"}], ")"}]}], " ", "-", 
                   RowBox[{"h6", " ", "k"}], "+", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["g2", "2"], "-", 
                    RowBox[{"2", " ", "g1", " ", "g3"}]}], ")"}], " ", "beta",
                     " ", "r0"}]}], ")"}]}], "+", 
                RowBox[{"h", 
                 RowBox[{"(", " ", 
                  RowBox[{"g1", " ", "-", 
                   FractionBox[
                    RowBox[{" ", 
                    RowBox[{"g2", "  ", "c2"}]}], "r"], "+", 
                   FractionBox[
                    RowBox[{"h", " ", "c3", " ", "c2"}], 
                    RowBox[{"r", " ", 
                    SuperscriptBox["r0", "3"]}]], "-", 
                   FractionBox[
                    RowBox[{"h", " ", "c20"}], 
                    RowBox[{"beta", " ", 
                    SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}], "/.", 
             " ", 
             RowBox[{"h6", " ", "\[Rule]", " ", 
              RowBox[{
               RowBox[{"2", " ", 
                SuperscriptBox["g2", "2"]}], "-", 
               RowBox[{"3", " ", "g1", " ", "g3"}]}]}]}], "/.", " ", 
            RowBox[{"zeta", " ", "\[Rule]", " ", 
             RowBox[{"k", "-", 
              RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
           RowBox[{"d", " ", "\[Rule]", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"h", "+", 
               RowBox[{"eta", "*", "g2"}], "+", 
               RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}],
           " ", "/.", " ", 
          RowBox[{"h", " ", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
            RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
            RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
         RowBox[{"r", " ", "\[Rule]", " ", 
          RowBox[{
           RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
           RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
           RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "7"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.7780311653781567`*^9, 3.778031233578576*^9}, {
   3.7780313104409227`*^9, 3.77803153685684*^9}, {3.7780315752732286`*^9, 
   3.778031672996911*^9}, {3.7780319190746403`*^9, 3.778031964420319*^9}, {
   3.778032705408045*^9, 3.778032729582644*^9}, {3.778032834108721*^9, 
   3.7780328441217413`*^9}, {3.778033010904913*^9, 3.7780330192709837`*^9}, 
   3.7781753529678907`*^9},
 CellLabel->"",ExpressionUUID->"5b44f9ae-e77f-418d-88ab-e085887ab019"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox[
    RowBox[{"3", " ", "k", " ", "v0", " ", 
     SuperscriptBox["gamma", "4"]}], 
    RowBox[{"4", " ", 
     SuperscriptBox["beta", "2"]}]], "+", 
   FractionBox[
    RowBox[{"9", " ", "eta", " ", "k", " ", "v0", " ", 
     SuperscriptBox["gamma", "5"]}], 
    RowBox[{"10", " ", 
     SuperscriptBox["beta", 
      RowBox[{"5", "/", "2"}]], " ", "r0"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "171"}], " ", 
          SuperscriptBox["eta", "2"], " ", "k"}], "-", 
         RowBox[{"34", " ", 
          SuperscriptBox["k", "2"], " ", "r0"}], "+", 
         RowBox[{"54", " ", "beta", " ", "k", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}], " ", "v0"}], ")"}], " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"360", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["beta", "3"], " ", 
       SuperscriptBox["r0", "2"]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "63"}], " ", 
          SuperscriptBox["eta", "3"], " ", "k"}], "-", 
         RowBox[{"686", " ", "eta", " ", 
          SuperscriptBox["k", "2"], " ", "r0"}], "+", 
         RowBox[{"747", " ", "beta", " ", "eta", " ", "k", " ", 
          SuperscriptBox["r0", "2"]}]}], ")"}], " ", "v0"}], ")"}], " ", 
     SuperscriptBox["gamma", "7"]}], 
    RowBox[{"2520", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["beta", 
        RowBox[{"7", "/", "2"}]], " ", 
       SuperscriptBox["r0", "3"]}], ")"}]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "8"],
    SeriesData[$CellContext`gamma, 0, {}, 4, 8, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 
   0, {((Rational[
       3, 4] $CellContext`beta^(-2)) $CellContext`k) $CellContext`v0, ((((
        Rational[9, 10] $CellContext`beta^
         Rational[-5, 
           2]) $CellContext`eta) $CellContext`k)/$CellContext`r0) \
$CellContext`v0, (((
       Rational[-1, 
         360] $CellContext`beta^(-3)) $CellContext`r0^(-2)) (((-171) \
$CellContext`eta^2) $CellContext`k - (
       34 $CellContext`k^2) $CellContext`r0 + ((
         54 $CellContext`beta) $CellContext`k) $CellContext`r0^2)) \
$CellContext`v0, (((
       Rational[-1, 2520] $CellContext`beta^
        Rational[-7, 
          2]) $CellContext`r0^(-3)) (((-63) $CellContext`eta^3) \
$CellContext`k - ((
        686 $CellContext`eta) $CellContext`k^2) $CellContext`r0 + (((
          747 $CellContext`beta) $CellContext`eta) $CellContext`k) \
$CellContext`r0^2)) $CellContext`v0}, 4, 8, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.7780311686488743`*^9, 3.7780312059082813`*^9}, {
   3.77803130729702*^9, 3.778031314911496*^9}, {3.778031350965007*^9, 
   3.778031358798779*^9}, {3.778031422762061*^9, 3.778031488151025*^9}, {
   3.778031613861537*^9, 3.778031636508378*^9}, 3.778031674053792*^9, 
   3.778031775995742*^9, {3.7780319245264378`*^9, 3.778031965480562*^9}, {
   3.77803270672087*^9, 3.778032730425826*^9}, 3.778032845915766*^9},
 CellLabel->
  "Out[186]=",ExpressionUUID->"49023a46-ad24-4ea0-b51d-abc0ffd6ecab"]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{
  3.778031244770913*^9},ExpressionUUID->"55a33ee1-48c5-4665-9fb5-\
f52a9bd700a2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
       RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
       RowBox[{"k", " ", "g2"}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
       RowBox[{"2", " ", "g2", " ", "k"}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"eta", " ", "g0"}], "+", 
       RowBox[{"g1", " ", "zeta"}]}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"g0", " ", "r0", " ", "g1"}], " ", "+", " ", 
       RowBox[{"g0", " ", "eta", " ", "g2"}], " ", "+", " ", 
       RowBox[{"g0", " ", "k", " ", "g3"}], " ", "+", "  ", 
       RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
       RowBox[{"2", " ", "k", " ", "g3"}]}], " ", ")"}]}], "-", 
    RowBox[{"beta", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
       RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
       RowBox[{"k", " ", "g2"}]}], ")"}], " ", "g1", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
       RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
       RowBox[{"k", " ", "g3"}]}], ")"}]}]}], " ", "/.", " ", 
   RowBox[{"zeta", " ", "\[Rule]", " ", 
    RowBox[{"k", "-", 
     RowBox[{"beta", " ", "r0"}]}]}]}]}]], "Input",
 CellChangeTimes->{{3.778031699254187*^9, 3.778031704037156*^9}, {
  3.778031976702099*^9, 3.778031984180716*^9}},
 CellLabel->
  "In[177]:=",ExpressionUUID->"3d85475b-7977-4c44-a55c-59d19fe2374d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], "+", 
     RowBox[{"2", " ", "g2", " ", "k"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], "+", 
     RowBox[{"g2", " ", "k"}], "+", 
     RowBox[{"g0", " ", "r0"}]}], ")"}]}], "-", 
  RowBox[{"beta", " ", "g1", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g1"}], "+", 
     RowBox[{"g2", " ", "k"}], "+", 
     RowBox[{"g0", " ", "r0"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g2"}], "+", 
     RowBox[{"g3", " ", "k"}], "+", 
     RowBox[{"g1", " ", "r0"}]}], ")"}]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g2"}], "+", 
     RowBox[{"eta", " ", "g0", " ", "g2"}], "+", 
     RowBox[{"2", " ", "g3", " ", "k"}], "+", 
     RowBox[{"g0", " ", "g3", " ", "k"}], "+", 
     RowBox[{"g0", " ", "g1", " ", "r0"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", "g0"}], "+", 
     RowBox[{"g1", " ", 
      RowBox[{"(", 
       RowBox[{"k", "-", 
        RowBox[{"beta", " ", "r0"}]}], ")"}]}]}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.778031984897933*^9},
 CellLabel->
  "Out[177]=",ExpressionUUID->"b5e6b6f4-309b-42a9-a8a2-539f8c0e0b4a"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect", "[", 
  RowBox[{"%", ",", 
   RowBox[{"{", 
    RowBox[{"r0", ",", "eta", ",", "k"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.778031711255645*^9, 3.778031716733096*^9}},
 CellLabel->
  "In[178]:=",ExpressionUUID->"e8c5d08b-7695-4e95-9f78-c15d7daf90eb"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox["eta", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["g1", "2"], "-", 
     RowBox[{"g0", " ", "g2"}], "-", 
     RowBox[{
      SuperscriptBox["g0", "2"], " ", "g2"}], "-", 
     RowBox[{"beta", " ", 
      SuperscriptBox["g1", "2"], " ", "g2"}]}], ")"}]}], "+", 
  RowBox[{"eta", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", "g1", " ", "g2"}], "-", 
     RowBox[{"g0", " ", "g1", " ", "g2"}], "-", 
     RowBox[{"beta", " ", "g1", " ", 
      SuperscriptBox["g2", "2"]}], "-", 
     RowBox[{"2", " ", "g0", " ", "g3"}], "-", 
     RowBox[{
      SuperscriptBox["g0", "2"], " ", "g3"}], "-", 
     RowBox[{"beta", " ", 
      SuperscriptBox["g1", "2"], " ", "g3"}]}], ")"}], " ", "k"}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["g2", "2"]}], "-", 
     RowBox[{"2", " ", "g1", " ", "g3"}], "-", 
     RowBox[{"g0", " ", "g1", " ", "g3"}], "-", 
     RowBox[{"beta", " ", "g1", " ", "g2", " ", "g3"}]}], ")"}], " ", 
   SuperscriptBox["k", "2"]}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"eta", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"g0", " ", "g1"}], "-", 
        RowBox[{
         SuperscriptBox["g0", "2"], " ", "g1"}], "-", 
        RowBox[{"beta", " ", 
         SuperscriptBox["g1", "3"]}], "+", 
        RowBox[{"beta", " ", "g1", " ", "g2"}]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "g0"}], " ", 
         SuperscriptBox["g1", "2"]}], "+", 
        RowBox[{"2", " ", "g0", " ", "g2"}], "-", 
        RowBox[{"beta", " ", 
         SuperscriptBox["g1", "2"], " ", "g2"}], "+", 
        RowBox[{"2", " ", "beta", " ", "g1", " ", "g3"}]}], ")"}], " ", 
      "k"}]}], ")"}], " ", "r0"}]}]], "Output",
 CellChangeTimes->{3.7780319934803123`*^9},
 CellLabel->
  "Out[178]=",ExpressionUUID->"a3a6b14d-2fa5-4b24-bdad-9738fcb00af1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.778032000024354*^9, 3.778032001241584*^9}},
 CellLabel->
  "In[179]:=",ExpressionUUID->"b5697605-47e3-4916-9031-ba16eb752f85"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["eta", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g0", " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", "g0"}], ")"}], " ", "g2"}], "+", 
     RowBox[{
      SuperscriptBox["g1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"beta", " ", "g2"}]}], ")"}]}]}], ")"}]}], "-", 
  RowBox[{"eta", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g1", " ", "g2", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "2"}], "+", "g0", "+", 
        RowBox[{"beta", " ", "g2"}]}], ")"}]}], "+", 
     RowBox[{"g0", " ", 
      RowBox[{"(", 
       RowBox[{"2", "+", "g0"}], ")"}], " ", "g3"}], "+", 
     RowBox[{"beta", " ", 
      SuperscriptBox["g1", "2"], " ", "g3"}]}], ")"}], " ", "k"}], "+", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["g2", "2"]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"2", "+", "g0"}], ")"}], " ", "g1", " ", "g3"}], "-", 
     RowBox[{"beta", " ", "g1", " ", "g2", " ", "g3"}]}], ")"}], " ", 
   SuperscriptBox["k", "2"]}], "+", 
  RowBox[{"eta", " ", "g1", " ", 
   RowBox[{"(", 
    RowBox[{"g0", "-", 
     SuperscriptBox["g0", "2"], "+", 
     RowBox[{"beta", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox["g1", "2"]}], "+", "g2"}], ")"}]}]}], ")"}], " ", 
   "r0"}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g0", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["g1", "2"], "-", 
        RowBox[{"2", " ", "g2"}]}], ")"}]}], "+", 
     RowBox[{"beta", " ", "g1", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"g1", " ", "g2"}], "-", 
        RowBox[{"2", " ", "g3"}]}], ")"}]}]}], ")"}], " ", "k", " ", 
   "r0"}]}]], "Output",
 CellChangeTimes->{3.778032002153946*^9},
 CellLabel->
  "Out[179]=",ExpressionUUID->"3e370009-4be3-487a-86c9-0c62342151fd"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "eta"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"g3", " ", 
      RowBox[{"(", 
       RowBox[{"1", "+", " ", 
        RowBox[{"2", "g0"}]}], ")"}]}], "-", 
     RowBox[{"g1", " ", "g2"}]}], ")"}], " ", "k"}], "+", 
  RowBox[{"h6", " ", 
   SuperscriptBox["k", "2"]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["g2", "2"], "-", 
     RowBox[{"2", " ", "g1", " ", "g3"}]}], ")"}], " ", "beta", " ", "k", " ",
    "r0"}]}]], "Input",
 CellChangeTimes->{{3.778032021561049*^9, 3.778032100270296*^9}, {
  3.778032138903254*^9, 3.7780322279336243`*^9}, {3.778032296531288*^9, 
  3.778032341052197*^9}, {3.778032455483478*^9, 3.778032457802697*^9}, {
  3.778032524060321*^9, 3.7780325275565443`*^9}, {3.778032562641513*^9, 
  3.7780326942032957`*^9}, {3.778032770504093*^9, 
  3.77803278379887*^9}},ExpressionUUID->"8abd4998-473f-4657-9511-\
1ecdf34a1530"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"2", "*", "g0", "*", "g3"}], "-", 
         RowBox[{"g1", "*", "g2"}], "+", "g3"}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "7"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.778174258158543*^9, 3.778174283589251*^9}, {
  3.7781744071202908`*^9, 3.778174408806273*^9}},
 CellLabel->
  "In[358]:=",ExpressionUUID->"be6e0e37-6867-47cb-8c7a-c33748ab83a1"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     SuperscriptBox["gamma", "5"], 
     RowBox[{"15", " ", 
      SuperscriptBox["beta", 
       RowBox[{"3", "/", "2"}]]}]]}], "+", 
   FractionBox[
    RowBox[{"13", " ", 
     SuperscriptBox["gamma", "7"]}], 
    RowBox[{"1260", " ", 
     SuperscriptBox["beta", 
      RowBox[{"3", "/", "2"}]]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "8"],
    SeriesData[$CellContext`gamma, 0, {}, 5, 8, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[-1, 15] $CellContext`beta^Rational[-3, 2], 0, 
    Rational[13, 1260] $CellContext`beta^Rational[-3, 2]}, 5, 8, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.778174284296021*^9},
 CellLabel->
  "Out[358]=",ExpressionUUID->"c32a17fb-f279-4293-a542-bd3cef76b1a8"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "g0", " ", "g3"}], " ", "-", " ", 
      RowBox[{"g1", " ", "g2"}], " ", "+", " ", "g3"}], " ", "/.", " ", 
     RowBox[{"g0", " ", "\[Rule]", " ", 
      RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
    RowBox[{"g1", " ", "\[Rule]", " ", 
     RowBox[{"Sin", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
   RowBox[{"g2", " ", "\[Rule]", " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}]}]}], " ", "/.", " ", 
  RowBox[{"g3", " ", "\[Rule]", " ", 
   RowBox[{"(", 
    RowBox[{"gamma", "-", 
     RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.778174298947598*^9, 3.7781743277365227`*^9}},
 CellLabel->
  "In[360]:=",ExpressionUUID->"b8a76998-9b7e-4fdc-9683-15bf6d761421"],

Cell[BoxData[
 RowBox[{"gamma", "+", 
  RowBox[{"2", " ", 
   RowBox[{"Cos", "[", "gamma", "]"}], " ", 
   RowBox[{"(", 
    RowBox[{"gamma", "-", 
     RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}]}], "-", 
  RowBox[{"Sin", "[", "gamma", "]"}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", 
     RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], " ", 
   RowBox[{"Sin", "[", "gamma", "]"}]}]}]], "Output",
 CellChangeTimes->{{3.778174309121621*^9, 3.778174328146734*^9}},
 CellLabel->
  "Out[360]=",ExpressionUUID->"f1ff3851-6a2d-4204-8084-f43d089f993b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrigReduce", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.778174329306157*^9, 3.778174331449132*^9}},
 CellLabel->
  "In[361]:=",ExpressionUUID->"9ffc0832-f459-4305-b499-f4dab6244bca"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"2", " ", "gamma"}], "+", 
    RowBox[{"4", " ", "gamma", " ", 
     RowBox[{"Cos", "[", "gamma", "]"}]}], "-", 
    RowBox[{"4", " ", 
     RowBox[{"Sin", "[", "gamma", "]"}]}], "-", 
    RowBox[{"Sin", "[", 
     RowBox[{"2", " ", "gamma"}], "]"}]}], ")"}]}]], "Output",
 CellChangeTimes->{3.7781743317740583`*^9},
 CellLabel->
  "Out[361]=",ExpressionUUID->"ad5b3229-028f-4c15-8e5b-3300319debce"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"gamma", "+", 
   RowBox[{"2", " ", "gamma", " ", 
    RowBox[{"Cos", "[", "gamma", "]"}]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{"Sin", "[", "gamma", "]"}]}], "-", 
   RowBox[{
    RowBox[{"Sin", "[", "gamma", "]"}], " ", 
    RowBox[{"Cos", "[", "gamma", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.77817435319541*^9, 3.778174367905517*^9}},
 CellLabel->
  "In[362]:=",ExpressionUUID->"bd54d80c-f805-4f3a-bf44-977406504414"],

Cell[BoxData[
 RowBox[{"gamma", "+", 
  RowBox[{"2", " ", "gamma", " ", 
   RowBox[{"Cos", "[", "gamma", "]"}]}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"Sin", "[", "gamma", "]"}]}], "-", 
  RowBox[{
   RowBox[{"Cos", "[", "gamma", "]"}], " ", 
   RowBox[{"Sin", "[", "gamma", "]"}]}]}]], "Output",
 CellChangeTimes->{3.778174368477632*^9},
 CellLabel->
  "Out[362]=",ExpressionUUID->"ec8804dc-a0a2-416c-afe8-fd456e3edce3"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         SuperscriptBox["g2", "2"], "-", 
         RowBox[{"2", " ", "g1", " ", "g3"}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "7"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{{3.7781746383777943`*^9, 3.7781746600412397`*^9}},
 CellLabel->
  "In[364]:=",ExpressionUUID->"19b3e010-d600-40fa-b81b-e63390f88a46"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     SuperscriptBox["gamma", "4"], 
     RowBox[{"12", " ", 
      SuperscriptBox["beta", "2"]}]]}], "+", 
   FractionBox[
    RowBox[{"11", " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"360", " ", 
     SuperscriptBox["beta", "2"]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "8"],
    SeriesData[$CellContext`gamma, 0, {}, 4, 8, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[-1, 12] $CellContext`beta^(-2), 0, 
    Rational[11, 360] $CellContext`beta^(-2)}, 4, 8, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.778174660453926*^9},
 CellLabel->
  "Out[364]=",ExpressionUUID->"63d70d36-4482-49f1-abfb-5d7be8d33005"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"x0", " ", 
           RowBox[{"(", 
            RowBox[{"g1", "-", 
             FractionBox[
              RowBox[{"g2", " ", "c2"}], "r"], "+", 
             FractionBox[
              RowBox[{"h", " ", "c3", " ", "c2"}], 
              RowBox[{"r", " ", 
               SuperscriptBox["r0", "3"]}]], "-", 
             FractionBox[
              RowBox[{"h", " ", "c20"}], 
              RowBox[{"beta", " ", 
               SuperscriptBox["r0", "3"]}]]}], ")"}]}], "-", 
          RowBox[{"(", 
           RowBox[{"x0", 
            RowBox[{"(", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"beta", "*", "g1", "*", "g2"}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"eta", "*", "g2"}], "+", 
                    RowBox[{"k", "*", "g3"}]}], ")"}], "*", "eta", "*", "g0", 
                  "*", "c3"}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"r", "*", 
                  RowBox[{"r0", "^", "3"}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"g1", "*", "g0", "*", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"2", "k", "*", 
                    RowBox[{"eta", "^", "2"}], "*", "g2"}], "+", 
                    RowBox[{"3", "eta", "*", 
                    RowBox[{"k", "^", "2"}], "*", "g3"}]}], ")"}]}], ")"}], 
                "/", 
                RowBox[{"(", 
                 RowBox[{"beta", "*", "r", "*", 
                  RowBox[{"r0", "^", "2"}]}], ")"}]}], "-", 
               RowBox[{
                RowBox[{"k", "/", 
                 RowBox[{"(", 
                  RowBox[{"beta", "*", 
                   RowBox[{"r0", "^", "3"}]}], ")"}]}], "*", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"eta", "*", "g1", "*", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"eta", "*", "g2"}], "+", 
                    RowBox[{"k", "*", "g3"}]}], ")"}]}], "+", 
                  RowBox[{"g3", "*", "g0", "*", 
                   RowBox[{"r0", "^", "2"}], "*", "beta"}], "+", 
                  RowBox[{"2", "h", "*", "g2", "*", "k"}]}], ")"}]}], "+", 
               RowBox[{
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"g1", "*", "zeta"}], ")"}], "/", "r"}], "*", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"h", "*", "c3"}], ")"}], "/", 
                   RowBox[{"r0", "^", "3"}]}], "-", "g2"}], ")"}]}], "-", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"eta", "*", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"beta", "*", "g2", "*", "g0", "*", "r0"}], "+", 
                    RowBox[{"k", "*", 
                    RowBox[{"g1", "^", "2"}]}]}], ")"}], "*", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"eta", "*", "g1"}], "+", 
                    RowBox[{"k", "*", "g2"}]}], ")"}]}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"beta", "*", "r", "*", 
                  RowBox[{"r0", "^", "2"}]}], ")"}]}]}], ")"}], "\n", ")"}]}],
            ")"}]}], "/.", " ", 
         RowBox[{"zeta", " ", "\[Rule]", " ", 
          RowBox[{"k", "-", 
           RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
        RowBox[{"d", " ", "\[Rule]", " ", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"h", "+", 
            RowBox[{"eta", "*", "g2"}], "+", 
            RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}], 
       " ", "/.", " ", 
       RowBox[{"h", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
         RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
         RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
      RowBox[{"r", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
        RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
        RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
     RowBox[{"g0", " ", "\[Rule]", " ", 
      RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
    RowBox[{"g1", " ", "\[Rule]", " ", 
     RowBox[{
      RowBox[{"Sin", "[", "gamma", "]"}], "/", 
      RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
   RowBox[{"g2", " ", "\[Rule]", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ", "/.",
   " ", 
  RowBox[{"g3", " ", "\[Rule]", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"gamma", "-", 
       RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
    RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}]], "Input",
 CellChangeTimes->{{3.778187345355399*^9, 3.778187454104939*^9}, {
  3.778187492961492*^9, 3.778187515829214*^9}},
 CellLabel->
  "In[370]:=",ExpressionUUID->"bbd8dea2-530f-479f-a2ce-44fdb4e66d75"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"v0", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
                  FractionBox["c18", "beta"], "-", 
                  FractionBox[
                   RowBox[{"d", " ", "c2"}], "r"], "+", 
                  FractionBox[
                   RowBox[{"2", " ", "g2", " ", "h", " ", "c2"}], "r"], "-", 
                  FractionBox[
                   RowBox[{
                    SuperscriptBox["h", "2"], " ", "c3", " ", "c2"}], 
                   RowBox[{"r", " ", 
                    SuperscriptBox["r0", "3"]}]], "+", 
                  FractionBox[
                   RowBox[{
                    SuperscriptBox["h", "2"], " ", "c20"}], 
                   RowBox[{"beta", " ", 
                    SuperscriptBox["r0", "3"]}]]}], ")"}]}], "+", 
               RowBox[{"v0", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FractionBox["k", 
                    RowBox[{"beta", " ", "r"}]], 
                   RowBox[{"(", " ", 
                    RowBox[{
                    RowBox[{"eta", " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"beta", " ", "g2", " ", "g3"}], " ", "-", " ", 
                    "h8"}], ")"}]}], " ", "-", 
                    RowBox[{"h6", " ", "k"}], "+", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{
                    SuperscriptBox["g2", "2"], "-", 
                    RowBox[{"2", " ", "g1", " ", "g3"}]}], ")"}], " ", "beta",
                     " ", "r0"}]}], ")"}]}], "+", 
                  RowBox[{"h", 
                   RowBox[{"(", " ", 
                    RowBox[{"g1", " ", "-", 
                    FractionBox[
                    RowBox[{" ", 
                    RowBox[{"g2", "  ", "c2"}]}], "r"], "+", 
                    FractionBox[
                    RowBox[{"h", " ", "c3", " ", "c2"}], 
                    RowBox[{"r", " ", 
                    SuperscriptBox["r0", "3"]}]], "-", 
                    FractionBox[
                    RowBox[{"h", " ", "c20"}], 
                    RowBox[{"beta", " ", 
                    SuperscriptBox["r0", "3"]}]]}], ")"}]}]}], ")"}]}]}], 
              "\n", " ", "/.", " ", 
              RowBox[{"h8", " ", "\[Rule]", " ", 
               RowBox[{
                RowBox[{"g1", " ", "g2"}], " ", "-", " ", 
                RowBox[{"3", " ", "g0", " ", "g3"}]}]}]}], " ", "/.", " ", 
             RowBox[{"h6", " ", "\[Rule]", " ", 
              RowBox[{
               RowBox[{"2", 
                SuperscriptBox["g2", "2"]}], "-", 
               RowBox[{"3", " ", "g1", " ", "g3"}]}]}]}], "  ", "/.", " ", 
            RowBox[{"zeta", " ", "\[Rule]", " ", 
             RowBox[{"k", "-", 
              RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
           RowBox[{"d", " ", "\[Rule]", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"h", "+", 
               RowBox[{"eta", "*", "g2"}], "+", 
               RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}],
           " ", "/.", " ", 
          RowBox[{"h", " ", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
            RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
            RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
         RowBox[{"r", " ", "\[Rule]", " ", 
          RowBox[{
           RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
           RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
           RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "10"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{
  3.7781874867133503`*^9, 3.778187575725026*^9, {3.7781876145323677`*^9, 
   3.7781876162671347`*^9}, {3.778187669168495*^9, 3.778187718325417*^9}, {
   3.778187764368429*^9, 3.778187789477723*^9}, 3.7781879348923693`*^9, {
   3.778188163561355*^9, 3.778188204354539*^9}, {3.7781883024712133`*^9, 
   3.778188304948353*^9}},
 CellLabel->
  "In[389]:=",ExpressionUUID->"ff433890-6b35-45ec-8ad2-8e799bed76fc"],

Cell[BoxData[
 InterpretationBox[
  SuperscriptBox[
   RowBox[{"O", "[", "gamma", "]"}], "11"],
  SeriesData[$CellContext`gamma, 0, {}, 11, 11, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.778188181637824*^9, 3.77818820482655*^9}, 
   3.778188305789885*^9, 3.778188361686034*^9},
 CellLabel->
  "Out[389]=",ExpressionUUID->"845708b0-a3ca-4975-be82-52af42143ca1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Simplify", "[", "%", "]"}]], "Input",
 CellChangeTimes->{{3.778187705173954*^9, 3.778187706300379*^9}},
 CellLabel->
  "In[382]:=",ExpressionUUID->"fc106a9a-b7e2-464e-aee1-9e11a90c70c8"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "k"}], ")"}], " ", "k", " ", "v0"}], ")"}],
       " ", 
      SuperscriptBox["gamma", "4"]}], 
     RowBox[{"12", " ", 
      SuperscriptBox["beta", "2"]}]]}], "+", 
   FractionBox[
    RowBox[{"eta", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "k"}], ")"}], " ", "k", " ", "v0", " ", 
     SuperscriptBox["gamma", "5"]}], 
    RowBox[{"12", " ", 
     SuperscriptBox["beta", 
      RowBox[{"5", "/", "2"}]], " ", "r0"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "k"}], ")"}], " ", "k", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"15", " ", 
          SuperscriptBox["eta", "2"]}], "+", 
         RowBox[{"r0", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "3"}], " ", "k"}], "+", 
            RowBox[{"2", " ", "beta", " ", "r0"}]}], ")"}]}]}], ")"}], " ", 
       "v0"}], ")"}], " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"180", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["beta", "3"], " ", 
       SuperscriptBox["r0", "2"]}], ")"}]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "7"],
    SeriesData[$CellContext`gamma, 0, {}, 4, 7, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[-1, 
      12] $CellContext`beta^(-2) (-1 + $CellContext`k) $CellContext`k \
$CellContext`v0, 
    Rational[1, 12] $CellContext`beta^
     Rational[-5, 
       2] $CellContext`eta (-1 + $CellContext`k) $CellContext`k \
$CellContext`r0^(-1) $CellContext`v0, 
    Rational[-1, 
      180] $CellContext`beta^(-3) (-1 + $CellContext`k) $CellContext`k \
$CellContext`r0^(-2) (
     15 $CellContext`eta^2 + $CellContext`r0 ((-3) $CellContext`k + 
       2 $CellContext`beta $CellContext`r0)) $CellContext`v0}, 4, 7, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.7781877067869368`*^9, 3.778187722475874*^9}, {
   3.778187777332932*^9, 3.778187792853603*^9}, 3.778187938238348*^9},
 CellLabel->
  "Out[382]=",ExpressionUUID->"1c599b0d-1bff-45b8-94aa-cfd1a7dcce90"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"v0", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"-", "2"}], " ", "g1", " ", "h"}], "+", 
                 FractionBox["c18", "beta"], "-", 
                 FractionBox[
                  RowBox[{"d", " ", "c2"}], "r"], "+", 
                 FractionBox[
                  RowBox[{"2", " ", "g2", " ", "h", " ", "c2"}], "r"], "-", 
                 FractionBox[
                  RowBox[{
                   SuperscriptBox["h", "2"], " ", "c3", " ", "c2"}], 
                  RowBox[{"r", " ", 
                   SuperscriptBox["r0", "3"]}]], "+", 
                 FractionBox[
                  RowBox[{
                   SuperscriptBox["h", "2"], " ", "c20"}], 
                  RowBox[{"beta", " ", 
                   SuperscriptBox["r0", "3"]}]]}], ")"}]}], "/.", " ", 
              RowBox[{"h8", " ", "\[Rule]", " ", 
               RowBox[{
                RowBox[{"g1", " ", "g2"}], " ", "-", " ", 
                RowBox[{"3", " ", "g0", " ", "g3"}]}]}]}], " ", "/.", " ", 
             RowBox[{"h6", " ", "\[Rule]", " ", 
              RowBox[{
               RowBox[{"2", 
                SuperscriptBox["g2", "2"]}], "-", 
               RowBox[{"3", " ", "g1", " ", "g3"}]}]}]}], "  ", "/.", " ", 
            RowBox[{"zeta", " ", "\[Rule]", " ", 
             RowBox[{"k", "-", 
              RowBox[{"beta", " ", "r0"}]}]}]}], "/.", " ", 
           RowBox[{"d", " ", "\[Rule]", " ", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"h", "+", 
               RowBox[{"eta", "*", "g2"}], "+", 
               RowBox[{"2", "*", "k", "*", "g3"}]}], ")"}], "/", "beta"}]}]}],
           " ", "/.", " ", 
          RowBox[{"h", " ", "\[Rule]", " ", 
           RowBox[{
            RowBox[{"r0", " ", "g1"}], " ", "+", " ", 
            RowBox[{"eta", " ", "g2"}], " ", "+", " ", 
            RowBox[{"k", " ", "g3"}]}]}]}], "  ", "/.", " ", 
         RowBox[{"r", " ", "\[Rule]", " ", 
          RowBox[{
           RowBox[{"r0", " ", "g0"}], " ", "+", " ", 
           RowBox[{"eta", " ", "g1"}], " ", "+", " ", 
           RowBox[{"k", " ", "g2"}]}]}]}], " ", "/.", " ", 
        RowBox[{"g0", " ", "\[Rule]", " ", 
         RowBox[{"Cos", "[", "gamma", "]"}]}]}], " ", "/.", " ", 
       RowBox[{"g1", " ", "\[Rule]", " ", 
        RowBox[{
         RowBox[{"Sin", "[", "gamma", "]"}], "/", 
         RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], " ", "/.", " ", 
      RowBox[{"g2", " ", "\[Rule]", " ", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", 
          RowBox[{"Cos", "[", "gamma", "]"}]}], ")"}], "/", "beta"}]}]}], " ",
      "/.", " ", 
     RowBox[{"g3", " ", "\[Rule]", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"gamma", "-", 
          RowBox[{"Sin", "[", "gamma", "]"}]}], ")"}], "/", "beta"}], "/", 
       RowBox[{"Sqrt", "[", "beta", "]"}]}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"gamma", ",", "0", ",", "6"}], "}"}]}], "]"}], " "}]], "Input",
 CellChangeTimes->{3.778187973137973*^9},
 CellLabel->
  "In[383]:=",ExpressionUUID->"9fca56e5-7f5b-41b2-929b-9f7fc992ffd0"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   FractionBox[
    RowBox[{"3", " ", "k", " ", "v0", " ", 
     SuperscriptBox["gamma", "4"]}], 
    RowBox[{"4", " ", 
     SuperscriptBox["beta", "2"]}]], "+", 
   FractionBox[
    RowBox[{"9", " ", "eta", " ", "k", " ", "v0", " ", 
     SuperscriptBox["gamma", "5"]}], 
    RowBox[{"10", " ", 
     SuperscriptBox["beta", 
      RowBox[{"5", "/", "2"}]], " ", "r0"}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"171", " ", 
        SuperscriptBox["eta", "2"], " ", "k"}], "+", 
       RowBox[{"34", " ", 
        SuperscriptBox["k", "2"], " ", "r0"}], "-", 
       RowBox[{"54", " ", "beta", " ", "k", " ", 
        SuperscriptBox["r0", "2"]}]}], ")"}], " ", "v0", " ", 
     SuperscriptBox["gamma", "6"]}], 
    RowBox[{"360", " ", 
     SuperscriptBox["beta", "3"], " ", 
     SuperscriptBox["r0", "2"]}]], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "gamma", "]"}], "7"],
    SeriesData[$CellContext`gamma, 0, {}, 4, 7, 1],
    Editable->False]}],
  SeriesData[$CellContext`gamma, 0, {
   Rational[3, 4] $CellContext`beta^(-2) $CellContext`k $CellContext`v0, 
    Rational[9, 10] $CellContext`beta^
     Rational[-5, 
       2] $CellContext`eta $CellContext`k $CellContext`r0^(-1) \
$CellContext`v0, 
    Rational[1, 360] $CellContext`beta^(-3) $CellContext`r0^(-2) (
     171 $CellContext`eta^2 $CellContext`k + 
     34 $CellContext`k^2 $CellContext`r0 - 
     54 $CellContext`beta $CellContext`k $CellContext`r0^2) $CellContext`v0}, 
   4, 7, 1],
  Editable->False]], "Output",
 CellChangeTimes->{3.7781879737247334`*^9},
 CellLabel->
  "Out[383]=",ExpressionUUID->"1d05c08e-77c7-4768-a2cf-f0794a3a4d96"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   SuperscriptBox["g2", "2"]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"2", "+", "g0"}], ")"}], " ", "g1", " ", "g3"}], "-", 
  RowBox[{"beta", " ", "g1", " ", "g2", " ", "g3"}]}]], "Input",
 CellLabel->
  "In[384]:=",ExpressionUUID->"a7abe7e2-35f8-44e4-9dd3-7c378503bfca"],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   SuperscriptBox["g2", "2"]}], "-", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"2", "+", "g0"}], ")"}], " ", "g1", " ", "g3"}], "-", 
  RowBox[{"beta", " ", "g1", " ", "g2", " ", "g3"}]}]], "Output",
 CellChangeTimes->{3.778188051688497*^9},
 CellLabel->
  "Out[384]=",ExpressionUUID->"e879ca6c-1988-4bcd-8124-bf67fc4c043c"]
}, Open  ]]
},
WindowSize->{1143, 705},
WindowMargins->{{0, Automatic}, {Automatic, 11}},
FrontEndVersion->"12.0 for Mac OS X x86 (64-bit) (April 8, 2019)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 2647, 67, 241, "Input",ExpressionUUID->"53329b6c-a80c-45ac-ae62-f6fb243392bd"],
Cell[CellGroupData[{
Cell[3230, 91, 1300, 34, 72, "Input",ExpressionUUID->"e5f914b2-2bf0-439b-99e7-b7d2a93687e8"],
Cell[4533, 127, 1037, 30, 68, "Output",ExpressionUUID->"4955c78b-2203-47ea-9a21-650a64b5616b"]
}, Open  ]],
Cell[5585, 160, 287, 5, 30, "Input",ExpressionUUID->"3d3905a2-a8e5-4aff-90a6-57805de85496"],
Cell[CellGroupData[{
Cell[5897, 169, 1534, 45, 51, "Input",ExpressionUUID->"ec64950c-8ed3-4bc5-92a5-293069313a0b"],
Cell[7434, 216, 2212, 69, 113, "Output",ExpressionUUID->"fc05a247-3d57-4595-b5df-ae9e6d744f8c"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9683, 290, 402, 9, 30, "Input",ExpressionUUID->"c4b0219d-ef0f-4a85-945c-de2fde90772c"],
Cell[10088, 301, 2601, 82, 132, "Output",ExpressionUUID->"fc76868f-18c3-4155-85d7-ea4774492c50"]
}, Open  ]],
Cell[12704, 386, 267, 5, 30, "Input",ExpressionUUID->"47b9fc33-5d07-4434-a043-7f9a73672425"],
Cell[CellGroupData[{
Cell[12996, 395, 1233, 37, 92, "Input",ExpressionUUID->"2c452f06-9851-4112-9a70-63932153ef0e"],
Cell[14232, 434, 1182, 38, 60, "Output",ExpressionUUID->"4bb8a2af-287c-48fb-bb5b-881170e64fdc"]
}, Open  ]],
Cell[15429, 475, 268, 5, 30, "Input",ExpressionUUID->"c9e66de5-4f55-4c4c-9961-82f75df0e5e8"],
Cell[CellGroupData[{
Cell[15722, 484, 852, 26, 49, "Input",ExpressionUUID->"68f55207-a636-4c7c-a40a-237f2da6c8d5"],
Cell[16577, 512, 793, 26, 52, "Output",ExpressionUUID->"b0ac71be-dfd7-4696-8b66-e735fc2bbc66"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17407, 543, 6016, 174, 212, "Input",ExpressionUUID->"04b71007-a2aa-46f4-a938-b8fcee4ceb50"],
Cell[23426, 719, 4701, 149, 191, "Output",ExpressionUUID->"6f0e28ed-9bc4-4425-bc09-07b4d1c655e1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[28164, 873, 308, 7, 30, "Input",ExpressionUUID->"fec8e82c-efd6-4867-a823-49d5929c062a"],
Cell[28475, 882, 6013, 186, 249, "Output",ExpressionUUID->"f214e0f6-85fc-456d-bba1-9cdbe3c090bf"]
}, Open  ]],
Cell[34503, 1071, 324, 8, 30, "Input",ExpressionUUID->"9fa6d673-9623-427b-9407-78624850703f"],
Cell[CellGroupData[{
Cell[34852, 1083, 2736, 79, 103, "Input",ExpressionUUID->"c9d2383d-a036-4f78-b220-89a9454bf72a"],
Cell[37591, 1164, 3003, 90, 110, "Output",ExpressionUUID->"ff30ac28-e574-47ce-8999-03449efe1c2d"]
}, Open  ]],
Cell[40609, 1257, 322, 8, 30, "Input",ExpressionUUID->"23904c48-5c7f-4f38-a10d-1dffce1fbd7f"],
Cell[CellGroupData[{
Cell[40956, 1269, 1678, 49, 56, "Input",ExpressionUUID->"66f595b3-9a00-4612-95df-a4919c2ed717"],
Cell[42637, 1320, 2179, 66, 110, "Output",ExpressionUUID->"e1020760-5a71-4965-84e5-11e7c2c10310"]
}, Open  ]],
Cell[44831, 1389, 316, 8, 30, "Input",ExpressionUUID->"f7d26cbe-f478-4850-8b72-772aeff8061f"],
Cell[CellGroupData[{
Cell[45172, 1401, 7811, 223, 347, "Input",ExpressionUUID->"f882e2b1-47a9-4619-b178-8b87b1331dda"],
Cell[52986, 1626, 7257, 221, 301, "Output",ExpressionUUID->"bd780f74-c409-447b-b977-8bccc8641338"]
}, Open  ]],
Cell[CellGroupData[{
Cell[60280, 1852, 8325, 230, 396, "Input",ExpressionUUID->"5ddd5a2f-d46b-484e-813e-96d850cb906d"],
Cell[68608, 2084, 7015, 214, 336, "Output",ExpressionUUID->"b933522a-6e79-447f-b846-a9ae7d2c4941"]
}, Open  ]],
Cell[CellGroupData[{
Cell[75660, 2303, 207, 3, 30, "Input",ExpressionUUID->"a7ffe149-f88e-46ce-9bb0-27b379ce4166"],
Cell[75870, 2308, 12984, 395, 541, "Output",ExpressionUUID->"d9d42105-a8fa-4468-849d-15fd8a5cf370"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88891, 2708, 208, 3, 30, "Input",ExpressionUUID->"b266fc52-da22-4737-a7d7-167082344612"],
Cell[89102, 2713, 148, 2, 34, "Output",ExpressionUUID->"4feff2ca-31fe-43e4-b153-429a09fb5e65"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89287, 2720, 1250, 34, 178, "Input",ExpressionUUID->"7b09522d-a9ad-429b-8e6a-62696f3cc0de"],
Cell[90540, 2756, 359, 6, 34, "Output",ExpressionUUID->"f1b9f8da-fce5-496f-9171-45625c4e27fa"],
Cell[90902, 2764, 387, 7, 34, "Output",ExpressionUUID->"a8308bde-92c6-48d1-ad6f-06ad58a58d3f"],
Cell[91292, 2773, 406, 8, 34, "Output",ExpressionUUID->"9b3f0f51-4daf-415b-9a48-da8cc060944f"],
Cell[91701, 2783, 395, 7, 34, "Output",ExpressionUUID->"262cbdb9-d94f-4fef-bc3f-e98a39df3a3c"],
Cell[92099, 2792, 394, 7, 34, "Output",ExpressionUUID->"e604a93d-e38e-4d40-95a7-1e76eead12ad"],
Cell[92496, 2801, 471, 10, 34, "Output",ExpressionUUID->"6b52690d-0193-4ecf-a077-f0651ee862ec"]
}, Open  ]],
Cell[92982, 2814, 440, 9, 30, "Input",ExpressionUUID->"5ca034f5-8343-48cc-8f0a-07db0c3ba317"],
Cell[CellGroupData[{
Cell[93447, 2827, 3821, 108, 140, "Input",ExpressionUUID->"749d2d9f-b759-4bbd-8637-75a3f6857347"],
Cell[97271, 2937, 6998, 215, 301, "Output",ExpressionUUID->"4e20d382-8e52-4b4d-8ebe-14a672d67e1e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[104306, 3157, 305, 5, 30, "Input",ExpressionUUID->"85ecd29e-7dc6-4a46-bb6b-50623f9d5fc4"],
Cell[104614, 3164, 2986, 97, 141, "Output",ExpressionUUID->"af1653e1-804c-496b-ba53-04fe5b8489c0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[107637, 3266, 208, 3, 30, "Input",ExpressionUUID->"da464f5d-15d8-455a-967b-5f814bfda060"],
Cell[107848, 3271, 170, 2, 34, "Output",ExpressionUUID->"7ad9fa14-b279-4a14-8f34-51f79d68ba82"]
}, Open  ]],
Cell[108033, 3276, 372, 8, 30, "Input",ExpressionUUID->"cb695c37-a5f2-4ae9-9f5d-c447b915243f"],
Cell[CellGroupData[{
Cell[108430, 3288, 1553, 44, 77, "Input",ExpressionUUID->"7b93e751-e210-4f4f-82cf-2aee0376560b"],
Cell[109986, 3334, 2938, 95, 106, "Output",ExpressionUUID->"cc39e4d9-bdc2-415c-bbc2-0921a86b7f30"]
}, Open  ]],
Cell[112939, 3432, 1608, 46, 54, "Input",ExpressionUUID->"67d76b09-86e0-4efb-a78b-a12cdacdc517"],
Cell[CellGroupData[{
Cell[114572, 3482, 1164, 33, 33, "Input",ExpressionUUID->"0ea04d4e-d261-404e-b23c-65483ecdafc0"],
Cell[115739, 3517, 1270, 41, 37, "Output",ExpressionUUID->"19c3bdb2-1c2c-4c4f-8ad1-44660fd17e7b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[117046, 3563, 210, 4, 30, "Input",ExpressionUUID->"80a0bb6f-4739-40d4-8cec-775969014b4a"],
Cell[117259, 3569, 1393, 37, 56, "Output",ExpressionUUID->"fa82927d-9c72-48af-9f2d-0c39810cfd90"]
}, Open  ]],
Cell[118667, 3609, 293, 7, 30, "Input",ExpressionUUID->"d190f851-72af-4d84-9145-dcfafee72abd"],
Cell[118963, 3618, 1127, 35, 33, "Input",ExpressionUUID->"f885f30a-c63a-4759-8152-17ba648fe1c9"],
Cell[CellGroupData[{
Cell[120115, 3657, 1842, 55, 96, "Input",ExpressionUUID->"4c41ccac-c0ea-4308-ac55-1dec6c9c77a1"],
Cell[121960, 3714, 2017, 62, 140, "Output",ExpressionUUID->"79ed7c08-1c23-482f-9b6c-2bb474ef1762"]
}, Open  ]],
Cell[CellGroupData[{
Cell[124014, 3781, 4071, 110, 167, "Input",ExpressionUUID->"76d5ec85-e7d3-4444-86cf-b3b5c22db434"],
Cell[128088, 3893, 7040, 229, 244, "Output",ExpressionUUID->"1786ef8d-ebd6-4c93-9e24-fc9852f9ac2e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[135165, 4127, 209, 4, 30, "Input",ExpressionUUID->"c58a2338-2944-49e5-9a0d-fe159321e125"],
Cell[135377, 4133, 1218, 39, 59, "Output",ExpressionUUID->"1d7ab61c-c3d4-4663-89e0-05d19e0aaddc"]
}, Open  ]],
Cell[136610, 4175, 336, 8, 30, "Input",ExpressionUUID->"346d9a50-3203-4a55-bec9-c10ff1e4e411"],
Cell[136949, 4185, 398, 10, 30, "Input",ExpressionUUID->"9bea5a63-e6fd-4f17-8cb6-b09087afe8f5"],
Cell[137350, 4197, 525, 16, 50, "Input",ExpressionUUID->"2786cd89-4914-46b4-baae-d874c1c8d48f"],
Cell[CellGroupData[{
Cell[137900, 4217, 1080, 30, 30, "Input",ExpressionUUID->"0aa8508b-6614-4f8c-bd79-45bb90af864c"],
Cell[138983, 4249, 1353, 44, 52, "Output",ExpressionUUID->"f17344e3-2c78-4e1c-a694-995f7d92755e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[140373, 4298, 1287, 40, 33, "Input",ExpressionUUID->"3681943e-f99b-4f72-855c-3abb28b3a865"],
Cell[141663, 4340, 1260, 40, 37, "Output",ExpressionUUID->"d0941fe1-b499-432e-8bcb-1abf0179423f"]
}, Open  ]],
Cell[CellGroupData[{
Cell[142960, 4385, 210, 4, 30, "Input",ExpressionUUID->"fc972c3a-4923-43dd-9cba-62e600961b00"],
Cell[143173, 4391, 1413, 38, 56, "Output",ExpressionUUID->"665635f9-57a1-4bbe-9e66-f44dd8a6b0e7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[144623, 4434, 212, 4, 30, "Input",ExpressionUUID->"e17b271c-d958-4592-b36a-31565452ebd2"],
Cell[144838, 4440, 1432, 43, 58, "Output",ExpressionUUID->"615dbd83-84ce-4244-bb96-782cfadc0d23"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146307, 4488, 1648, 47, 56, "Input",ExpressionUUID->"ce792024-a8f3-4b39-bfc2-c2622c5dc987"],
Cell[147958, 4537, 1623, 53, 60, "Output",ExpressionUUID->"ed4cd0bc-3a06-4e07-a0fb-eaca39e0af4a"]
}, Open  ]],
Cell[149596, 4593, 1126, 34, 33, "Input",ExpressionUUID->"211f4346-5eec-4e40-85a0-57a5260796e1"],
Cell[CellGroupData[{
Cell[150747, 4631, 650, 16, 30, "Input",ExpressionUUID->"d8a22895-ddf5-442c-b4c4-275c57c4ab99"],
Cell[151400, 4649, 627, 18, 34, "Output",ExpressionUUID->"175381db-2169-4bd1-ba37-100030d6f3b3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[152064, 4672, 290, 6, 30, "Input",ExpressionUUID->"a498c6a4-63d8-43f1-aca6-58b39dcce549"],
Cell[152357, 4680, 445, 14, 37, "Output",ExpressionUUID->"104f5b5a-05a9-48f6-88e4-8a2c5aeac77f"]
}, Open  ]],
Cell[152817, 4697, 228, 6, 30, "Input",ExpressionUUID->"8f680817-27a8-4159-97f3-a4b6451a334c"],
Cell[153048, 4705, 175, 3, 30, "Input",ExpressionUUID->"feb7a859-b25a-42d9-8b72-7533fad5596e"],
Cell[CellGroupData[{
Cell[153248, 4712, 602, 15, 30, "Input",ExpressionUUID->"4acf9634-22ef-48d4-ac60-9d9d952b54cd"],
Cell[153853, 4729, 555, 17, 34, "Output",ExpressionUUID->"8c68fcb8-c605-44b8-b206-8a0039276741"]
}, Open  ]],
Cell[154423, 4749, 290, 6, 30, "Input",ExpressionUUID->"c5c8c83a-5af2-4cd9-8b52-76cdadcb59fc"],
Cell[154716, 4757, 200, 5, 30, "Input",ExpressionUUID->"cef57c66-7f51-4aff-a8b7-46248b4923d4"],
Cell[154919, 4764, 324, 6, 30, "Input",ExpressionUUID->"9b8320e8-f2c4-446c-b832-4d14c4974f2c"],
Cell[CellGroupData[{
Cell[155268, 4774, 3280, 83, 133, "Input",ExpressionUUID->"86ce98b7-7406-4b04-b880-bed88514cd67"],
Cell[158551, 4859, 5929, 145, 206, "Output",ExpressionUUID->"7cf803f0-1bd6-464d-a759-dc2fa9b60088"]
}, Open  ]],
Cell[164495, 5007, 485, 12, 52, "Input",ExpressionUUID->"38e88ef0-d371-4d60-9f1a-89558da3eaa5"],
Cell[CellGroupData[{
Cell[165005, 5023, 2659, 68, 130, "Input",ExpressionUUID->"71147eb9-1811-43ab-9c97-da5dc2252d01"],
Cell[167667, 5093, 2755, 75, 62, "Output",ExpressionUUID->"07e48ec0-c825-4123-8d3f-ddf11ce3435a"]
}, Open  ]],
Cell[CellGroupData[{
Cell[170459, 5173, 1453, 39, 72, "Input",ExpressionUUID->"4110c55b-bdae-4210-8ad1-f6e1d5d95dff"],
Cell[171915, 5214, 846, 25, 53, "Output",ExpressionUUID->"808646b7-8e16-4ea6-aa1e-9f152e7a4e05"]
}, Open  ]],
Cell[CellGroupData[{
Cell[172798, 5244, 208, 3, 30, "Input",ExpressionUUID->"2d608bd3-89ba-4f2d-89ef-df899f9a030d"],
Cell[173009, 5249, 1136, 28, 55, "Output",ExpressionUUID->"e6186320-58cb-442f-891b-c65ccf64b3cc"]
}, Open  ]],
Cell[174160, 5280, 1232, 29, 56, "Input",ExpressionUUID->"a83389ef-650e-4248-b84d-87ed17b8d491"],
Cell[175395, 5311, 1050, 32, 33, "Input",ExpressionUUID->"8225ac50-168c-4cab-b13f-0ce22d867e34"],
Cell[176448, 5345, 628, 19, 52, "Input",ExpressionUUID->"38ebeebc-98f9-4b22-aa59-3a8206aba9b3"],
Cell[CellGroupData[{
Cell[177101, 5368, 3098, 78, 133, "Input",ExpressionUUID->"19429d45-8e46-42c8-9bd7-ddab5ab6732b"],
Cell[180202, 5448, 2727, 74, 62, "Output",ExpressionUUID->"c002ef47-5c2d-4cc5-8939-daff0aa64451"]
}, Open  ]],
Cell[182944, 5525, 277, 5, 30, "Input",ExpressionUUID->"5ef14266-0107-46d9-8ef5-871eba81c558"],
Cell[183224, 5532, 475, 7, 30, "Input",ExpressionUUID->"cc1b2afa-2caf-43db-bbd4-8dab27b3d74d"],
Cell[183702, 5541, 256, 5, 30, "Input",ExpressionUUID->"554b7b6a-85ab-4746-acc6-5ff1d4f2e304"],
Cell[CellGroupData[{
Cell[183983, 5550, 5294, 125, 188, "Input",ExpressionUUID->"ce5dfe0a-4173-4e35-a882-4c0137d14c26"],
Cell[189280, 5677, 3444, 85, 62, "Output",ExpressionUUID->"d92883a1-814f-41a1-bbc1-99b0b7d461e0"]
}, Open  ]],
Cell[192739, 5765, 313, 7, 30, "Input",ExpressionUUID->"09f4ea65-efdd-43db-8197-fc1a3f2aa7ff"],
Cell[CellGroupData[{
Cell[193077, 5776, 2912, 75, 133, "Input",ExpressionUUID->"af329d1b-7fe8-45ec-9e77-0081c2432267"],
Cell[195992, 5853, 2567, 66, 104, "Output",ExpressionUUID->"dc766367-d0b9-4d04-9d9d-4b309ba73938"]
}, Open  ]],
Cell[198574, 5922, 152, 3, 30, "Input",ExpressionUUID->"137ae34a-5865-42be-addb-dc0a7bc2dbb3"],
Cell[CellGroupData[{
Cell[198751, 5929, 4140, 101, 151, "Input",ExpressionUUID->"5b44f9ae-e77f-418d-88ab-e085887ab019"],
Cell[202894, 6032, 3308, 86, 115, "Output",ExpressionUUID->"49023a46-ad24-4ea0-b51d-abc0ffd6ecab"]
}, Open  ]],
Cell[206217, 6121, 128, 3, 30, "Input",ExpressionUUID->"55a33ee1-48c5-4665-9fb5-f52a9bd700a2"],
Cell[CellGroupData[{
Cell[206370, 6128, 1587, 43, 52, "Input",ExpressionUUID->"3d85475b-7977-4c44-a55c-59d19fe2374d"],
Cell[207960, 6173, 1277, 40, 56, "Output",ExpressionUUID->"b5e6b6f4-309b-42a9-a8a2-539f8c0e0b4a"]
}, Open  ]],
Cell[CellGroupData[{
Cell[209274, 6218, 294, 7, 30, "Input",ExpressionUUID->"e8c5d08b-7695-4e95-9f78-c15d7daf90eb"],
Cell[209571, 6227, 1973, 58, 60, "Output",ExpressionUUID->"a3a6b14d-2fa5-4b24-bdad-9738fcb00af1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[211581, 6290, 210, 4, 30, "Input",ExpressionUUID->"b5697605-47e3-4916-9031-ba16eb752f85"],
Cell[211794, 6296, 1977, 65, 60, "Output",ExpressionUUID->"3e370009-4be3-487a-86c9-0c62342151fd"]
}, Open  ]],
Cell[213786, 6364, 936, 25, 33, "Input",ExpressionUUID->"8abd4998-473f-4657-9511-1ecdf34a1530"],
Cell[CellGroupData[{
Cell[214747, 6393, 1300, 35, 52, "Input",ExpressionUUID->"be6e0e37-6867-47cb-8c7a-c33748ab83a1"],
Cell[216050, 6430, 872, 26, 55, "Output",ExpressionUUID->"c32a17fb-f279-4293-a542-bd3cef76b1a8"]
}, Open  ]],
Cell[CellGroupData[{
Cell[216959, 6461, 861, 22, 30, "Input",ExpressionUUID->"b8a76998-9b7e-4fdc-9683-15bf6d761421"],
Cell[217823, 6485, 559, 15, 34, "Output",ExpressionUUID->"f1ff3851-6a2d-4204-8084-f43d089f993b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[218419, 6505, 212, 4, 30, "Input",ExpressionUUID->"9ffc0832-f459-4305-b499-f4dab6244bca"],
Cell[218634, 6511, 494, 14, 50, "Output",ExpressionUUID->"ad5b3229-028f-4c15-8e5b-3300319debce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[219165, 6530, 469, 12, 30, "Input",ExpressionUUID->"bd54d80c-f805-4f3a-bf44-977406504414"],
Cell[219637, 6544, 422, 11, 34, "Output",ExpressionUUID->"ec8804dc-a0a2-416c-afe8-fd456e3edce3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[220096, 6560, 1242, 34, 56, "Input",ExpressionUUID->"19b3e010-d600-40fa-b81b-e63390f88a46"],
Cell[221341, 6596, 793, 24, 55, "Output",ExpressionUUID->"63d70d36-4482-49f1-abfb-5d7be8d33005"]
}, Open  ]],
Cell[222149, 6623, 5329, 136, 158, "Input",ExpressionUUID->"bbd8dea2-530f-479f-a2ce-44fdb4e66d75"],
Cell[CellGroupData[{
Cell[227503, 6763, 5241, 127, 198, "Input",ExpressionUUID->"ff433890-6b35-45ec-8ad2-8e799bed76fc"],
Cell[232747, 6892, 374, 9, 68, "Output",ExpressionUUID->"845708b0-a3ca-4975-be82-52af42143ca1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[233158, 6906, 210, 4, 30, "Input",ExpressionUUID->"fc106a9a-b7e2-464e-aee1-9e11a90c70c8"],
Cell[233371, 6912, 2361, 70, 62, "Output",ExpressionUUID->"1c599b0d-1bff-45b8-94aa-cfd1a7dcce90"]
}, Open  ]],
Cell[CellGroupData[{
Cell[235769, 6987, 3413, 86, 154, "Input",ExpressionUUID->"9fca56e5-7f5b-41b2-929b-9f7fc992ffd0"],
Cell[239185, 7075, 1728, 47, 57, "Output",ExpressionUUID->"1d05c08e-77c7-4768-a2cf-f0794a3a4d96"]
}, Open  ]],
Cell[CellGroupData[{
Cell[240950, 7127, 325, 9, 30, "Input",ExpressionUUID->"a7abe7e2-35f8-44e4-9dd3-7c378503bfca"],
Cell[241278, 7138, 368, 10, 34, "Output",ExpressionUUID->"e879ca6c-1988-4bcd-8124-bf67fc4c043c"]
}, Open  ]]
}
]
*)

